@@ -5,8 +5,8 @@
 #include "../h/gsupport.h"
 
 #ifdef TypTrc
-int typealloc = 0;		/* type allocation switch */
-long typespace = 0;		/* type allocation amount */
+   int typealloc = 0;		/* type allocation switch */
+   long typespace = 0;		/* type allocation amount */
 #endif					/* TypTrc */
 
 /*
@@ -21,7 +21,7 @@ char *s;
    s1 = (char *)malloc((msize)(strlen(s) + 1));
    if (s1 == NULL) {
       fprintf(stderr, "salloc(%d): out of memory\n",(int)strlen(s) + 1);
-      exit(ErrorExit);
+      exit(EXIT_FAILURE);
       }
    return strcpy(s1, s);
    }
@@ -59,7 +59,7 @@ unsigned int n;
    a = calloc((msize)n,sizeof(char));
    if (a == NULL) {
       fprintf(stderr, "alloc(%d): out of memory\n", (int)n);
-      exit(ErrorExit);
+      exit(EXIT_FAILURE);
       }
    return a;
    }
