@@ -1,11 +1,10 @@
 /*
  * This file contains pathfind(), fparse(), makename(), and smatch().
  */
-#include <ctype.h>
 #include "../h/gsupport.h"
 
-hidden char *pathelem	Params((char *s, char *buf));
-hidden char *tryfile	Params((char *buf, char *dir, char *name, char *extn));
+static char *pathelem	(char *s, char *buf);
+static char *tryfile	(char *buf, char *dir, char *name, char *extn);
 
 /*
  * The following code is operating-system dependent [@filepart.01].
@@ -18,65 +17,65 @@ hidden char *tryfile	Params((char *buf, char *dir, char *name, char *extn));
  */
 
 #if PORT
-#define Prefix "/"
-#define FileSep '/'
-Deliberate Syntax Error
+   #define Prefix "/"
+   #define FileSep '/'
+   Deliberate Syntax Error
 #endif					/* PORT */
 
 #if AMIGA
-#define Prefix "/:"
-#define FileSep '/'
+   #define Prefix "/:"
+   #define FileSep '/'
 #endif					/* AMIGA */
 
 #if ARM
-#define Prefix ".:"
-#define DefPath "Icon: Lib:Icon."
+   #define Prefix ".:"
+   #define DefPath "Icon: Lib:Icon."
 #endif					/* ARM */
 
 #if ATARI_ST
-#define Prefix "/:\\"
+   #define Prefix "/:\\"
 #endif					/* ATARI_ST */
 
 #if MSDOS || OS2
-#define Prefix "/:\\"
-#define DefPath ";"
-#if HIGHC_386
-#define FileSep '\\'
-#else					/* HIGHC_386 */
-#define FileSep '/'
-#endif					/* HIGHC_386 */
+   #define Prefix "/:\\"
+   #define DefPath ";"
+   #if HIGHC_386
+      #define FileSep '\\'
+   #else				/* HIGHC_386 */
+      #define FileSep '/'
+   #endif				/* HIGHC_386 */
 #endif					/* MSDOS || OS2 */
 
 #if MACINTOSH
-#define Prefix ":"
-#define FileSep ':'
-#if MPW || LWC
-#define DefPath ":"
-#endif					/* MPW || LSC */
-#if MPW
-#define PathSep ','
-#endif					/* MPW */
+   #define Prefix ":"
+   #define FileSep ':'
+   #if MPW || LWC
+      #define DefPath ":"
+   #endif				/* MPW || LSC */
+   #if MPW
+      #define PathSep ','
+   #endif				/* MPW */
 #endif					/* MACINTOSH */
 
 #if MVS
-#define Prefix ""
-#define FileSep '('
-#if SASC
-#define DefPath "iconlib ddn:::lib"
-#endif					/* SASC */
+   #define Prefix ""
+   #define FileSep '('
+   #if SASC
+      #define DefPath "iconlib ddn:::lib"
+   #endif				/* SASC */
 #endif					/* MVS */
 
 #if VM
-#define Prefix ""
+   #define Prefix ""
 #endif					/* VM */
 
 #if UNIX
-#define Prefix "/"
-#define FileSep '/'
+   #define Prefix "/"
+   #define FileSep '/'
 #endif					/* UNIX */
 
 #if VMS
-#define Prefix "]:"
+   #define Prefix "]:"
 #endif					/* VMS */
 
 /*
@@ -84,11 +83,11 @@ Deliberate Syntax Error
  */
 
 #ifndef DefPath
-#define DefPath ""
+   #define DefPath ""
 #endif					/* DefPath */
 
 #ifndef PathSep
-#define PathSep ' '
+   #define PathSep ' '
 #endif					/* PathSep */
 
 /*
@@ -116,13 +115,7 @@ char *buf, *path, *name, *extn;
       path = DefPath;
    s = path;
 
-#if SCCX_MX
-   /* Avoids compiler warning */
-   while( (s = pathelem(s, pbuf)) != 0)    /* for each path element */
-#else
-   while (s = pathelem(s, pbuf))	/* for each path element */
-#endif					/* SCCX_MX */
-
+   while ((s = pathelem(s, pbuf)) != 0)		/* for each path element */
       if (tryfile(buf, pbuf, name, extn))	/* look for file */
          return buf;
    return NULL;				/* return NULL if no file found */
@@ -152,7 +145,7 @@ char *s, *buf;
     * We have to append a path separator here.
     *  Seems like makename should really be the one to do that.
     */
-   if (!index(Prefix, buf[-1])) {	/* if separator not already there */
+   if (!strchr(Prefix, buf[-1])) {	/* if separator not already there */
       *buf++ = FileSep;
       }
 #endif					/* FileSep */
@@ -203,7 +196,7 @@ char *s;
    char *bp = buf;
 
    /* First, skip any filing system prefix */
-   s1 = index(s,':');
+   s1 = strchr(s,':');
    if (s1 == NULL)
       s1 = s;
    else
@@ -319,7 +312,7 @@ char *s;
 #if MVS
    static char extbuf [MaxFileName+2] ;
 
-   p = index(s, '(');
+   p = strchr(s, '(');
    if (p) {
       fp.member = p+1;
       memcpy(extbuf, s, p-s);
@@ -334,7 +327,7 @@ char *s;
    while (--p >= s) {
       if (*p == '.' && *fp.ext == '\0')
          fp.ext = p;
-      else if (index(Prefix,*p)) {
+      else if (strchr(Prefix,*p)) {
          q = p+1;
          break;
          }
@@ -351,7 +344,7 @@ char *s;
 
 #if VMS
    /* if a version is included, get separate extension and version */
-   if (p = index(fp.ext, ';')) {
+   if (p = strchr(fp.ext, ';')) {
       fp.version = p;
       p = fp.ext;
       fp.ext = fp.name + n + 1;
@@ -431,3 +424,101 @@ char *s, *t;
          return 0;
       }
    }
+
+
+#if MSDOS
+#if NT
+#include <sys/stat.h>
+#include <direct.h>
+#endif					/* NT */
+
+/*
+ * this version of pathfind, unlike the one above, is looking on
+ * the real path to find an executable.
+ */
+int pathFind(char target[], char buf[], int n)
+   {
+   char *path;
+   register int i;
+   int res;
+   struct stat sbuf;
+
+   if ((path = getenv("PATH")) == 0)
+      path = "";
+
+   if (!getcwd(buf, n)) {		/* get current working directory */
+      *buf = 0;		/* may be better to do something nicer if we can't */
+      return 0;		/* find out where we are -- struggling to achieve */
+      }			/* something can be better than not trying */
+
+   /* attempt to find the icode file in the current directory first */
+   /* this mimicks the behavior of COMMAND.COM */
+   if ((i = strlen(buf)) > 0) {
+      i = buf[i - 1];
+      if (i != '\\' && i != '/' && i != ':')
+         strcat(buf, "/");
+      }
+   strcat(buf, target);
+   res = stat(buf, &sbuf);
+
+   while(res && *path) {
+      for (i = 0; *path && *path != ';'; ++i)
+         buf[i] = *path++;
+      if (*path)			/* skip the ; or : separator */
+         ++path;
+      if (i == 0)			/* skip empty fragments in PATH */
+         continue;
+      if (i > 0 && buf[i - 1] != '/' && buf[i - 1] != '\\' &&
+         buf[i - 1] != ':')
+            buf[i++] = '/';
+      strcpy(buf + i, target);
+      res = stat(buf, &sbuf);
+      /* exclude directories (and any other nasties) from selection */
+      if (res == 0 && sbuf.st_mode & S_IFDIR)
+         res = -1;
+      }
+   if (res != 0)
+      *buf = 0;
+   return res == 0;
+   }
+#endif					/* MSDOS */
+
+
+
+#if MSDOS || OS2
+FILE *pathOpen(fname, mode)
+   char *fname;
+   char *mode;
+   {
+#if OS2
+   char buf[260 + 1];
+#else					/* OS2 */
+   char buf[150 + 1];
+#endif					/* OS2 */
+   int i, use = 1;
+
+#if SCCX_MX
+   /* Avoid compiler warning */
+   for( i = 0; (buf[i] = fname[i]) != 0; ++i)
+#else
+   for( i = 0; buf[i] = fname[i]; ++i)
+#endif					/* SCCX_MX */
+
+      /* find out if a path has been given in the file name */
+      if (buf[i] == '/' || buf[i] == ':' || buf[i] == '\\')
+         use = 0;
+
+   /* If a path has been given with the file name, don't bother to
+      use the PATH */
+
+#if OS2
+   if (use && DosSearchPath(SEARCH_CUR_DIRECTORY | SEARCH_ENVIRONMENT, 
+                            "PATH", fname, buf, 260))
+#else					/* OS2 */
+   if (use && !pathFind(fname, buf, 150))
+#endif 					/* OS2 */
+       return 0;
+
+   return fopen(buf, mode);
+   }
+#endif					/* MSDOS || OS2 */
