@@ -22,12 +22,8 @@
  *
  *  See ../h/features.h for the set of predefined symbols.
  */
-
+ 
 #include "../h/gsupport.h"
-#ifdef THINK_C
-#include <stdlib.h>
-#endif					/* THINK_C */
-#include <ctype.h>
 
 #define HTBINS 256			/* number of hash bins */
 
@@ -61,30 +57,30 @@ typedef struct cd {			/* structure holding a definition */
    char s[1];				/* name then value, as needed, no \0 */
    } cdefn;
 
-hidden	int	ppopen	Params((char *fname, int m4));
-hidden	FILE *	m4pipe	Params((char *fname));
-hidden	char *	rline	Params((FILE *fp));
-hidden	novalue	pushdef	Params((cdefn *d));
-hidden	novalue	pushline Params((char *fname, long lno));
-hidden	novalue	ppdir	Params((char *line));
-hidden	novalue	pfatal	Params((char *s1, char *s2));
-hidden	novalue	skipcode Params((int doelse, int report));
-hidden	char *	define	Params((char *s));
-hidden	char *	undef	Params((char *s));
-hidden	char *	ifdef	Params((char *s));
-hidden	char *	ifndef	Params((char *s));
-hidden	char *	ifxdef	Params((char *s, int f));
-hidden	char *	elsedir	Params((char *s));
-hidden	char *	endif	Params((char *s));
-hidden	char *	errdir	Params((char *s));
-hidden	char *	include	Params((char *s));
-hidden	char *	setline	Params((char *s));
-hidden	char *	wskip	Params((char *s));
-hidden	char *	nskip	Params((char *s));
-hidden	char *	matchq	Params((char *s));
-hidden	char *	getidt	Params((char *dst, char *src));
-hidden	char *	getfnm	Params((char *dst, char *src));
-hidden	cdefn *	dlookup	Params((char *name, int len, char *val));
+static	int	ppopen	(char *fname, int m4);
+static	FILE *	m4pipe	(char *fname);
+static	char *	rline	(FILE *fp);
+static	void	pushdef	(cdefn *d);
+static	void	pushline (char *fname, long lno);
+static	void	ppdir	(char *line);
+static	void	pfatal	(char *s1, char *s2);
+static	void	skipcode (int doelse, int report);
+static	char *	define	(char *s);
+static	char *	undef	(char *s);
+static	char *	ifdef	(char *s);
+static	char *	ifndef	(char *s);
+static	char *	ifxdef	(char *s, int f);
+static	char *	elsedir	(char *s);
+static	char *	endif	(char *s);
+static	char *	errdir	(char *s);
+static	char *	include	(char *s);
+static	char *	setline	(char *s);
+static	char *	wskip	(char *s);
+static	char *	nskip	(char *s);
+static	char *	matchq	(char *s);
+static	char *	getidt	(char *dst, char *src);
+static	char *	getfnm	(char *dst, char *src);
+static	cdefn *	dlookup	(char *name, int len, char *val);
 
 struct ppcmd {
    char *name;
@@ -153,10 +149,7 @@ int m4;
    /*
     * initialize variables and open source file 
     */
-#ifdef EnvVars
    lpath = getenv("LPATH");
-#endif					/* EnvVars */
-
    curfile = &nofile;			/* init file struct pointer */
    return ppopen(fname, m4);		/* open main source file */
    }
@@ -209,13 +202,6 @@ int m4;
 static FILE *m4pipe(filename)
 char *filename;
    {
-
-#if MACINTOSH
-#if MPW
-/* #pragma unused(filename) */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
-
 #if ARM || UNIX
       {
       FILE *f;
@@ -237,7 +223,7 @@ char *filename;
  *  Otherwise, defines s to have the value v.
  *  No error is given for a redefinition.
  */
-novalue ppdef(s, v)
+void ppdef(s, v)
 char *s, *v;
    {
    dlookup(s, -1, (char *)NULL);
@@ -248,7 +234,7 @@ char *s, *v;
 /*
  * ppecho() -- run input through preprocessor and echo directly to stdout.
  */
-novalue ppecho()
+void ppecho()
    {
    int c;
 
@@ -288,15 +274,7 @@ int ppch()
             while (p < blim && (isalnum(c = *p) || c == '_'))	/* find end */
                p++;
             bstop = p;			/* safe to consume through end */
-
-#if SCCX_MX
-            /* Avoids compiler warning */
-            if ( ((d = dlookup(bnxt, p - bnxt, bnxt)) == 0)  ||
-                  (d->inuse == 1) ) {
-#else
-            if (!(d = dlookup(bnxt, p - bnxt, bnxt)) || d->inuse == 1) {
-#endif					/* SCCX_MX */
-
+            if (((d = dlookup(bnxt, p-bnxt, bnxt)) == 0)  || (d->inuse == 1)) {
                bnxt++;
                return f;		/* not defined; just use it */
                }
@@ -399,6 +377,7 @@ int ppch()
 
 #if ARM || UNIX
          if (fs->m4flag) {			/* if m4 preprocessing */
+	    void quit();
             if (pclose(fs->fp) != 0)		/* close pipe */
                quit("m4 terminated abnormally");
             }
@@ -463,7 +442,7 @@ FILE *fp;
       lbuf = realloc(lbuf, (unsigned int)llen);
       if (!lbuf) {
          fprintf(stderr, "rline(%d): out of memory\n", llen);
-         exit(ErrorExit);
+         exit(EXIT_FAILURE);
          }
       p = lbuf + llen - LINE_SIZE_INCR - 2;
       n = LINE_SIZE_INCR;
@@ -473,7 +452,7 @@ FILE *fp;
 /*
  * pushdef(d) -- insert definition into the input stream.
  */
-static novalue pushdef(d)
+static void pushdef(d)
 cdefn *d;
    {
    buffer *b;
@@ -497,7 +476,7 @@ cdefn *d;
 /*
  * pushline(fname,lno) -- push #line directive into input stream.
  */
-static novalue pushline(fname, lno)
+static void pushline(fname, lno)
 char *fname;
 long lno;
    {
@@ -513,7 +492,7 @@ long lno;
  *
  *  s is the portion of the line following the $.
  */
-static novalue ppdir(s)
+static void ppdir(s)
 char *s;
    {
    char b0, *cmd, *errmsg;
@@ -547,7 +526,7 @@ char *s;
  *  We can't use tfatal() because we have our own line counter which may be
  *  out of sync with the lexical analyzer's.
  */
-static novalue pfatal(s1, s2)
+static void pfatal(s1, s2)
 char *s1, *s2;
    {
    int n;
@@ -751,7 +730,7 @@ char *s;
  *
  *  If report is nonzero, generate #line directive at end of skip.
  */
-static novalue skipcode(doelse, report)
+static void skipcode(doelse, report)
 int doelse, report;
    {
    char c, *p, *cmd;
@@ -828,14 +807,7 @@ char *s;
    while (isspace(c = *s))
       s++;
    if (c == '#')
-
-#if SCCX_MX
-      /* Avoids compiler warning */
-      while( (c = *++s) != 0)
-#else
-      while (c = *++s)
-#endif					/* SCCX_MX */
-
+      while ((c = *++s) != 0)
          ;
    return s;
    }
