@@ -84,15 +84,15 @@ struct b_proc {			/* procedure block */
    word title;			/*   T_Proc */
    word blksize;		/*   size of block */
 
-#if COMPILER
-   int (*ccode)();
-#else					/* COMPILER */
-   union {			/*   entry points for */
-      int (*ccode)();		/*     C routines */
-      uword ioff;		/*     and icode as offset */
-      pointer icode;		/*     and icode as absolute pointer */
-      } entryp;
-#endif					/* COMPILER */
+   #if COMPILER
+      int (*ccode)();
+   #else				/* COMPILER */
+      union {			/*   entry points for */
+         int (*ccode)();	/*     C routines */
+         uword ioff;		/*     and icode as offset */
+         pointer icode;		/*     and icode as absolute pointer */
+         } entryp;
+   #endif				/* COMPILER */
 
    word nparam;			/*   number of parameters */
    word ndynam;			/*   number of dynamic locals */
@@ -240,9 +240,9 @@ struct region {
    };
 
 #ifdef Double
-/*
- * Data type the same size as a double but without alignment requirements.
- */
+   /*
+    * Data type the same size as a double but without alignment requirements.
+    */
    struct size_dbl {
        char s[sizeof(double)];
        };
@@ -254,15 +254,14 @@ struct region {
 /*
  * Structures for the compiler.
  */
-
-struct p_frame {
-   struct p_frame *old_pfp;
-   struct descrip *old_argp;
-   struct descrip *rslt;
-   continuation succ_cont;
-   struct tend_desc tend;
-   };
-#endif					/* COMPILER */
+   struct p_frame {
+      struct p_frame *old_pfp;
+      struct descrip *old_argp;
+      struct descrip *rslt;
+      continuation succ_cont;
+      struct tend_desc tend;
+      };
+   #endif				/* COMPILER */
 
 /*
  * when debugging is enabled a debug struct is placed after the tended
@@ -277,11 +276,9 @@ struct debug {
 union numeric {			/* long integers or real numbers */
    long integer;
    double real;
-
-#ifdef LargeInts
-   struct b_bignum *big;
-#endif				/* LargeInts */
-
+   #ifdef LargeInts
+      struct b_bignum *big;
+   #endif				/* LargeInts */
    };
 
 #if COMPILER
@@ -347,6 +344,7 @@ struct progstate {
    struct descrip eventcode;		/* &eventcode */
    struct descrip eventval;		/* &eventval */
    struct descrip eventsource;		/* &eventsource */
+   dptr Glbl_argp;			/* global argp */
 
    /*
     * trapped variable keywords' values
@@ -362,13 +360,13 @@ struct progstate {
    char *Ecode;
    word *Records;
    int *Ftabp;
-#ifdef FieldTableCompression
-   short Ftabwidth, Foffwidth;
-   unsigned char *Ftabcp, *Focp;
-   short *Ftabsp, *Fosp;
-   int *Fo;
-   char *Bm;
-#endif					/* FieldTableCompression */
+   #ifdef FieldTableCompression
+      short Ftabwidth, Foffwidth;
+      unsigned char *Ftabcp, *Focp;
+      short *Ftabsp, *Fosp;
+      int *Fo;
+      char *Bm;
+   #endif				/* FieldTableCompression */
    dptr Fnames, Efnames;
    dptr Globals, Eglobals;
    dptr Gnames, Egnames;
@@ -379,21 +377,21 @@ struct progstate {
    struct ipc_line *Ilines, *Elines;
    struct ipc_line * Current_line_ptr;
 
-#ifdef Graphics
-   struct descrip AmperX, AmperY, AmperRow, AmperCol;	/* &x, &y, &row, &col */
-   struct descrip AmperInterval;			/* &interval */
-   struct descrip LastEventWin;				/* last Event() win */
-   int LastEvFWidth;
-   int LastEvLeading;
-   int LastEvAscent;
-   uword PrevTimeStamp;					/* previous timestamp */
-   uword Xmod_Control, Xmod_Shift, Xmod_Meta;		/* control,shift,meta */
-   struct descrip Kywd_xwin[2];				/* &window + ... */
-#endif					/* Graphics */
-
-#ifdef EventMon
-   word Linenum, Column, Lastline, Lastcol;
-#endif					/* EventMon */
+   #ifdef Graphics
+      struct descrip AmperX, AmperY, AmperRow, AmperCol;/* &x, &y, &row, &col */
+      struct descrip AmperInterval;			/* &interval */
+      struct descrip LastEventWin;			/* last Event() win */
+      int LastEvFWidth;
+      int LastEvLeading;
+      int LastEvAscent;
+      uword PrevTimeStamp;				/* previous timestamp */
+      uword Xmod_Control, Xmod_Shift, Xmod_Meta;	/* control,shift,meta */
+      struct descrip Kywd_xwin[2];			/* &window + ... */
+   #endif				/* Graphics */
+   
+   #ifdef EventMon
+      word Linenum, Column, Lastline, Lastcol;
+   #endif				/* EventMon */
 
    word Coexp_ser;			/* this program's serial numbers */
    word List_ser;
@@ -414,6 +412,7 @@ struct progstate {
    dptr Xargp;
    word Xnargs;
 
+   struct descrip K_current;
    int K_errornumber;
    char *K_errortext;
    struct descrip K_errorvalue;
@@ -428,7 +427,7 @@ struct progstate {
    struct b_file K_output;
    };
 
-#endif				/* MultiThread */
+#endif					/* MultiThread */
 
 /*
  * Frame markers
@@ -450,9 +449,9 @@ struct pf_marker {		/* procedure frame marker */
    word pf_ilevel;		/*   saved ilevel */
    dptr pf_scan;		/*   saved scanning environment */
 
-#ifdef MultiThread
-   struct progstate *pf_prog;	/*   saved program state pointer */
-#endif				/* MultiThread */
+   #ifdef MultiThread
+      struct progstate *pf_prog;/*   saved program state pointer */
+   #endif				/* MultiThread */
 
    struct descrip pf_locals[1];	/*   descriptors for locals */
    };
@@ -516,9 +515,9 @@ struct b_coexpr {		/* co-expression stack block */
    struct descrip freshblk;	/*   refresh block pointer */
    struct astkblk *es_actstk;	/*   pointer to activation stack structure */
 
-#ifdef MultiThread
-   struct progstate *program;
-#endif				/* MultiThread */
+   #ifdef MultiThread
+      struct progstate *program;
+   #endif				/* MultiThread */
 
    word cstate[CStateSize];	/*   C state information */
    };
@@ -535,11 +534,6 @@ struct b_refresh {		/* co-expression block */
 #endif					/* COMPILER */
 
 union block {			/* general block */
-
-#ifdef LargeInts
-   struct b_bignum bignumblk;
-#endif					/* LargeInts */
-
    struct b_real realblk;
    struct b_cset cset;
    struct b_file file;
@@ -557,4 +551,8 @@ union block {			/* general block */
    struct b_coexpr coexpr;
    struct b_external externl;
    struct b_slots slots;
+
+   #ifdef LargeInts
+      struct b_bignum bignumblk;
+   #endif				/* LargeInts */
    };
