@@ -6,25 +6,21 @@
  */
 
 #include "../h/define.h"
-#if VMS
-/* don't need path.h */
-#else					/* VMS */
-#include "../h/path.h"
-#endif					/* VMS */
 #include "../h/config.h"
 #include "../h/sys.h"
 #include "../h/typedefs.h"
 #include "../h/cstructs.h"
+#include "../h/proto.h"
 #include "../h/cpuconf.h"
 #include "../h/monitor.h"
 #include "../h/rmacros.h"
 #include "../h/rstructs.h"
 
 #ifdef Graphics
-#include "../h/graphics.h"
+   #include "../h/graphics.h"
 #endif					/* Graphics */
 
 #include "../h/rexterns.h"
-#include "../h/proto.h"
 #include "../h/rproto.h"
+
 #endif					/* RT_DOT_H */
