@@ -13,7 +13,7 @@
 /*
  * Prototypes.
  */
-hidden novalue	trans1		Params((char *filename));
+static void	trans1		(char *filename);
 
 /*
  * Variables.
@@ -36,9 +36,7 @@ int trans()
    register struct pentry *proc;
    struct srcfile *sf;
 
-#ifdef EnvVars
-   lpath = getenv("LPATH");	/* else remains null */
-#endif					/* EnvVars */
+   lpath = getenv("LPATH");	/* remains null if unspecified */
 
    for (sf = srclst; sf != NULL; sf = sf->next)
       trans1(sf->name);	/* translate each file in turn */
@@ -107,7 +105,7 @@ int trans()
 /*
  * translate one file.
  */
-static novalue trans1(filename)
+static void trans1(filename)
 char *filename;
    {
    in_line = 1;			/* start with line 1, column 0 */
@@ -142,7 +140,7 @@ char *filename;
 /*
  * writecheck - check the return code from a stdio output operation
  */
-novalue writecheck(rc)
+void writecheck(rc)
    int rc;
 
    {
@@ -153,7 +151,7 @@ novalue writecheck(rc)
 /*
  * lnkdcl - find file locally or on LPATH and add to source list.
  */
-novalue lnkdcl(name)
+void lnkdcl(name)
 char *name;
 {
    struct srcfile **pp;
@@ -170,7 +168,7 @@ char *name;
  * src_file - add the file name to the list of source files to be translated,
  *   if it is not already on the list.
  */
-novalue src_file(name)
+void src_file(name)
 char *name;
    {
    struct srcfile **pp;
