@@ -20,42 +20,44 @@
  * Prototypes.
  */
 
-hidden novalue	backpatch	Params((int lab));
-hidden novalue	clearlab	Params((noargs));
-hidden novalue	flushcode	Params((noargs));
-hidden novalue	align		Params((noargs));
-hidden novalue	misalign	Params((noargs));
-hidden novalue	intout		Params((int oint));
-hidden novalue	charout		Params((unsigned char oint));
-hidden novalue	shortout	Params((short oint));
-hidden novalue	lemit		Params((int op,char *name));
-hidden novalue	lemitcon	Params((int k));
-hidden novalue	lemitin		Params((int op,word offset,int n,char *name));
-hidden novalue	lemitint	Params((int op,long i,char *name));
-hidden novalue	lemitl		Params((int op,int lab,char *name));
-hidden novalue	lemitn		Params((int op,word n,char *name));
-hidden novalue	lemitproc
-   Params((word name,int nargs,int ndyn,int nstat, int fstat));
-hidden novalue	lemitr		Params((int op,word loc,char *name));
-hidden novalue	outblock	Params((char *addr,int count));
-hidden novalue	setfile		Params((noargs));
-hidden novalue	wordout		Params((word oword));
+static void	align		(void);
+static void	backpatch	(int lab);
+static void	clearlab	(void);
+static void	flushcode	(void);
+static void	intout		(int oint);
+static void	lemit		(int op,char *name);
+static void	lemitcon	(int k);
+static void	lemitin		(int op,word offset,int n,char *name);
+static void	lemitint	(int op,long i,char *name);
+static void	lemitl		(int op,int lab,char *name);
+static void	lemitn		(int op,word n,char *name);
+static void	lemitproc    (word name,int nargs,int ndyn,int nstat,int fstat);
+static void	lemitr		(int op,word loc,char *name);
+static void	misalign	(void);
+static void	outblock	(char *addr,int count);
+static void	setfile		(void);
+static void	wordout		(word oword);
+
+#ifdef FieldTableCompression
+static void	charout		(unsigned char oint);
+static void	shortout	(short oint);
+#endif					/* FieldTableCompression */
 
 #ifdef DeBugLinker
-hidden novalue	dumpblock	Params((char *addr,int count));
+   static void	dumpblock	(char *addr,int count);
 #endif					/* DeBugLinker */
 
 #if AMIGA
-#include <fcntl.h>
+   #include <fcntl.h>
 #endif					/* AMIGA */
 
 #if MSDOS
-extern long fileOffsetOfStuffThatGoesInICX;
-  /* defined in Globals.h, set in link.c, used below */
+   extern long fileOffsetOfStuffThatGoesInICX;
+   /* defined in Globals.h, set in link.c, used below */
 #endif                                  /* MSDOS */
 
 #if MVS
-extern char *routname;
+   extern char *routname;
 #endif					/* MVS */
 
 word pc = 0;		/* simulated program counter */
@@ -75,7 +77,7 @@ word pc = 0;		/* simulated program counter */
  *  of arguments and usually culminates in the call of the appropriate
  *  lemit* routine.
  */
-novalue gencode()
+void gencode()
    {
    register int op, k, lab;
    int j, nargs, flags, implicit;
@@ -301,9 +303,9 @@ novalue gencode()
 
 
 #ifdef LineCodes
-#ifndef EventMon
+   #ifndef EventMon
             lemit(Op_Noop,"noop");
-#endif					/* EventMon */
+   #endif				/* EventMon */
 #endif					/* LineCodes */
 
             break;
@@ -468,7 +470,7 @@ novalue gencode()
 /*
  * setfile - handle Op_Filen.
  */
-hidden novalue setfile()
+static void setfile()
    {
    if (fnmfree >= &fnmtbl[fnmsize])
       fnmtbl = (struct ipc_fname *) trealloc(fnmtbl, &fnmfree,
@@ -500,7 +502,7 @@ hidden novalue setfile()
  *  Note that the majority of the code for the lemit* routines is for debugging
  *  purposes.
  */
-static novalue lemit(op, name)
+static void lemit(op, name)
 int op;
 char *name;
    {
@@ -508,16 +510,12 @@ char *name;
 #ifdef DeBugLinker
    if (Dflag)
       fprintf(dbgfile, "%ld:\t%d\t\t\t\t# %s\n", (long)pc, op, name);
-#else					/* DeBugLinker */
-#if MACINTOSH && MPW
-/* #pragma unused(name)	*/
-#endif					/* MACINTOSH && MPW */
 #endif					/* DeBugLinker */
 
    outop(op);
    }
 
-static novalue lemitl(op, lab, name)
+static void lemitl(op, lab, name)
 int op, lab;
 char *name;
    {
@@ -526,10 +524,6 @@ char *name;
 #ifdef DeBugLinker
    if (Dflag)
       fprintf(dbgfile, "%ld:\t%d\tL%d\t\t\t# %s\n", (long)pc, op, lab, name);
-#else					/* DeBugLinker */
-#if MACINTOSH && MPW
-/* #pragma unused(name)	*/
-#endif					/* MACINTOSH && MPW */
 #endif					/* DeBugLinker */
 
    if (lab >= maxlabels)
@@ -549,7 +543,7 @@ char *name;
 #endif					/* CRAY */
    }
 
-static novalue lemitn(op, n, name)
+static void lemitn(op, n, name)
 int op;
 word n;
 char *name;
@@ -560,10 +554,6 @@ char *name;
    if (Dflag)
       fprintf(dbgfile, "%ld:\t%d\t%ld\t\t\t# %s\n", (long)pc, op, (long)n,
          name);
-#else					/* DeBugLinker */
-#if MACINTOSH && MPW
-/* #pragma unused(name) */
-#endif					/* MACINTOSH && MPW */
 #endif					/* DeBugLinker */
 
    outop(op);
@@ -571,7 +561,7 @@ char *name;
    }
 
 
-static novalue lemitr(op, loc, name)
+static void lemitr(op, loc, name)
 int op;
 word loc;
 char *name;
@@ -593,17 +583,13 @@ char *name;
          fprintf(dbgfile, "%ld:\t%d\t*-%ld\t\t\t# %s\n",(long) pc, op,
             (long)-loc, name);
       }
-#else					/* DeBugLinker */
-#if MACINTOSH && MPW
-/* #pragma unused(name) */
-#endif					/* MACINTOSH && MPW */
 #endif					/* DeBugLinker */
 
    outop(op);
    outword(loc);
    }
 
-static novalue lemitin(op, offset, n, name)
+static void lemitin(op, offset, n, name)
 int op, n;
 word offset;
 char *name;
@@ -614,10 +600,6 @@ char *name;
    if (Dflag)
       fprintf(dbgfile, "%ld:\t%d\t%d,S+%ld\t\t\t# %s\n", (long)pc, op, n,
          (long)offset, name);
-#else					/* DeBugLinker */
-#if MACINTOSH && MPW
-/* #pragma unused(name) */
-#endif					/* MACINTOSH && MPW */
 #endif					/* DeBugLinker */
 
    outop(op);
@@ -632,7 +614,7 @@ char *name;
  *  the same size as what the interpreter expects.  See op_int and op_intx
  *  in interp.s
  */
-static novalue lemitint(op, i, name)
+static void lemitint(op, i, name)
 int op;
 long i;
 char *name;
@@ -642,17 +624,13 @@ char *name;
 #ifdef DeBugLinker
    if (Dflag)
       fprintf(dbgfile,"%ld:\t%d\t%ld\t\t\t# %s\n",(long)pc,op,(long)i,name);
-#else					/* DeBugLinker */
-#if MACINTOSH && MPW
-/* #pragma unused(name) */
-#endif					/* MACINTOSH && MPW */
 #endif					/* DeBugLinker */
 
    outop(op);
    outword(i);
    }
 
-static novalue lemitcon(k)
+static void lemitcon(k)
 register int k;
    {
    register int i, j;
@@ -688,14 +666,14 @@ register int k;
       outword(T_Real);
 
 #ifdef Double
-#if WordBits != 64
-/* fill out real block with an empty word */
+   #if WordBits != 64
+      /* fill out real block with an empty word */
       outword(0);
-#ifdef DeBugLinker
-      if (Dflag)
-	 fprintf(dbgfile,"\t0\t\t\t\t\t# padding\n");
-#endif					/* DeBugLinker */
-#endif					/* WordBits != 64 */
+      #ifdef DeBugLinker
+         if (Dflag)
+	    fprintf(dbgfile,"\t0\t\t\t\t\t# padding\n");
+      #endif				/* DeBugLinker */
+   #endif				/* WordBits != 64 */
 #endif					/* Double */
 
       outblock(x.ovly,sizeof(double));
@@ -734,7 +712,7 @@ register int k;
       }
    }
 
-static novalue lemitproc(name, nargs, ndyn, nstat, fstat)
+static void lemitproc(name, nargs, ndyn, nstat, fstat)
 word name;
 int nargs, ndyn, nstat, fstat;
    {
@@ -838,7 +816,7 @@ int nargs, ndyn, nstat, fstat;
  *  identifier, and record tables, and built-in procedure blocks.
  */
 
-novalue gentables()
+void gentables()
    {
    register int i;
    register char *s;
@@ -944,10 +922,10 @@ novalue gentables()
     */
 #ifndef FieldTableCompression
    hdr.Ftab = pc;
-#ifdef DeBugLinker
-   if (Dflag)
-      fprintf(dbgfile, "\n%ld:\t\t\t\t\t# record/field table\n", (long)pc);
-#endif					/* DeBugLinker */
+   #ifdef DeBugLinker
+      if (Dflag)
+         fprintf(dbgfile, "\n%ld:\t\t\t\t\t# record/field table\n", (long)pc);
+   #endif				/* DeBugLinker */
 #else					/* FieldTableCompression */
    hdr.Fo = pc;
    /*
@@ -998,11 +976,11 @@ novalue gentables()
    for (fp = lffirst; fp != NULL; fp = fp->f_nextentry) {
 
 #ifndef FieldTableCompression
-#ifdef DeBugLinker
+   #ifdef DeBugLinker
       if (Dflag)
          fprintf(dbgfile, "%ld:\t\t\t\t\t# %s\n", (long)pc,
             &lsspace[fp->f_name]);
-#endif					/* DeBugLinker */
+   #endif				/* DeBugLinker */
 #endif					/* FieldTableCompression */
 
       rp = fp->f_rlist;
@@ -1045,10 +1023,10 @@ novalue gentables()
 #ifdef FieldTableCompression
             f_row[i] = -1;
 #else					/* FieldTableCompression */
-#ifdef DeBugLinker
+   #ifdef DeBugLinker
             if (Dflag)
                 fprintf(dbgfile, "\t-1\n");
-#endif					/* DeBugLinker */
+   #endif					/* DeBugLinker */
 
             outop(-1);
 #endif					/* FieldTableCompression */
@@ -1056,10 +1034,10 @@ novalue gentables()
          }
 
 #ifndef FieldTableCompression
-#ifdef DeBugLinker
+   #ifdef DeBugLinker
          if (Dflag && (i == nrecords || (i & 03) == 0))
             putc('\n', dbgfile);
-#endif					/* DeBugLinker */
+   #endif					/* DeBugLinker */
 #endif					/* FieldTableCompression */
 
       }
@@ -1118,10 +1096,10 @@ novalue gentables()
 
 #ifdef FieldTableCompression
    /* Write out the arrays. */
-#ifdef DeBugLinker
-   if (Dflag)
-      fprintf (dbgfile, "\n%ld:\t\t\t\t\t# field offset array\n", (long)pc);
-#endif					/* DeBugLinker */
+   #ifdef DeBugLinker
+      if (Dflag)
+         fprintf (dbgfile, "\n%ld:\t\t\t\t\t# field offset array\n", (long)pc);
+   #endif					/* DeBugLinker */
 
       /*
        * Compute largest value stored in fo array
@@ -1381,8 +1359,10 @@ novalue gentables()
             }
          fprintf(dbgfile, "   ");
          for (i = 0; i < 8; i++)
-            if (j < lsfree)
-               putc(isprint(c = lsspace[j++] & 0377) ? c : ' ', dbgfile);
+            if (j < lsfree) {
+               c = lsspace[j++];
+               putc(isprint(c & 0377) ? c : ' ', dbgfile);
+	       }
          putc('\n', dbgfile);
          }
       }
@@ -1424,26 +1404,25 @@ novalue gentables()
 #ifdef Header
    fseek(outfile, hdrsize, 0);
 #else                                   /* Header */
-
-#if MVS
-/*
- * This kind of backpatching cannot work on a PDS member, and that's
- *  probably where the code is going.  So the code goes out first to
- *  a temporary file, and then copied to the real icode file after
- *  the header is written.
- */
-   fseek(outfile, sizeof(hdr), SEEK_SET);
-   toutfile = outfile;
-   outfile = fopen(routname, WriteBinary);
-   if (outfile == NULL)
-      quitf("cannot create %s",routname);
-#else
-#if MSDOS
-   fseek(outfile, fileOffsetOfStuffThatGoesInICX, 0);
-#else					/* MSDOS */
-   fseek(outfile, 0L, 0);
-#endif					/* MSDOS */
-#endif                                  /* MVS */
+   #if MVS
+      /*
+       * This kind of backpatching cannot work on a PDS member, and that's
+       *  probably where the code is going.  So the code goes out first to
+       *  a temporary file, and then copied to the real icode file after
+       *  the header is written.
+       */
+      fseek(outfile, sizeof(hdr), SEEK_SET);
+      toutfile = outfile;
+      outfile = fopen(routname, WriteBinary);
+      if (outfile == NULL)
+         quitf("cannot create %s",routname);
+   #else
+      #if MSDOS
+         fseek(outfile, fileOffsetOfStuffThatGoesInICX, 0);
+      #else				/* MSDOS */
+         fseek(outfile, 0L, 0);
+      #endif				/* MSDOS */
+   #endif				/* MVS */
 #endif                                  /* Header */
 
    if (longwrite((char *)&hdr, (long)sizeof(hdr), outfile) < 0)
@@ -1481,7 +1460,7 @@ novalue gentables()
 /*
  * align() outputs zeroes as padding until pc is a multiple of WordSize.
  */
-static novalue align()
+static void align()
    {
    static word x = 0;
 
@@ -1494,7 +1473,7 @@ static novalue align()
  *  is not a multiple of WordSize.  This is for operations that output
  *  an int opcode followed by an operand that needs to be word-aligned.
  */
-static novalue misalign()
+static void misalign()
    {
    if ((pc + IntBits/ByteBits) % WordSize != 0)
       lemit(Op_Noop, "noop [pad]");
@@ -1504,7 +1483,7 @@ static novalue misalign()
  * intout(i) outputs i as an int that is used by the runtime system
  *  IntBits/ByteBits bytes must be moved from &word[0] to &codep[0].
  */
-static novalue intout(oint)
+static void intout(oint)
 int oint;
    {
    int i;
@@ -1527,7 +1506,7 @@ int oint;
 /*
  * charout(i) outputs i as an unsigned char that is used by the runtime system
  */
-static novalue charout(unsigned char ochar)
+static void charout(unsigned char ochar)
    {
    CodeCheck(1);
    *codep++ = (unsigned char)ochar;
@@ -1537,7 +1516,7 @@ static novalue charout(unsigned char ochar)
  * shortout(i) outputs i as a short that is used by the runtime system
  *  IntBits/ByteBits bytes must be moved from &word[0] to &codep[0].
  */
-static novalue shortout(short oint)
+static void shortout(short oint)
    {
    int i;
    union {
@@ -1561,7 +1540,7 @@ static novalue shortout(short oint)
  * wordout(i) outputs i as a word that is used by the runtime system
  *  WordSize bytes must be moved from &oword[0] to &codep[0].
  */
-static novalue wordout(oword)
+static void wordout(oword)
 word oword;
    {
    int i;
@@ -1583,7 +1562,7 @@ word oword;
 /*
  * outblock(a,i) output i bytes starting at address a.
  */
-static novalue outblock(addr,count)
+static void outblock(addr,count)
 char *addr;
 int count;
    {
@@ -1598,7 +1577,7 @@ int count;
  * dumpblock(a,i) dump contents of i bytes at address a, used only
  *  in conjunction with -L.
  */
-static novalue dumpblock(addr, count)
+static void dumpblock(addr, count)
 char *addr;
 int count;
    {
@@ -1615,7 +1594,7 @@ int count;
 /*
  * flushcode - write buffered code to the output file.
  */
-static novalue flushcode()
+static void flushcode()
    {
    if (codep > codeb)
       if (longwrite(codeb, DiffPtrs(codep,codeb), outfile) < 0)
@@ -1626,7 +1605,7 @@ static novalue flushcode()
 /*
  * clearlab - clear label table to all zeroes.
  */
-static novalue clearlab()
+static void clearlab()
    {
    register int i;
 
@@ -1637,7 +1616,7 @@ static novalue clearlab()
 /*
  * backpatch - fill in all forward references to lab.
  */
-static novalue backpatch(lab)
+static void backpatch(lab)
 int lab;
    {
    word p, r;
@@ -1671,7 +1650,7 @@ int lab;
    }
 
 #ifdef DeBugLinker
-novalue idump(s)		/* dump code region */
+void idump(s)		/* dump code region */
    char *s;
    {
    int *c;
