@@ -12,8 +12,8 @@
  * Prototypes.
  */
 
-hidden novalue	scanfile	Params((char *filename));
-hidden novalue	reference	Params((struct gentry *gp));
+static void	scanfile	(char *filename);
+static void	reference	(struct gentry *gp);
 
 int nrecords = 0;		/* number of records in program */
 
@@ -21,7 +21,7 @@ int nrecords = 0;		/* number of records in program */
  * readglob reads the global information from infile (.u2) and merges it with
  *  the global table and record table.
  */
-novalue readglob()
+void readglob()
    {
    register word id;
    register int n, op;
@@ -41,7 +41,7 @@ novalue readglob()
       fprintf(stderr,"\texpected version: %s\n",UVersion);
 
 
-      exit(ErrorExit);
+      exit(EXIT_FAILURE);
 
       }
    while ((op = getopc(&name)) != EOF) {
@@ -116,16 +116,16 @@ novalue readglob()
          case Op_Link:		/* link the named file */
             name = &lsspace[getrest()];	/* get the name and */
 
-#if ARM
-            {
+            #if ARM
+               {
                 char *flipname (char *);
 
                 /* put it on the list of files to link */
                 alsolink(flipname(name));
-            }
-#else					/* ARM */
-            alsolink(name);	/*  put it on the list of files to link */
-#endif					/* ARM */
+               }
+            #else			/* ARM */
+               alsolink(name);	/*  put it on the list of files to link */
+            #endif			/* ARM */
 
             newline();
             break;
@@ -141,7 +141,7 @@ novalue readglob()
  *
  * Called only if -fs is *not* specified (or implied by "invocable all").
  */
-novalue scanrefs()
+void scanrefs()
    {
    int i, n;
    char *t, *old;
@@ -156,11 +156,7 @@ novalue scanrefs()
     * Loop through .u1 files and accumulate reference lists.
     */
    lfls = llfiles;
-#if SCCX_MX
-   while( (lf = getlfile(&lfls)) != 0)
-#else					/* SCCX_MX */
-   while (lf = getlfile(&lfls))
-#endif					/* SCCX_MX */
+   while ((lf = getlfile(&lfls)) != 0)
       scanfile(lf->lf_name);
    lstatics = 0;			/* discard accumulated statics */
 
@@ -299,7 +295,7 @@ novalue scanrefs()
 /*
  * scanfile -- scan one file for references.
  */
-static novalue scanfile(filename)
+static void scanfile(filename)
 char *filename;
    {
    int i, k, f, op, nrefs, flags;
@@ -309,13 +305,13 @@ char *filename;
 
    makename(inname, SourceDir, filename, U1Suffix);
 
-#if MVS || VM
-   infile = fopen(inname, ReadBinary);
-   if (infile != NULL)		/* discard the extra blank we had */
-      (void)getc(infile);	/* to write to make it non-empty  */
-#else					/* MVS || VM */
-   infile = fopen(inname, ReadText);
-#endif					/* MVS || VM */
+   #if MVS || VM
+      infile = fopen(inname, ReadBinary);
+      if (infile != NULL)		/* discard the extra blank we had */
+         (void)getc(infile);		/* to write to make it non-empty  */
+   #else				/* MVS || VM */
+      infile = fopen(inname, ReadText);
+   #endif				/* MVS || VM */
 
    if (infile == NULL)
       quitf("cannot open %s", inname);
@@ -368,7 +364,7 @@ char *filename;
 /*
  *
  */
-static novalue reference(gp)
+static void reference(gp)
 struct gentry *gp;
    {
    struct gentry **rp;
@@ -376,11 +372,7 @@ struct gentry *gp;
    if (gp->g_flag & F_Unref) {
       gp->g_flag &= ~F_Unref;
       if ((rp = gp->g_refs) != NULL)
-#if SCCX_MX
-         while( (gp = *rp++) != 0)
-#else					/* SCCX_MX */
-         while (gp = *rp++)
-#endif					/* SCCX_MX */
+         while ((gp = *rp++) != 0)
             reference(gp);
       }
    }
