@@ -8,28 +8,25 @@
 #include "../h/header.h"
 
 #if SCCX_MX
-#undef ISICONT
-#define ISICONT
-#include "ixhdr.h"
+   #undef ISICONT
+   #define ISICONT
+   #include "ixhdr.h"
 #endif					/* SCCX_MX */
 
 #ifdef Header
-#ifndef ShellHeader
-#include "hdr.h"
-#endif					/* ShellHeader */
-
-
-#ifndef MaxHeader
-#define MaxHeader MaxHdr
-#endif					/* MaxHeader */
-
+   #ifndef ShellHeader
+      #include "hdr.h"
+   #endif					/* ShellHeader */
+   #ifndef MaxHeader
+      #define MaxHeader MaxHdr
+   #endif					/* MaxHeader */
 #endif					/* Header */
 
 /*
  * Prototype.
  */
 
-hidden	novalue	setexe	Params((char *fname));
+static	void	setexe	(char *fname);
 
 
 /*
@@ -39,7 +36,7 @@ hidden	novalue	setexe	Params((char *fname));
 
 #if PORT
    /* nothing to do */
-Deliberate Syntax Error
+   Deliberate Syntax Error
 #endif					/* PORT */
 
 #if AMIGA || ATARI_ST || MACINTOSH || VM || VMS
@@ -47,39 +44,39 @@ Deliberate Syntax Error
 #endif					/* AMIGA || ATARI_ST || ... */
 
 #if ARM
-#include "kernel.h"
-#include "swis.h"
+   #include "kernel.h"
+   #include "swis.h"
 #endif					/* ARM */
 
 #if MSDOS
    extern char pathToIconDOS[];
-#if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
-#include <fcntl.h>
-#endif					/* MICROSOFT || TURBO ... */
+   #if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
+      #include <fcntl.h>
+   #endif				/* MICROSOFT || TURBO ... */
 #endif					/* MSDOS */
 
 #if MVS
-char *routname;				/* real output file name */
+   char *routname;			/* real output file name */
 #endif					/* MVS */
 
 #if OS2
-#if MICROSOFT || CSET2
-#include <fcntl.h>
-#endif					/* MICROSOFT || CSET2 */
+   #if MICROSOFT || CSET2
+      #include <fcntl.h>
+   #endif				/* MICROSOFT || CSET2 */
 #endif					/* OS2 */
 
 #if UNIX
-#ifdef CRAY
-#define word word_fubar
-#include <sys/types.h>
-#include <sys/stat.h>
-#undef word
-#else					/* CRAY */
-#ifndef XWindows
-#include <sys/types.h>
-#endif					/* XWindows */
-#include <sys/stat.h>
-#endif					/* CRAY */
+   #ifdef CRAY
+      #define word word_fubar
+      #include <sys/types.h>
+      #include <sys/stat.h>
+      #undef word
+   #else				/* CRAY */
+      #ifndef XWindows
+         #include <sys/types.h>
+      #endif				/* XWindows */
+      #include <sys/stat.h>
+   #endif				/* CRAY */
 #endif					/* UNIX */
 
 /*
@@ -92,8 +89,8 @@ FILE *outfile;				/* interpreter code output file */
 extern char *ofile;			/* output file name */
 
 #ifdef DeBugLinker
-FILE *dbgfile;				/* debug file */
-static char dbgname[MaxFileName];	/* debug file name */
+   FILE *dbgfile;			/* debug file */
+   static char dbgname[MaxFileName];	/* debug file name */
 #endif					/* DeBugLinker */
 
 char inname[MaxFileName];		/* input file name */
@@ -141,12 +138,12 @@ char *outname;
       makename(icnname, TargetDir, filename, SourceSuffix);
 
 #if MVS || VM
-/*
- * Even though the ucode data is all reasonable text characters, use
- *  of text I/O may cause problems if a line is larger than LRECL.
- *  This is likely to be true with any compiler, though the precise
- *  disaster which results may vary.
- */
+      /*
+       * Even though the ucode data is all reasonable text characters, use
+       *  of text I/O may cause problems if a line is larger than LRECL.
+       *  This is likely to be true with any compiler, though the precise
+       *  disaster which results may vary.
+       */
       infile = fopen(inname, ReadBinary);
 #else					/* MVS || VM */
       infile = fopen(inname, ReadText);
@@ -173,7 +170,11 @@ char *outname;
    outfile = tmpfile();         /* write icode to temporary file to
                                    avoid fseek-PDS limitations */
 #else					/* MVS */
-   outfile = fopen(outname, WriteBinary);
+   #if AMIGA && __SASC
+      outfile = fopen(outname, ReadWriteBinary);
+   #else				/* AMIGA && __SASC */
+      outfile = fopen(outname, WriteBinary);
+   #endif				/* AMIGA && __SASC */
 #endif					/* MVS */
 
 /*
@@ -183,7 +184,7 @@ char *outname;
 
 #if PORT
    /* probably nothing */
-Deliberate Syntax Error
+   Deliberate Syntax Error
 #endif					/* PORT */
 
 #if AMIGA || ARM || ATARI_ST || MACINTOSH || MVS || UNIX || VM || VMS
@@ -191,16 +192,15 @@ Deliberate Syntax Error
 #endif					/* AMIGA || ARM || ATARI_ST || ... */
 
 #if MSDOS
-
-#if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
-   setmode(fileno(outfile),O_BINARY);	/* set for untranslated mode */
-#endif					/* MICROSOFT || TURBO ... */
+   #if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
+      setmode(fileno(outfile),O_BINARY);	/* set for untranslated mode */
+   #endif				/* MICROSOFT || TURBO ... */
 #endif					/* MSDOS */
 
 #if OS2
-#if MICROSOFT || CSET2
-   setmode(fileno(outfile),O_BINARY);
-#endif					/* MICROSOFT || CSET2 */
+   #if MICROSOFT || CSET2
+      setmode(fileno(outfile),O_BINARY);
+   #endif				/* MICROSOFT || CSET2 */
 #endif					/* OS2 */
 
 /*
@@ -320,26 +320,46 @@ Deliberate Syntax Error
     * Pad header to a multiple of 8 characters.
     */
    {
+
+#if AMIGA
+   char script[2 * MaxPath + 200];
+   sprintf(script,
+   "/* ARexx header for direct execution of Icon code */\n\
+   PARSE SOURCE type flag filename fullname ext host\n\
+   PARSE ARG 1 args 1\n\
+   SIGNAL ON ERROR\n\
+   ADDRESS command\n\
+   \"%s\" fullname args\n\
+   ERROR:\n\
+   EXIT\n\
+   /*/*/*/*/*/*/*/*/*/*/*/*/*/*/*/*\n[executable Icon binary follows]\n",
+   iconxloc);
+   strcat(script, "        \n\f\n" + ((int)(strlen(script) + 4) % 8));
+   hdrsize = strlen(script) + 1;	/* length includes \0 at end */
+   fwrite(script, hdrsize, 1, outfile);	/* write header */
+#else                                   /* AMIGA */
    char cmd[MaxPath + 100], script[2 * MaxPath + 200];
+#endif                                  /* AMIGA */
 
 #if NT
    /*
     * The NT and Win95 direct execution batch file turns echoing off,
-    * launches wiconx, attempts to terminate softly via iconnoop.bat,
+    * launches wiconx, attempts to terminate softly via noop.bat,
     * and terminates the hard way (by exiting the DOS shell) if that
     * fails, rather than fall through and start executing machine code
     * as if it were batch commands.
     */
    sprintf(script,
-           "@echo off\r\n%s %%0%s %%1 %%2 %%3 %%4 %%5 %%6 %%7 %%8 %%9\r\n",
-           ((iconxloc!=NULL)?iconxloc:"wiconx"), ".bat");
+           "@echo off\r\n%s %%0 %%1 %%2 %%3 %%4 %%5 %%6 %%7 %%8 %%9\r\n",
+           ((iconxloc!=NULL)?iconxloc:"wiconx"));
    strcat(script,"noop.bat\r\n@echo on\r\n");
    strcat(script,"pause missing noop.bat - press ^c or shell will exit\r\n");
    strcat(script,"exit\r\nrem [executable Icon binary follows]\r\n");
    strcat(script, "        \n\f\n" + ((int)(strlen(script) + 4) % 8));
    hdrsize = strlen(script) + 1;	/* length includes \0 at end */
    fwrite(script, hdrsize, 1, outfile);	/* write header */
-#else					/* NT */
+#endif					/* NT */
+#if UNIX
    sprintf(cmd, "exec \"${ICONX-%s}\" \"$0\"", iconxloc);
    sprintf(script, "%s   0) %s;;\n   *) %s \"$@\";;%s\n",
       "#!/bin/sh\n\ncase $# in\n", cmd, cmd,
@@ -347,7 +367,7 @@ Deliberate Syntax Error
    strcat(script, "        \n\f\n" + ((int)(strlen(script) + 4) % 8));
    hdrsize = strlen(script) + 1;	/* length includes \0 at end */
    fwrite(script, hdrsize, 1, outfile);	/* write header */
-#endif					/* NT */
+#endif					/* UNIX */
    }
 #else					/* ShellHeader */
    /*
@@ -380,14 +400,7 @@ Deliberate Syntax Error
     * Loop through input files and generate code for each.
     */
    lfls = llfiles;
-
-#if SCCX_MX
-   /* Avoid compiler warning */
-   while( (lf = getlfile(&lfls)) != 0) {
-#else
-   while (lf = getlfile(&lfls)) {
-#endif				/* SCCX_MX */
-
+   while ((lf = getlfile(&lfls)) != 0) {
       filename = lf->lf_name;
       makename(inname, SourceDir, filename, U1Suffix);
       makename(icnname, TargetDir, filename, SourceSuffix);
@@ -411,6 +424,45 @@ Deliberate Syntax Error
 
    gentables();		/* Generate record, field, global, global names,
 			   static, and identifier tables. */
+
+#if AMIGA
+#ifdef ShellHeader
+   /* The icode is enclosed in a deeply nested ARexx comment.
+      Here we close up the comment. */
+     {
+     int c, count = 0, state = 0;
+
+     rewind(outfile);
+     while ((c = fgetc(outfile)) != EOF)
+       switch(state){
+         case 0:  /* start state */
+           if (c == '/' || c == '*') state = c;
+           break;
+         case '/':
+           if (c == '*') count++;
+           state = 0;
+           break;
+         case '*':
+           if (c == '/') count--;
+           state = 0;
+           break;
+         }
+     if (count < 0){
+       fprintf(stderr, "       Failed to write direct execution header.\n\
+       You will have to use iconx to execute %s.\n", outname);
+       outname = NULL;
+       }
+     else {
+       fseek(outfile, 0, SEEK_END);
+       while (0 < count--){
+         fputc('*', outfile);
+         fputc('/', outfile);
+         }
+       }
+     }
+#endif					/* ShellHeader */
+#endif					/* AMIGA */
+
    fclose(outfile);
    lmfree();
    if (fatals > 0)
@@ -420,12 +472,12 @@ Deliberate Syntax Error
    }
 
 #ifdef ConsoleWindow
-extern FILE *flog;
+   extern FILE *flog;
 #endif					/* ConsoleWindow */
 /*
  * lwarn - issue a linker warning message.
  */
-novalue lwarn(s1, s2, s3)
+void lwarn(s1, s2, s3)
 char *s1, *s2, *s3;
    {
 
@@ -450,7 +502,7 @@ char *s1, *s2, *s3;
  * lfatal - issue a fatal linker error message.
  */
 
-novalue lfatal(s1, s2)
+void lfatal(s1, s2)
 char *s1, *s2;
    {
 
@@ -474,7 +526,7 @@ char *s1, *s2;
  * setexe - mark the output file as executable
  */
 
-static novalue setexe(fname)
+static void setexe(fname)
 char *fname;
    {
 
@@ -489,7 +541,24 @@ Deliberate Syntax Error
 #endif					/* PORT */
 
 #if AMIGA
-   /* not necessary */
+   #if __SASC
+      if (fname != NULL) {
+      struct DiskObject *dob;
+   
+      chmod(fname,S_ISCRIPT|S_IREAD|S_IWRITE|S_IEXECUTE|S_IDELETE);
+      /* Create a WorkBench Icon if necessary. */
+      dob = GetDiskObject(fname);
+      if (dob == NULL) {
+         dob = GetDiskObject(IconIcon);
+         if ( dob ) {
+            dob->do_CurrentX = NO_ICON_POSITION;
+            dob->do_CurrentY = NO_ICON_POSITION;
+            PutDiskObject(fname, dob);
+            }
+         }
+      if ( dob ) FreeDiskObject(dob);
+      }
+   #endif                                  /* __SASC */
 #endif					/* AMIGA */
 
 #if ARM
@@ -515,18 +584,17 @@ Deliberate Syntax Error
 #endif					/* ATARI_ST || MSDOS || ... */
 
 #if MACINTOSH
-#if MPW
-   /* Nothing to do here -- file is set to type TEXT
-      (so it can be executed as a script) in tmain.c.
-   */
-/* #pragma unused(fname) */
-#endif					/* MPW */
+   #if MPW
+      /* Nothing to do here -- file is set to type TEXT
+         (so it can be executed as a script) in tmain.c.
+      */
+   #endif				/* MPW */
 #endif					/* MACINTOSH */
 
 #if MSDOS
-#if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
-   chmod(fname,0755);	/* probably could be smarter... */
-#endif					/* MICROSOFT || TURBO ... */
+   #if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
+      chmod(fname,0755);	/* probably could be smarter... */
+   #endif				/* MICROSOFT || TURBO ... */
 #endif					/* MSDOS */
 
 #if OS2
@@ -568,10 +636,10 @@ Deliberate Syntax Error
 
 	system(cmdbuffer);
 
-	unlink(rcname);
+	remove(rcname);
 	makename(rcname,NULL,fname,".res");
-	unlink(rcname);
-	unlink(fname);
+	remove(rcname);
+	remove(fname);
     }
 #endif					/* OS2 */
 
