@@ -10,12 +10,12 @@
  * Prototypes.
  */
 
-hidden struct	lfile *alclfile	Params((char *name));
+static struct	lfile *alclfile	(char *name);
 
-novalue dumplfiles(noargs);
+void dumplfiles(void);
 
 #ifdef MultipleRuns
-hidden novalue	freelfile	Params((struct lfile *p));
+   static void	freelfile	(struct lfile *p);
 #endif					/* MultipleRuns */
 
 /*
@@ -48,17 +48,17 @@ struct gentry *lglast;		/* last global table entry */
 static char *ipath;		/* path for iconx */
 
 #ifdef MultipleRuns
-extern word pc;
-extern int fatals;
-extern int nlflag;
-extern int lstatics;
-extern int nfields;
+   extern word pc;
+   extern int fatals;
+   extern int nlflag;
+   extern int lstatics;
+   extern int nfields;
 #endif					/* MultipleRuns */
 
 /*
  * linit - scan the command line arguments and initialize data structures.
  */
-novalue linit()
+void linit()
    {
    struct gentry **gp;
    struct ientry **ip;
@@ -66,9 +66,7 @@ novalue linit()
 
    llfiles = NULL;		/* Zero queue of files to link. */
 
-#ifdef EnvVars
-   ipath = getenv(IPATH);	/* else just stays null */
-#endif					/* EnvVars */
+   ipath = getenv(IPATH);	/* remains null if unspecified */
 
    /*
     * Allocate the various data structures that are used by the linker.
@@ -107,17 +105,15 @@ novalue linit()
       *fp = NULL;
 
 #ifdef MultipleRuns
-
    /*
     * Initializations required for repeated program runs.
     */
-
    pc = 0;				/* In lcode.c	*/
    nrecords = 0;			/* In lglob.c	*/
 
-#ifdef EventMon
-   colmno = 0;				/* In link.c	*/
-#endif					/* EventMon */
+   #ifdef EventMon
+      colmno = 0;			/* In link.c	*/
+   #endif				/* EventMon */
 
    lineno = 0;				/* In link.c	*/
    fatals = 0;				/* In link.c	*/
@@ -139,7 +135,7 @@ novalue linit()
  * dumplfiles - print the list of files to link.  Used for debugging only.
  */
 
-novalue dumplfiles()
+void dumplfiles()
    {
    struct lfile *p,*lfls;
 
@@ -155,7 +151,7 @@ novalue dumplfiles()
  * alsolink - create an lfile structure for the named file and add it to the
  *  end of the list of files (llfiles) to generate link instructions for.
  */
-novalue alsolink(name)
+void alsolink(name)
 char *name;
    {
    struct lfile *nlf, *p;
@@ -219,18 +215,18 @@ char *name;
 /*
  * freelfile - free memory of an lfile structure.
  */
-static novalue freelfile(p)
+static void freelfile(p)
 struct lfile *p;
    {
    free((char *)p->lf_name);
    free((char *) p);
    }
-#endif						/* MultipleRuns */
+#endif					/* MultipleRuns */
 
 /*
  * lmfree - free memory used by the linker
  */
-novalue lmfree()
+void lmfree()
    {
    struct fentry *fp, *fp1;
    struct gentry *gp, *gp1;
@@ -279,12 +275,6 @@ novalue lmfree()
       freelfile(lf);
       }
    llfiles = NULL;
-
-#if MACINTOSH
-#if MPW
-/* #pragma unused(nlf,lf) */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
 #endif					/* MultipleRuns */
 
    }
