@@ -14,17 +14,17 @@
  * Prototypes.
  */
 
-hidden int	alclab		Params((int n));
-hidden novalue	binop		Params((int op));
-hidden novalue	emit		Params((char *s));
-hidden novalue	emitl		Params((char *s,int a));
-hidden novalue	emitlab		Params((int l));
-hidden novalue	emitn		Params((char *s,int a));
-hidden novalue	emits		Params((char *s,char *a));
-hidden novalue	setloc		Params((nodeptr n));
-hidden int	traverse	Params((nodeptr t));
-hidden novalue	unopa		Params((int op, nodeptr t));
-hidden novalue	unopb		Params((int op));
+static int	alclab		(int n);
+static void	binop		(int op);
+static void	emit		(char *s);
+static void	emitl		(char *s,int a);
+static void	emitlab		(int l);
+static void	emitn		(char *s,int a);
+static void	emits		(char *s,char *a);
+static void	setloc		(nodeptr n);
+static int	traverse	(nodeptr t);
+static void	unopa		(int op, nodeptr t);
+static void	unopb		(int op);
 
 extern int tfatals;
 extern int nocode;
@@ -68,7 +68,7 @@ static int nextlab;		/* next label allocated by alclab() */
  * codegen - traverse tree t, generating code.
  */
 
-novalue codegen(t)
+void codegen(t)
 nodeptr t;
    {
    nextlab = 1;
@@ -663,7 +663,7 @@ register nodeptr t;
  *  the name of operator is emitted.  For augmented operators, an "asgn"
  *  is emitted after the name of the operator.
  */
-static novalue binop(op)
+static void binop(op)
 int op;
    {
    register int asgn;
@@ -863,7 +863,7 @@ int op;
  *  number of "pnull" operations to receive the intermediate results.  This
  *  amounts to a pnull for each operation.
  */
-static novalue unopa(op,t)
+static void unopa(op,t)
 int op;
 nodeptr t;
    {
@@ -915,7 +915,7 @@ nodeptr t;
  *		emits "tabmat"
  *	    emits "compl"
  */
-static novalue unopb(op)
+static void unopb(op)
 int op;
    {
    register char *name;
@@ -999,7 +999,7 @@ int op;
 static char *lastfiln = NULL;
 static int lastlin = 0;
 
-static novalue setloc(n)
+static void setloc(n)
 nodeptr n;
    {
    if ((n != NULL) &&
@@ -1036,13 +1036,13 @@ nodeptr n;
 /*
  * Reinitialize last file name and line number for repeated runs.
  */
-novalue tcodeinit()
+void tcodeinit()
    {
    lastfiln = NULL;
 
-#ifdef EventMon
-   lastcol = 0;
-#endif					/* EventMon */
+   #ifdef EventMon
+      lastcol = 0;
+   #endif				/* EventMon */
 
    }
 #endif					/* Multiple Runs */
@@ -1056,26 +1056,26 @@ novalue tcodeinit()
  *  emitn(s,n) - emit instruction s with numeric argument a.
  *  emits(s,a) - emit instruction s with string argument a.
  */
-static novalue emitlab(l)
+static void emitlab(l)
 int l;
    {
    writecheck(fprintf(codefile, "lab L%d\n", l));
    }
 
-static novalue emit(s)
+static void emit(s)
 char *s;
    {
    writecheck(fprintf(codefile, "\t%s\n", s));
    }
 
-static novalue emitl(s, a)
+static void emitl(s, a)
 char *s;
 int a;
    {
    writecheck(fprintf(codefile, "\t%s\tL%d\n", s, a));
    }
 
-static novalue emitn(s, a)
+static void emitn(s, a)
 char *s;
 int a;
    {
@@ -1083,7 +1083,7 @@ int a;
    }
 
 
-static novalue emits(s, a)
+static void emits(s, a)
 char *s, *a;
    {
    writecheck(fprintf(codefile, "\t%s\t%s\n", s, a));
