@@ -6,78 +6,95 @@
 #include "tproto.h"
 
 #if SCCX_MX
-#include "../h/filepat.h"
-
-/* This sets the stack size so it runs in a Windows DOS box */
-unsigned _stack = 100000;
+   #include "../h/filepat.h"
+   /* This sets the stack size so it runs in a Windows DOS box */
+   unsigned _stack = 100000;
 #endif      /* SCCX_MX */
 
+#if AMIGA
+   #include <workbench/startup.h>
+   #if __SASC
+      #include <proto/dos.h>
+      #include <proto/icon.h>
+      #include <proto/wb.h>
+      #undef NOT            /* #defined in <intuition/intuition.h> */
+   #endif				/* __SASC */
+#endif                                  /* AMIGA */
+
 #if MACINTOSH
-#if THINK_C
-#include <string.h>
-#include "console.h"
-#include "config.h"
-#include "cpuconf.h"
-#include "macgraph.h"
-#include <AppleEvents.h>
-#include <GestaltEqu.h>
-/* #include <Values.h> */
+   #if THINK_C
+      #include "console.h"
+      #include "config.h"
+      #include "cpuconf.h"
+      #include "macgraph.h"
+      #include <AppleEvents.h>
+      #include <GestaltEqu.h>
+      /* #include <Values.h> */
 
-#define MAXLONG  (0x7fffffff)
+      #define MAXLONG  (0x7fffffff)
+      
+      /* #define kSleep MAXLONG */
+      #define kGestaltMask 1L
 
-/* #define kSleep MAXLONG */
-#define kGestaltMask 1L
-
-/* Global */
-Boolean gDone;
-Boolean g_cOption;
-
-
-#endif    /* THINK_C */
+      /* Global */
+      Boolean gDone;
+      Boolean g_cOption;
+   #endif    /* THINK_C */
 #endif    /* MACINTOSH */
 
 #ifdef MSWindows
-#ifdef NTConsole
-#define int_PASCAL int PASCAL
-#define LRESULT_CALLBACK LRESULT CALLBACK
-#include <windows.h>
-#include "../h/filepat.h"
-#endif					/* NTConsole */
+   #ifdef NTConsole
+      #define int_PASCAL int PASCAL
+      #define LRESULT_CALLBACK LRESULT CALLBACK
+      #include <windows.h>
+      #include "../h/filepat.h"
+   #endif				/* NTConsole */
 #endif					/* MSWindows */
 
+#if WildCards
+   #ifndef ConsoleWindow
+      #include "../h/filepat.h"
+   #endif				/* ConsoleWindow */
+#endif					/* WildCards */
+
 /*
  * Prototypes.
  */
 
-hidden	novalue	execute	Params((char *ofile,char *efile,char * *args));
-hidden	novalue	rmfiles Params((char **p));
-hidden	novalue	usage Params((noargs));
+static	void	execute	(char *ofile,char *efile,char * *args);
+static	void	rmfiles (char **p);
+static	void	usage (void);
+
+#if AMIGA && __SASC
+   extern void PostClip(char *file, int line, int number, char *text);
+   extern void CallARexx(char *script);
+#endif					/* AMIGA && __SASC */
 
 #if MACINTOSH
-#if THINK_C
-pascal void  MaxApplZone ( void );
-void         IncreaseStackSize ( Size extraBytes );
-
-void         ToolBoxInit ( void );
-void         EventInit ( void );
-void         EventLoop ( void );
-void         DoEvent ( EventRecord *eventPtr );
-pascal OSErr DoOpenDoc ( AppleEvent theAppleEvent,
+   #if THINK_C
+      pascal void  MaxApplZone ( void );
+      void         IncreaseStackSize ( Size extraBytes );
+      
+      void         ToolBoxInit ( void );
+      void         EventInit ( void );
+      void         EventLoop ( void );
+      void         DoEvent ( EventRecord *eventPtr );
+      pascal OSErr DoOpenDoc ( AppleEvent theAppleEvent,
                          AppleEvent reply,
                                long refCon );
-pascal OSErr DoQuitApp ( AppleEvent theAppleEvent,
+      pascal OSErr DoQuitApp ( AppleEvent theAppleEvent,
                          AppleEvent reply,
                                long refCon );
-OSErr        GotRequiredParams ( AppleEvent *appleEventPtr );
-novalue      MacMain ( int argc, char **argv );
+      OSErr        GotRequiredParams ( AppleEvent *appleEventPtr );
+      void      MacMain ( int argc, char **argv );
 
-void DoMouseDown (EventRecord *eventPtr);
-void HandleMenuChoice (long menuChoice);
-void HandleAppleChoice (short item);
-void HandleFileChoice (short item);
-void HandleOptionsChoice (short item);
-void MenuBarInit (void);
-#endif					/* THINK_C */
+      void DoMouseDown (EventRecord *eventPtr);
+      void HandleMenuChoice (long menuChoice);
+      void HandleAppleChoice (short item);
+      void HandleFileChoice (short item);
+      void HandleOptionsChoice (short item);
+      void MenuBarInit (void);
+   #endif					/* THINK_C */
 #endif					/* MACINTOSH */
 
 /*
@@ -86,15 +103,27 @@ void MenuBarInit (void);
  */
 
 #if PORT
-Deliberate syntax error
+   Deliberate syntax error
 #endif					/* PORT */
 
 #if AMIGA
-#include <libraries/dosextens.h>
+   #if __SASC
+      #include <dos.h>
+      #include <libraries/dos.h>
+      #include <proto/dos.h>
+      #include <proto/asl.h>
+      extern int _WBargc;
+      extern char **_WBargv;
+      char __stdiowin[] = "CON:10/40/640/200/IconT Console Window";
+      char OptTemplate[] = "NoLink/S,StrInv/S,Silent/S,Trace/S,UWarn/S,Output/K,Verbosity/K";
+      enum {opt_nolink,opt_strinv,opt_silent,opt_trace,opt_uwarn,opt_output,opt_verbosity,numopts}; 
+   #else				/* __SASC */
+      #include <libraries/dosextens.h>
+   #endif				/* __SASC */
 #endif					/* AMIGA */
 
 #if ARM || MVS || UNIX || VM || VMS
-/* nothing is needed */
+   /* nothing is needed */
 #endif					/* ARM || ... */
 
 #if MSDOS
@@ -102,28 +131,27 @@ Deliberate syntax error
 #endif					/* MSDOS */
 
 #if ATARI_ST
-char *patharg;
+   char *patharg;
 #endif					/* ATARI_ST */
 
 #if MACINTOSH
-#if MPW
-#include <fcntl.h>	/* MPW3 - for unlink() */
-#include <CursorCtl.h>
-void SortOptions();
-#endif					/* MPW */
+   #if MPW
+      #include <CursorCtl.h>
+      void SortOptions();
+   #endif				/* MPW */
 #endif					/* MACINTOSH */
 
 #if OS2
-#include <process.h>
+   #include <process.h>
 #endif					/* OS2 */
 /*
  * End of operating-system specific code.
  */
 
 #if IntBits == 16
-#ifdef strlen
-#undef strlen				/* pre-defined in some contexts */
-#endif					/* strlen */
+   #ifdef strlen
+   #undef strlen			/* pre-defined in some contexts */
+   #endif				/* strlen */
 #endif					/* Intbits == 16 */
 
 /*
@@ -145,15 +173,15 @@ char patchpath[MaxPath+18] = "%PatchStringHere->";
 
 #if PORT
    /* something is needed */
-Deliberate Syntax Error
+   Deliberate Syntax Error
 #endif					/* PORT */
 
 #if UNIX || AMIGA || ATARI_ST || MACINTOSH || MSDOS || MVS || OS2 || VM
-char *refpath = RefPath;
+   char *refpath = RefPath;
 #endif					/* UNIX ... */
 
 #if VMS
-char *refpath = "ICON_BIN:";
+   char *refpath = "ICON_BIN:";
 #endif					/* VMS */
 
 /*
@@ -163,12 +191,12 @@ char *refpath = "ICON_BIN:";
 /*
  * getopt() variables
  */
-extern int optindex;		/* index into parent argv vector */
+extern int optind;		/* index into parent argv vector */
 extern int optopt;		/* character checked for validity */
 extern char *optarg;		/* argument associated with option */
 
 #ifdef ConsoleWindow
-int ConsolePause = 1;
+   int ConsolePause = 1;
 #endif					/* ConsoleWindow */
 
 
@@ -244,7 +272,7 @@ void expand_proj(int *argc, char ***argv)
 
 #ifndef NTConsole
 #ifdef MSWindows
-novalue icont(int argc, char **argv);
+void icont(int argc, char **argv);
 #define int_PASCAL int PASCAL
 #define LRESULT_CALLBACK LRESULT CALLBACK
 #undef lstrlen
@@ -254,48 +282,95 @@ novalue icont(int argc, char **argv);
 
 int CmdParamToArgv(char *s, char ***avp)
    {
+   char tmp[MaxPath], dir[MaxPath];
    char *t, *t2;
-   int rv=0;
+   int rv=0, i;
+   FILE *f;
    t = salloc(s);
    t2 = t;
-   while (*t2) {
-      while (*t2 && isspace(*t2)) t2++;
-      if (!*t2) break;
-      rv++;
-      while (*t2 && !isspace(*t2)) t2++;
-      }
-   rv++; /* make room for "wicont" at front */
-
-   *avp = (char **)alloc((rv+1) * sizeof(char *));
-   rv = 0;
-   (*avp)[rv++] = salloc("wicont.exe");
-   t2 = t;
-   while (*t2) {
-      while (*t2 && isspace(*t2)) t2++;
-      if (!*t2) break;
-      (*avp)[rv++] = t2;
-      while (*t2 && !isspace(*t2)) t2++;
-      if (*t2) *t2++ = '\0';
-      }
+   *avp = malloc(2 * sizeof(char *));
    (*avp)[rv] = NULL;
+
+   while (*t2) {
+      while (*t2 && isspace(*t2)) t2++;
+      switch (*t2) {
+	 case '\0': break;
+	 case '"': {
+	    char *t3 = ++t2;			/* skip " */
+            while (*t2 && (*t2 != '"')) t2++;
+            if (*t2)
+	       *t2++ = '\0';
+	    *avp = realloc(*avp, (rv + 2) * sizeof (char *));
+	    (*avp)[rv++] = salloc(t3);
+            (*avp)[rv] = NULL;
+	    break;
+	    }
+         default: {
+            FINDDATA_T fd;
+	    char *t3 = t2;
+            while (*t2 && !isspace(*t2)) t2++;
+	    if (*t2)
+	       *t2++ = '\0';
+            strcpy(tmp, t3);
+	    if (!FINDFIRST(tmp, &fd)) {
+	       *avp = realloc(*avp, (rv + 2) * sizeof (char *));
+	       (*avp)[rv++] = salloc(t3);
+               (*avp)[rv] = NULL;
+               }
+	    else {
+               int end;
+               strcpy(dir, t3);
+	       do {
+	          end = strlen(dir)-1;
+	          while (end >= 0 && dir[end] != '\\' && dir[end] != '/' &&
+			dir[end] != ':') {
+                     dir[end] = '\0';
+		     end--;
+	             }
+		  strcat(dir, FILENAME(&fd));
+	          *avp = realloc(*avp, (rv + 2) * sizeof (char *));
+	          (*avp)[rv++] = salloc(dir);
+                  (*avp)[rv] = NULL;
+	          } while (FINDNEXT(&fd));
+	       FINDCLOSE(&fd);
+	       }
+            break;
+	    }
+         }
+      }
+   free(t);
    return rv;
    }
 
 
-LRESULT_CALLBACK WndProc	Params((HWND, UINT, WPARAM, LPARAM));
+LRESULT_CALLBACK WndProc	(HWND, UINT, WPARAM, LPARAM);
 char *lognam;
 char tmplognam[128];
 extern FILE *flog;
 
-novalue MSStartup(int argc, char **argv, HINSTANCE hInstance, HINSTANCE hPrevInstance)
+void MSStartup(int argc, char **argv, HINSTANCE hInstance, HINSTANCE hPrevInstance)
    {
    WNDCLASS wc;
-   /* generate log file?  might make this an option */
-   if ((lognam = getenv("ICONTLOG"))!=NULL) {
+
+   /*
+    * Select log file name.  Might make this a command-line option.
+    * Default to "WICON.LOG".  The log file is used by Wi to report
+    * translation errors and jump to the offending source code line.
+    */
+   if ((lognam = getenv("WICONLOG")) == NULL)
+      lognam = "WICON.LOG";
+   if (flog = fopen(lognam, "r")) {
+      fclose(flog);
+      flog = NULL;
       remove(lognam);
-      lognam = strdup(lognam);
-      flog = fopen(tmpnam(tmplognam), "w");
       }
+   lognam = strdup(lognam);
+   flog = fopen(tmpnam(tmplognam), "w");
+   if (flog == NULL) {
+      fprintf(stderr, "unable to open logfile");
+      exit(EXIT_FAILURE);
+      }
+
    if (!hPrevInstance) {
 #if NT
       wc.style = CS_HREDRAW | CS_VREDRAW;
@@ -332,13 +407,16 @@ int_PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
    mswinInstance = hInstance;
    ncmdShow = nCmdShow;
-   argc = CmdParamToArgv(lpszCmdParam, &argv);
+   argc = CmdParamToArgv(GetCommandLine(), &argv);
    MSStartup(argc, argv, hInstance, hPrevInstance);
 #if BORLAND_286
    _InitEasyWin();
 #endif					/* BORLAND_286 */
    if (setjmp(mark_sj) == 0)
       icont(argc,argv);
+   while (--argc>=0)
+      free(argv[argc]);
+   free(argv);
    wfreersc();
    return 0;
 }
@@ -353,9 +431,9 @@ int_PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  */
 
 #if !THINK_C
-novalue main(argc, argv)
+   int main(argc, argv)
 #else                           /* THINK_C */
-novalue MacMain(argc,argv)
+   void MacMain(argc,argv)
 #endif                          /* !THINKC_C */
 
 int argc;
@@ -396,6 +474,96 @@ char **argv;
       exit(-1);
       }
 #endif					/* AZTEC_C */
+#if __SASC
+   if ( argc == 0 ) {    /* We were started from the WorkBench. */
+      struct FileRequester *fr;
+      int  n, Rargc = 0;
+      char *pattern, **Rargv;
+      struct WBArg *wba;
+      struct RDArgs *rda;
+      long *options;
+
+   /* Get options from the IcontOptions environment variable. */ 
+      rda = (struct RDArgs *)alloc(sizeof(struct RDArgs));
+      options = (long *)alloc(numopts*sizeof(long));
+      if (rda->RDA_Source.CS_Buffer = getenv("IcontOptions") ) {
+         rda->RDA_Source.CS_Length = strlen(rda->RDA_Source.CS_Buffer);
+         ReadArgs(OptTemplate, options, rda);
+         FreeArgs(rda);
+
+         if (options[opt_nolink]   ) nolink = 1; 
+         if (options[opt_strinv]   ) strinv = 1;
+         if (options[opt_silent]   ) {silent = 1; verbose = 0;}
+         if (options[opt_trace]    ) trace = -1;
+         if (options[opt_uwarn]    ) uwarn = 1  ;
+         if (options[opt_output]   ) ofile = salloc((char *)options[opt_output]) ;
+         if (options[opt_verbosity]) {
+            if (sscanf( (char *)options[opt_verbosity], "%d%c", &verbose, &ch) != 1)
+               quitf("bad operand to -v option: %s",optarg);
+            if (verbose == 0)
+               silent = 1;
+            }
+         }
+   
+   /* Get file arguments from the WorkBench startup message. */
+      sprintf(buf, "#?%s", SourceSuffix);
+      pattern = salloc(buf);
+      fr = (struct FileRequester *)AllocAslRequestTags(ASL_FileRequest,
+           ASL_Hail,     "Select Icon source files",
+           ASL_Pattern,  (ULONG)pattern,
+           ASL_FuncFlags, FILF_MULTISELECT | FILF_PATGAD,
+           ASL_Dir,      "Icon:",
+           TAG_DONE);
+   
+      if ( fr != NULL) {
+         if (_WBargc == 1 ) {  /* No input files were specified */
+            if ( AslRequest(fr, NULL) ) {
+               Rargv = (char **)alloc((fr->fr_NumArgs + 2)*sizeof(char *));
+               Rargv[0] = _WBargv[0];
+               Rargc = 1;
+               wba = fr->fr_ArgList;
+               for(n = 0; n < fr->fr_NumArgs; n++, wba++){
+                  if (wba->wa_Name != NULL &&
+                     NameFromLock(wba->wa_Lock, buf, sizeof(buf)) != 0) {
+                     AddPart(buf, wba->wa_Name, sizeof(buf));
+                     Rargv[Rargc] = salloc(buf);
+                     Rargc++;
+                     }
+                  }
+               Rargv[Rargc] = NULL;
+               }
+            }
+   
+         /* Initialize argv and argc */
+         if ( Rargc > 1 ) {
+            argc = Rargc;
+            argv = Rargv;
+            }
+         else {
+            argc = _WBargc;
+            argv = _WBargv;
+            }
+   
+         /* The Workbench args come in random order.  So if we have
+            more than one ask for the name of the output file. */
+         if (ofile == NULL && nolink == 0 && argc > 2) {
+            strcpy(buf, argv[1]);
+            *PathPart(buf) = '\0';
+            if ( AslRequestTags(fr,
+                  ASL_Hail,      "Specify output (icode) file",
+                  ASL_Pattern,   NULL,
+                  ASL_FuncFlags, FILF_SAVE,
+                  ASL_Dir,       buf,
+                  ASL_File,      NULL,
+                  TAG_DONE) ) {
+               SetCurrentDirName(fr->fr_Drawer);
+               ofile = salloc(fr->fr_File);
+               }
+            }
+         FreeAslRequest(fr);
+         }
+      }
+#endif					/* __SASC */
 #endif					/* AMIGA */
 
 
@@ -480,7 +648,7 @@ char **argv;
             efile = optarg;
             break;
          case 'f':			/* -f features: enable features */
-            if (index(optarg, 's') || index(optarg, 'a'))
+            if (strchr(optarg, 's') || strchr(optarg, 'a'))
                strinv = 1;		/* this is the only icont feature */
             break;
 
@@ -548,7 +716,7 @@ char **argv;
          if (!pathCursor) {
             fprintf (stderr,
                "Can't understand what directory icont was run from.\n");
-            exit(ErrorExit);
+            exit(EXIT_FAILURE);
             }
             strcpy( ++pathCursor, (makeExe==1) ?  "ixhdr.exe" : "iconx.exe");
          }
@@ -557,11 +725,11 @@ char **argv;
    /*
     * Allocate space for lists of file names.
     */
-   n = argc - optindex + 1;
+   n = argc - optind + 1;
 
 #if WildCards
    {
-   for(j=optindex; j < argc; j++) {
+   for(j=optind; j < argc; j++) {
       if (FINDFIRST(argv[j], &fd))
          while(FINDNEXT(&fd)) n++;
       FINDCLOSE(&fd);
@@ -576,10 +744,10 @@ char **argv;
    /*
     * Scan file name arguments.
     */
-   while (optindex < argc)  {
-      if (strcmp(argv[optindex],"-x") == 0)	/* stop at -x */
+   while (optind < argc)  {
+      if (strcmp(argv[optind],"-x") == 0)	/* stop at -x */
          break;
-      else if (strcmp(argv[optindex],"-") == 0) {
+      else if (strcmp(argv[optind],"-") == 0) {
 
 #if ARM
 	/* Different file naming, so we need a different strategy... */
@@ -603,13 +771,13 @@ char **argv;
 	 char tmp[MaxPath], dir[MaxPath];
          int matches = 0;
 
-	 fp = fparse(argv[optindex]);
+	 fp = fparse(argv[optind]);
 	 /* Save because *fp will get overwritten frequently */
 	 strcpy(dir, fp->dir);
 	 if (*fp->ext == '\0')
-	    makename(tmp,NULL,argv[optindex], SourceSuffix);
+	    makename(tmp,NULL,argv[optind], SourceSuffix);
 	 else
-	    strcpy(tmp, argv[optindex]);
+	    strcpy(tmp, argv[optind]);
 
 	 if (strchr(tmp, '*') || strchr(tmp, '?')) {
 	    if (!FINDFIRST(tmp, &fd)) {
@@ -621,36 +789,37 @@ char **argv;
          do {
 	 if (matches) {
 	    makename(tmp,dir,FILENAME(&fd),NULL);
-	    argv[optindex] = tmp;
+	    argv[optind] = tmp;
 	    }
 #endif					/* WildCards */
-         fp = fparse(argv[optindex]);		/* parse file name */
+         fp = fparse(argv[optind]);		/* parse file name */
          if (*fp->ext == '\0' || smatch(fp->ext, SourceSuffix)) {
-            makename(buf,SourceDir,argv[optindex], SourceSuffix);
+            makename(buf,SourceDir,argv[optind], SourceSuffix);
 #if VMS
 	    strcat(buf, fp->version);
 #endif					/* VMS */
             *tptr++ = salloc(buf);		/* translate the .icn file */
-            makename(buf,TargetDir,argv[optindex],U1Suffix);
+            makename(buf,TargetDir,argv[optind],U1Suffix);
             *lptr++ = *rptr++ = salloc(buf);	/* link & remove .u1 */
-            makename(buf,TargetDir,argv[optindex],U2Suffix);
+            makename(buf,TargetDir,argv[optind],U2Suffix);
             *rptr++ = salloc(buf);		/* also remove .u2 */
             }
          else if (smatch(fp->ext,U1Suffix) || smatch(fp->ext,U2Suffix)
                || smatch(fp->ext,USuffix)) {
-            makename(buf,TargetDir,argv[optindex],U1Suffix);
+            makename(buf,TargetDir,argv[optind],U1Suffix);
             *lptr++ = salloc(buf);
             }
          else
-            quitf("bad argument %s",argv[optindex]);
+            quitf("bad argument %s",argv[optind]);
 #if WildCards
 	 if (!matches)
 	    break;
          } while (FINDNEXT(&fd));
+	 if (matches)
 	 FINDCLOSE(&fd);
 #endif					/* WildCards */
          }
-      optindex++;
+      optind++;
       }
 
    *tptr = *lptr = *rptr = NULL;	/* terminate filename lists */
@@ -673,8 +842,13 @@ char **argv;
       if (!pponly)
          report("Translating");
       errors = trans(tfiles);
-      if (errors > 0)			/* exit if errors seen */
-         exit(ErrorExit);
+      if (errors > 0) {			/* exit if errors seen */
+#if AMIGA && __SASC
+         PostClip("END", 0, 0, "END");
+         CallARexx(IcontRexx);
+#endif					/* AMIGA && __SASC */
+         exit(EXIT_FAILURE);
+	 }
       }
 
    /*
@@ -697,7 +871,11 @@ char **argv;
 #endif					/* MPW */
 #endif					/* MACINTOSH */
 
-      exit(NormalExit);
+#if AMIGA && __SASC
+      if ( !silent && _WBenchMsg != NULL ) fprintf(stderr,"Done.\n");
+#endif				/* AMIGA && __SASC */
+
+      exit(EXIT_SUCCESS);
       }
 
 #if MSDOS
@@ -705,20 +883,21 @@ char **argv;
    {
    if (ofile == NULL)  {                /* if no -o file, synthesize a name */
       ofile = salloc(makename(buf,TargetDir,lfiles[0],
-                              makeExe ? ".bat" : IcodeSuffix));
+                              makeExe ? ".exe" : IcodeSuffix));
       }
    else {                             /* add extension if necessary */
       fp = fparse(ofile);
       if (*fp->ext == '\0' && *IcodeSuffix != '\0') /* if no ext given */
          ofile = salloc(makename(buf,NULL,ofile,
-                                 makeExe ? ".bat" : IcodeSuffix));
+                                 makeExe ? ".exe" : IcodeSuffix));
       }
    }
 #else					/* NT */
    if (ofile == NULL)  {                /* if no -o file, synthesize a name */
       ofile = salloc(makename(buf,TargetDir,lfiles[0],
                               makeExe ? ".Exe" : IcodeSuffix));
-   } else {                             /* add extension if necessary */
+      }
+   else {                             /* add extension if necessary */
       fp = fparse(ofile);
       if (*fp->ext == '\0' && *IcodeSuffix != '\0') /* if no ext given */
          ofile = salloc(makename(buf,NULL,ofile,
@@ -741,6 +920,48 @@ char **argv;
    report("Linking");
    errors = ilink(lfiles,ofile);	/* link .u files to make icode file */
 
+#if NT
+   if (!stricmp(".exe", ofile+strlen(ofile)-4)) {
+      FILE *f, *f2;
+      char tmp[MaxPath], tmp2[MaxPath];
+      /* if we generated a .exe, we need to rename it and then prepend wiconx
+       */
+      strcpy(tmp, ofile);
+      strcpy(tmp+strlen(tmp)-4, ".bat");
+      rename(ofile, tmp);
+#ifdef MSWindows
+      if ((f = pathOpen("wiconx.exe", ReadBinary)) == NULL) {
+	 report("Tried to open wiconx to build .exe, but couldn't\n");
+         errors++;
+         }
+#else					/* MS Windows */
+      if ((f = pathOpen("nticonx.exe", ReadBinary)) == NULL) {
+	 report("Tried to open wiconx to build .exe, but couldn't\n");
+	 errors++;
+         }
+#endif					/* MS Windows */
+      else {
+         f2 = fopen(ofile, WriteBinary);
+	 while ((c = fgetc(f)) != EOF) {
+	    fputc(c, f2);
+	    }
+	 fclose(f);
+	 if ((f = fopen(tmp, ReadBinary)) == NULL) {
+	    report("tried to read .bat to append to .exe, but couldn't\n");
+	    errors++;
+	    }
+	 else {
+	    while ((c = fgetc(f)) != EOF) {
+	       fputc(c, f2);
+	       }
+	    fclose(f);
+	    }
+	 fclose(f2);
+	 unlink(tmp);
+         }
+      }
+#endif					/* NT */
+
    /*
     * Finish by removing intermediate files.
     *  Execute the linked program if so requested and if there were no errors.
@@ -755,8 +976,8 @@ char **argv;
 
    rmfiles(rfiles);			/* remove intermediate files */
    if (errors > 0) {			/* exit if linker errors seen */
-      unlink(ofile);
-      exit(ErrorExit);
+      remove(ofile);
+      exit(EXIT_FAILURE);
       }
 #ifdef ConsoleWindow
    else
@@ -764,19 +985,28 @@ char **argv;
 #endif					/* ConsoleWindow */
 
 #if !(MACINTOSH && MPW)
-   if (optindex < argc)  {
+   if (optind < argc)  {
       report("Executing");
-      execute (ofile, efile, argv+optindex+1);
+      execute (ofile, efile, argv+optind+1);
       }
 #endif					/* !(MACINTOSH && MPW) */
 
-   exit(NormalExit);
+#if AMIGA && __SASC
+   if (!silent && _WBenchMsg != NULL) fprintf(stderr, "Done.\n");
+#endif				/* AMIGA && __SASC */
+
+   free(tfiles);
+   free(lfiles);
+   free(rfiles);
+
+   exit(EXIT_SUCCESS);
+   return 0;
    }
 
 /*
  * execute - execute iconx to run the icon program
  */
-static novalue execute(ofile,efile,args)
+static void execute(ofile,efile,args)
 char *ofile, *efile, **args;
    {
 
@@ -788,11 +1018,14 @@ char *ofile, *efile, **args;
       ;
    p = argv = (char **)alloc((unsigned int)((n + 5) * sizeof(char *)));
 
+#if !UNIX	/* exec the file, not iconx; stderr already redirected  */
    *p++ = iconxloc;			/* set iconx pathname */
    if (efile != NULL) {			/* if -e given, copy it */
       *p++ = "-e";
       *p++ = efile;
       }
+#endif					/* UNIX */
+
    *p++ = ofile;			/* pass icode file name */
 
 #if AMIGA && LATTICE
@@ -805,19 +1038,19 @@ char *ofile, *efile, **args;
 #ifdef MSWindows
 #ifndef NTConsole
    {
-     char cmdline[256], *tmp;
+      char cmdline[256], *tmp;
 
-     strcpy(cmdline, "wiconx ");
-     if (efile != NULL) {
-       strcat(cmdline, "-e ");
-       strcat(cmdline, efile);
-       strcat(cmdline, " ");
-     }
+      strcpy(cmdline, "wiconx ");
+      if (efile != NULL) {
+         strcat(cmdline, "-e ");
+         strcat(cmdline, efile);
+         strcat(cmdline, " ");
+      }
    strcat(cmdline, ofile);
    strcat(cmdline, " ");
    while ((tmp = *args++) != NULL) {	/* copy args into argument vector */
-     strcat(cmdline, tmp);
-     strcat(cmdline, " ");
+      strcat(cmdline, tmp);
+      strcat(cmdline, " ");
    }
 
    WinExec(cmdline, SW_SHOW);
@@ -826,13 +1059,7 @@ char *ofile, *efile, **args;
 #endif					/* NTConsole */
 #endif					/* MSWindows */
 
-#if SCCX_MX
-   /* Avoids compiler warning */
-   while( (*p++ = *args++) != 0)      /* copy args into argument vector */
-#else
-   while (*p++ = *args++)       /* copy args into argument vector */
-#endif					/* SCCX_MX */
-
+   while ((*p++ = *args++) != 0)      /* copy args into argument vector */
       ;
 #endif					/* AMIGA && LATTICE */
 
@@ -862,6 +1089,23 @@ Deliberate Syntax Error
          }
       }
 #endif					/* LATTICE */
+#if __SASC
+      {
+      struct ProcID procid;
+      struct FORKENV env = { 0,       /* priority */
+                             20000,   /* stack    */
+                             NULL,    /* default stdin */
+                             NULL,    /* default stdout */
+                             NULL,    /* default console */
+                             NULL};   /* default message port */
+      env.std_in = Input();
+      env.std_out = Output();
+      if (forkv(iconxloc,argv,&env,&procid) == 0) { 
+         wait(&procid);
+         return;
+         }
+      }
+#endif					/* __SASC */
 #endif					/* AMIGA */
 
 #if ARM
@@ -931,32 +1175,18 @@ Deliberate Syntax Error
 
 #if UNIX
       /*
-       * If an ICONX environment variable is defined, use that.
-       *  If not, first try the predefined path, then search $PATH via execvp.
+       * Just execute the file.  It knows how to find iconx.
+       * First, though, must redirect stderr if requested.
        */
-      if ((argv[0] = getenv("ICONX")) != NULL && argv[0][0] != '\0') {
-         execv(argv[0], argv);	/* exec file specified by $ICONX */
-         quitf("cannot execute $ICONX (%s)", argv[0]);
+      if (efile != NULL) {
+         close(fileno(stderr));
+         if (strcmp(efile, "-") == 0)
+            dup(fileno(stdout));
+         else if (freopen(efile, "w", stderr) == NULL)
+            quitf("could not redirect stderr to %s\n", efile);
          }
-
-#ifdef HardWiredPaths
-#ifdef CRAY
-      argv[0] = "iconx";
-      execv(iconxloc, argv);
-#else					/* CRAY */
-      argv[0] = iconxloc;		/* try predefined file */
-      execv(argv[0], argv);
-#endif					/* CRAY */
-#endif					/* HardWiredPaths */
-
-      argv[0] = "iconx";
-      execvp(argv[0], argv);	/* if no iconxloc, search path for "iconx" */
-
-#ifdef HardWiredPaths
-      quitf("cannot run %s", iconxloc);
-#else					/* HardWiredPaths */
-      quitf("cannot find iconx", "");
-#endif					/* HardWiredPaths */
+      execv(ofile, argv);
+      quitf("could not execute %s", ofile);
 #endif					/* UNIX */
 
 #if VMS
@@ -971,11 +1201,11 @@ Deliberate Syntax Error
 
 #else					/* !(MACINTOSH && MPW) */
    printf("-x not supported\n");
-#endif					/* !(MACINZTOSH && MPW) */
+#endif					/* !(MACINTOSH && MPW) */
 
    }
 
-novalue report(s)
+void report(s)
 char *s;
    {
    char *c = (strchr(s, '\n') ? "" : ":\n") ;
@@ -991,11 +1221,11 @@ char *s;
  * rmfiles - remove a list of files
  */
 
-static novalue rmfiles(p)
+static void rmfiles(p)
 char **p;
    {
    for (; *p; p++) {
-      unlink(*p);
+      remove(*p);
       }
    }
 
@@ -1003,7 +1233,7 @@ char **p;
  * Print an error message if called incorrectly.  The message depends
  *  on the legal options for this system.
  */
-static novalue usage()
+static void usage()
    {
 
 #if MVS || VM
@@ -1014,7 +1244,7 @@ static novalue usage()
    fprintf(stderr,"usage: %s %s file ... [-x args]\n", progname, TUsage);
 #endif
 
-   exit(ErrorExit);
+   exit(EXIT_FAILURE);
    }
 
 
