@@ -14,7 +14,7 @@
  * Prototypes.
  */
 
-hidden	novalue	trans1		Params((char *filename));
+static	void	trans1		(char *filename);
 
 int tfatals;			/* number of fatal errors in file */
 int afatals;			/* total number of fatal errors */
@@ -63,7 +63,7 @@ char **ifiles;
 /*
  * translate one file.
  */
-static novalue trans1(filename)
+static void trans1(filename)
 char *filename;
 {
    char oname1[MaxFileName];	/* buffer for constructing file name */
@@ -146,15 +146,15 @@ char *filename;
 #endif					/* VarTran */
 
    if (tfatals) {
-      unlink(oname1);
-      unlink(oname2);
+      remove(oname1);
+      remove(oname2);
       }
    }
 
 /*
  * writecheck - check the return code from a stdio output operation
  */
-novalue writecheck(rc)
+void writecheck(rc)
 int rc;
    {
    if (rc < 0)
