@@ -8,22 +8,20 @@
 /*
  * Prototypes for static functions.
  */
-hidden struct macro     **m_find      Params((char *mname));
-hidden int                eq_id_lst   Params((struct id_lst *lst1,
-                                        struct id_lst *lst2));
-hidden int                eq_tok_lst  Params((struct tok_lst *lst1,
-                                        struct tok_lst *lst2));
-hidden int                parm_indx   Params((char *id, struct macro *m));
-hidden novalue            cpy_str     Params((char *ldelim, char *image,
-                                        char *rdelim, struct str_buf *sbuf));
-hidden struct token      *stringize   Params((struct token *trigger,
-                                        struct mac_expand *me));
-hidden struct paste_lsts *paste_parse Params((struct token *t,
-                                        struct mac_expand *me));
-hidden int               *cpy_image   Params((struct token *t, int *s));
+static struct macro     **m_find      (char *mname);
+static int            eq_id_lst   (struct id_lst *lst1, struct id_lst *lst2);
+static int            eq_tok_lst  (struct tok_lst *lst1, struct tok_lst *lst2);
+static int            parm_indx   (char *id, struct macro *m);
+static void            cpy_str     (char *ldelim, char *image,
+                                        char *rdelim, struct str_buf *sbuf);
+static struct token      *stringize   (struct token *trigger,
+                                        struct mac_expand *me);
+static struct paste_lsts *paste_parse (struct token *t,
+                                        struct mac_expand *me);
+static int               *cpy_image   (struct token *t, int *s);
 
 #define MacTblSz 149
-#define MHash(x) (((unsigned int) x) % MacTblSz)
+#define MHash(x) (((unsigned int)(unsigned long)(x)) % MacTblSz)
 
 static struct macro *m_table[MacTblSz];	/* hash table of macros */
 
@@ -95,13 +93,13 @@ struct tok_lst *lst2;
 /*
  * init_macro - initialize this module, setting up standard macros.
  */
-novalue init_macro()
+void init_macro()
    {
    int i;
    struct macro **mpp;
    struct token *t;
-   char *s;
-   char time_str[26];
+   time_t tv;
+   char *s, *tval;
    static char *time_buf;
    static char *date_buf;
    static short first_time = 1;
@@ -143,18 +141,19 @@ novalue init_macro()
     * __TIME__ and __DATE__ must be initialized to the current time and
     *  date.
     */
-   getctime(time_str);
+   time(&tv);
+   tval = ctime(&tv);
    date_buf = (char *)alloc((unsigned int)12);
    time_buf = (char *)alloc((unsigned int)9);
    s = date_buf;
    for (i = 4; i <= 10; ++i)
-      *s++ = time_str[i];
+      *s++ = tval[i];
    for (i = 20; i <= 23; ++i)
-      *s++ = time_str[i];
+      *s++ = tval[i];
    *s = '\0';
    s = time_buf;
    for (i = 11; i <= 18; ++i)
-      *s++ = time_str[i];
+      *s++ = tval[i];
    *s = '\0';
    date_buf = spec_str(date_buf);
    time_buf = spec_str(time_buf);
@@ -180,7 +179,7 @@ novalue init_macro()
 /*
  * m_install - install a macro.
  */
-novalue m_install(mname, category, multi_line, prmlst, body)
+void m_install(mname, category, multi_line, prmlst, body)
 struct token *mname;	/* name of macro */
 int multi_line;		/* flag indicating if this is a multi-line macro */
 int category;		/* # parms, or NoArgs if it is object-like macro */
@@ -230,7 +229,7 @@ struct tok_lst *body;	/* replacement list */
 /*
  * m_delete - delete a macro.
  */
-novalue m_delete(mname)
+void m_delete(mname)
 struct token *mname;
    {
    struct macro **mpp, *mp;
@@ -302,7 +301,7 @@ struct macro *m;
 /*
  * cpy_str - copy a string into a string buffer, adding delimiters.
  */
-static novalue cpy_str(ldelim, image, rdelim, sbuf)
+static void cpy_str(ldelim, image, rdelim, sbuf)
 char *ldelim;
 char *image;
 char *rdelim;
