@@ -17,25 +17,24 @@
 /*
  * prototypes for static functions.
  */
-hidden novalue add_ptr   Params((struct node *dcltor));
-hidden novalue alloc_ilc Params((int il_c_type));
-hidden novalue flush_str Params((noargs));
-hidden novalue ilc_chnl  Params((struct token *t));
-hidden novalue ilc_cnv   Params((struct node *cnv_typ, struct node *src,
-                           struct node *dflt, struct node *dest));
-hidden novalue ilc_cgoto Params((int neg, struct node *cond, word lbl));
-hidden novalue ilc_goto  Params((word lbl));
-hidden novalue ilc_lbl   Params((word lbl));
-hidden novalue ilc_ret   Params((struct token *t, int ilc_typ, struct node *n));
-hidden novalue ilc_str   Params((char *s));
-hidden novalue ilc_tok   Params((struct token *t));
-hidden novalue ilc_var   Params((struct sym_entry *sym, int just_desc,
-                           int may_mod));
-hidden novalue ilc_walk  Params((struct node *n, int may_mod, int const_cast));
-hidden novalue init_ilc  Params((noargs));
-hidden novalue insrt_str Params((noargs));
-hidden novalue new_ilc   Params((int il_c_type));
-hidden	struct	il_c	*sep_ilc	Params((char *s1,struct node *n,char *s2));
+static void add_ptr   (struct node *dcltor);
+static void alloc_ilc (int il_c_type);
+static void flush_str (void);
+static void ilc_chnl  (struct token *t);
+static void ilc_cnv   (struct node *cnv_typ, struct node *src,
+                           struct node *dflt, struct node *dest);
+static void ilc_cgoto (int neg, struct node *cond, word lbl);
+static void ilc_goto  (word lbl);
+static void ilc_lbl   (word lbl);
+static void ilc_ret   (struct token *t, int ilc_typ, struct node *n);
+static void ilc_str   (char *s);
+static void ilc_tok   (struct token *t);
+static void ilc_var   (struct sym_entry *sym, int just_desc, int may_mod);
+static void ilc_walk  (struct node *n, int may_mod, int const_cast);
+static void init_ilc  (void);
+static void insrt_str (void);
+static void new_ilc   (int il_c_type);
+static	struct	il_c	*sep_ilc	(char *s1,struct node *n,char *s2);
 
 #define SBufSz 256
 
@@ -123,7 +122,7 @@ struct sym_entry *sym;
  * add_ptr - add another level of indirection to a declarator. Print it in
  *   the output file and proceduce in-line C code.
  */
-static novalue add_ptr(dcltor)
+static void add_ptr(dcltor)
 struct node *dcltor;
    {
    while (dcltor->nd_id == ConCatNd) {
@@ -227,7 +226,7 @@ struct node *init;
  *  init_ilc - initialize the code list by pointing to ilc_base. Initialize
  *   the string buffer.
  */
-static novalue init_ilc()
+static void init_ilc()
    {
    nxt_char = 0;
    line_ref = NULL;
@@ -241,7 +240,7 @@ static novalue init_ilc()
 /*
  * - ilc_chnl - check for new-line.
  */
-static novalue ilc_chnl(t)
+static void ilc_chnl(t)
 struct token *t;
    {
    /*
@@ -259,7 +258,7 @@ struct token *t;
  * ilc_tok - convert a token to its string representation, quoting it
  *  if it is a string or character literal.
  */
-static novalue ilc_tok(t)
+static void ilc_tok(t)
 struct token *t;
    {
    char *s;
@@ -295,7 +294,7 @@ struct token *t;
 /*
  * ilc_str - append a string to the string buffer.
  */
-static novalue ilc_str(s)
+static void ilc_str(s)
 char *s;
    {
    /*
@@ -321,7 +320,7 @@ char *s;
  * insrt_str - insert the string in the buffer into the list of in-line
  *  code.
  */
-static novalue insrt_str()
+static void insrt_str()
    {
    alloc_ilc(ILC_Str);
    sbuf[nxt_char] = '\0';
@@ -333,7 +332,7 @@ static novalue insrt_str()
  * flush_str - if the string buffer is not empty, flush it to the list
  *  of in-line code.
  */
-static novalue flush_str()
+static void flush_str()
    {
    if (insert_nl)
       ilc_str("");
@@ -346,7 +345,7 @@ static novalue flush_str()
  *   is called for non-string elements. If necessary it flushes the
  *   string buffer to another element first.
  */
-static novalue new_ilc(il_c_type)
+static void new_ilc(il_c_type)
 int il_c_type;
    {
    flush_str();
@@ -357,7 +356,7 @@ int il_c_type;
  * alloc_ilc - allocate a new element for the list of in-line C code
  *   and add it to the list.
  */
-static novalue alloc_ilc(il_c_type)
+static void alloc_ilc(il_c_type)
 int il_c_type;
    {
    int i;
@@ -404,7 +403,7 @@ char *s2;
 /*
  * ilc_var - create in-line C code for a variable in the symbol table.
  */
-static novalue ilc_var(sym, just_desc, may_mod)
+static void ilc_var(sym, just_desc, may_mod)
 struct sym_entry *sym;
 int just_desc;
 int may_mod;
@@ -494,7 +493,7 @@ int may_mod;
  *   code. This function needs to know if the code is in a modifying context,
  *   such as the left-hand-side of an assignment.
  */
-static novalue ilc_walk(n, may_mod, const_cast)
+static void ilc_walk(n, may_mod, const_cast)
 struct node *n;
 int may_mod;
 int const_cast;
@@ -723,7 +722,7 @@ int const_cast;
                break;
             case Struct:
             case Union:
-            case Enum:
+            case TokEnum:
                /*
                 * <struct-union-enum> <identifier>
                 * <struct-union-enum> { <component-list> }
@@ -1087,7 +1086,7 @@ int const_cast;
 /*
  * ilc_cnv - produce code for a cnv: or def: statement.
  */
-static novalue ilc_cnv(cnv_typ, src, dflt, dest)
+static void ilc_cnv(cnv_typ, src, dflt, dest)
 struct node *cnv_typ;
 struct node *src;
 struct node *dflt;
@@ -1153,7 +1152,7 @@ struct node *dest;
 /*
  * ilc_ret - produce in-line code for suspend/return statement.
  */
-static novalue ilc_ret(t, ilc_typ, n)
+static void ilc_ret(t, ilc_typ, n)
 struct token *t;
 int ilc_typ;
 struct node *n;
@@ -1282,7 +1281,7 @@ struct node *n;
 /*
  * ilc_goto - produce in-line C code for a goto to a numbered label.
  */
-static novalue ilc_goto(lbl)
+static void ilc_goto(lbl)
 word lbl;
    {
    insert_nl = 1;
@@ -1296,7 +1295,7 @@ word lbl;
  * ilc_cgoto - produce in-line C code for a conditional goto to a numbered
  *  label. The condition may be negated.
  */
-static novalue ilc_cgoto(neg, cond, lbl)
+static void ilc_cgoto(neg, cond, lbl)
 int neg;
 struct node *cond;
 word lbl;
@@ -1316,7 +1315,7 @@ word lbl;
 /*
  * ilc_lbl - produce in-line C code for a numbered label.
  */
-static novalue ilc_lbl(lbl)
+static void ilc_lbl(lbl)
 word lbl;
    {
    insert_nl = 1;
@@ -1332,7 +1331,7 @@ word lbl;
  *  was encountered and that it is consistent with this return,
  *  suspend, or fail.
  */
-novalue chkabsret(tok, ret_typ)
+void chkabsret(tok, ret_typ)
 struct token *tok;
 int ret_typ;
    {
@@ -1363,7 +1362,7 @@ int ret_typ;
  * just_type - strip non-type information from a type-qualifier list. Print
  *   it in the output file and if ilc is set, produce in-line C code.
  */
-novalue just_type(typ, indent, ilc)
+void just_type(typ, indent, ilc)
 struct node *typ;
 int indent;
 int ilc;
@@ -1381,7 +1380,7 @@ int ilc;
          case Extern:
          case Static:
          case Auto:
-         case Register:
+         case TokRegister:
          case Const:
          case Volatile:
             return;         /* Don't output these declaration elements */
