@@ -5,13 +5,12 @@
  *  are valid tokens, and converts preprocessor constants into the
  *  various C constants.
  */
-#include <ctype.h>
 #include "rtt.h"
 
 /*
  * Prototype for static function.
  */
-hidden int int_suffix Params((char *s));
+static int int_suffix (char *s);
 
 int lex_state = DfltLex;
 
@@ -132,7 +131,7 @@ static int good_char[128] = {
 /*
  * init_lex - initialize lexical analyzer.
  */
-novalue init_lex()
+void init_lex()
    {
    struct sym_entry *sym;
    int i;
@@ -147,24 +146,24 @@ novalue init_lex()
       sym_add(Auto,          spec_str("auto"),          OtherDcl, 0);
       sym_add(Break,         spec_str("break"),         OtherDcl, 0);
       sym_add(Case,          spec_str("case"),          OtherDcl, 0);
-      sym_add(Char,          spec_str("char"),          OtherDcl, 0);
+      sym_add(TokChar,       spec_str("char"),          OtherDcl, 0);
       sym_add(Const,         spec_str("const"),         OtherDcl, 0);
       sym_add(Continue,      spec_str("continue"),      OtherDcl, 0);
       sym_add(Default,       spec_str("default"),       OtherDcl, 0);
       sym_add(Do,            spec_str("do"),            OtherDcl, 0);
       sym_add(Doubl,         spec_str("double"),        OtherDcl, 0);
       sym_add(Else,          spec_str("else"),          OtherDcl, 0);
-      sym_add(Enum,          spec_str("enum"),          OtherDcl, 0);
+      sym_add(TokEnum,       spec_str("enum"),          OtherDcl, 0);
       sym_add(Extern,        spec_str("extern"),        OtherDcl, 0);
       sym_add(Float,         spec_str("float"),         OtherDcl, 0);
       sym_add(For,           spec_str("for"),           OtherDcl, 0);
       sym_add(Goto,          spec_str("goto"),          OtherDcl, 0);
       sym_add(If,            spec_str("if"),            OtherDcl, 0);
       sym_add(Int,           spec_str("int"),           OtherDcl, 0);
-      sym_add(Long,          spec_str("long"),          OtherDcl, 0);
-      sym_add(Register,      spec_str("register"),      OtherDcl, 0);
+      sym_add(TokLong,       spec_str("long"),          OtherDcl, 0);
+      sym_add(TokRegister,   spec_str("register"),      OtherDcl, 0);
       sym_add(Return,        spec_str("return"),        OtherDcl, 0);
-      sym_add(Short,         spec_str("short"),         OtherDcl, 0);
+      sym_add(TokShort,      spec_str("short"),         OtherDcl, 0);
       sym_add(Signed,        spec_str("signed"),        OtherDcl, 0);
       sym_add(Sizeof,        spec_str("sizeof"),        OtherDcl, 0);
       sym_add(Static,        spec_str("static"),        OtherDcl, 0);
@@ -197,7 +196,7 @@ novalue init_lex()
       sym_add(Errorfail,     spec_str("errorfail"),     OtherDcl, 0);
       sym_add(Exact,         spec_str("exact"),         OtherDcl, 0);
       sym_add(Fail,          spec_str("fail"),          OtherDcl, 0);
-      sym_add(Function,      spec_str("function"),      OtherDcl, 0);
+      sym_add(TokFunction,   spec_str("function"),      OtherDcl, 0);
       sym_add(Inline,        spec_str("inline"),        OtherDcl, 0);
       sym_add(Is,            spec_str("is"),            OtherDcl, 0);
       sym_add(Keyword,       spec_str("keyword"),       OtherDcl, 0);
@@ -215,7 +214,7 @@ novalue init_lex()
       sym_add(Then,          spec_str("then"),          OtherDcl, 0);
       sym_add(Tmp_cset,      spec_str("tmp_cset"),      OtherDcl, 0);
       sym_add(Tmp_string,    spec_str("tmp_string"),    OtherDcl, 0);
-      sym_add(Type,          spec_str("type"),          OtherDcl, 0);
+      sym_add(TokType,       spec_str("type"),          OtherDcl, 0);
       sym_add(Type_case,     spec_str("type_case"),     OtherDcl, 0);
       sym_add(Underef,       spec_str("underef"),       OtherDcl, 0);
       sym_add(Variable,      spec_str("variable"),      OtherDcl, 0);
@@ -396,7 +395,7 @@ int yylex()
       return yylex();
       }
    else if (lex_state == OpHead && yylval.t->tok_id != '}' &&
-     GoodChar(yylval.t->image[0])) {
+     GoodChar((int)yylval.t->image[0])) {
       /*
        * This should be the operator symbol in the header of an operation
        *  declaration. Concatenate all operator symbols into one token
@@ -406,7 +405,7 @@ int yylex()
       for (s = yylval.t->image; *s != '\0'; ++s)
          AppChar(*sbuf, *s);
       lk_ahead = preproc();
-      while (lk_ahead != NULL && GoodChar(lk_ahead->image[0])) {
+      while (lk_ahead != NULL && GoodChar((int)lk_ahead->image[0])) {
          for (s = lk_ahead->image; *s != '\0'; ++s)
             AppChar(*sbuf, *s);
          free_t(lk_ahead);
