@@ -1,6 +1,8 @@
 #include "rtt.h"
 
+#if !NT
 #include "../h/filepat.h"		/* added for filepat change */
+#endif					/* NT */
 
 #if MACINTOSH
 #include <console.h>
@@ -12,7 +14,7 @@ unsigned _stack = 100000;
 /*
  * prototypes for static functions.
  */
-hidden novalue add_tdef Params((char *name));
+static void add_tdef (char *name);
 
 /*
  * refpath is used to locate the standard include files for the Icon.
@@ -68,8 +70,8 @@ Deliberate Syntax Error
 #endif					/* ATARI_ST ... */
 
 #if AMIGA
-char *grttin_path = "/h/grttin.h";
-char *rt_path = "/h/rt.h";
+char *grttin_path = "/src/h/grttin.h";
+char *rt_path = "/src/h/rt.h";
 #endif					/* AMIGA */
 
 #if MACINTOSH
@@ -134,7 +136,7 @@ static char *curlst_nm = "rttcur.lst";
 static FILE *curlst;
 static char *cur_src;
 
-extern line_cntrl;
+extern int line_cntrl;
 
 /*
  * tdefnm is used to construct a list of identifiers that
@@ -155,7 +157,7 @@ static struct tdefnm *tdefnm_lst = NULL;
 /*
  * getopt() variables
  */
-extern int optindex;		/* index into parent argv vector */
+extern int optind;		/* index into parent argv vector */
 extern int optopt;		/* character checked for validity */
 extern char *optarg;		/* argument associated with option */
 
@@ -178,14 +180,6 @@ int ConsolePause = 1;
 int rtt(int argc, char **argv);
 #define int_PASCAL int PASCAL
 #define LRESULT_CALLBACK LRESULT CALLBACK
-#undef Type
-#undef Short
-#undef Long
-#undef Char
-#undef Equal
-#undef Function
-#undef Enum
-#undef Register
 #undef Reset
 #include <windows.h>
 #include "../wincap/dibutil.h"
@@ -217,9 +211,9 @@ int CmdParamToArgv(char *s, char ***avp)
    return rv;
    }
 
-LRESULT_CALLBACK WndProc	Params((HWND, UINT, WPARAM, LPARAM));
+LRESULT_CALLBACK WndProc	(HWND, UINT, WPARAM, LPARAM);
 
-novalue MSStartup(int argc, char **argv, HINSTANCE hInstance, HINSTANCE hPrevInstance)
+void MSStartup(int argc, char **argv, HINSTANCE hInstance, HINSTANCE hPrevInstance)
    {
    WNDCLASS wc;
    if (!hPrevInstance) {
@@ -371,7 +365,7 @@ char **argv;
    if (!iconx_flg) {
       fprintf(stdout,
          "rtt was compiled to only support the intepreter, use -x\n");
-      exit(ErrorExit);
+      exit(EXIT_FAILURE);
       }
 #endif					/* Rttx */
 
@@ -393,7 +387,7 @@ char **argv;
    /*
     * At least one file name must be given on the command line.
     */
-   if (optindex == argc)
+   if (optind == argc)
      show_usage();
 
    /*
@@ -423,29 +417,29 @@ char **argv;
    /*
     * Scan file name arguments, and translate the files.
     */
-   while (optindex < argc)  {
+   while (optind < argc)  {
 
 #if MVS
-      src_file_nm = argv[optindex];
+      src_file_nm = argv[optind];
 #endif                                  /* MVS */
 
 #if PatternMatch
       FINDDATA_T fd;
 
-      if (!FINDFIRST(argv[optindex], &fd)) {
-         fprintf(stderr,"File %s: no match\n", argv[optindex]);
+      if (!FINDFIRST(argv[optind], &fd)) {
+         fprintf(stderr,"File %s: no match\n", argv[optind]);
 	 fflush(stderr);
-	 exit(ErrorExit);
+	 exit(EXIT_FAILURE);
          }
       do {
-      argv[optindex] = FILENAME(&fd);
+      argv[optind] = FILENAME(&fd);
 #endif					/* PatternMatch */
-      trans(argv[optindex]);
+      trans(argv[optind]);
 #if PatternMatch
       } while (FINDNEXT(&fd));
       FINDCLOSE(&fd);
 #endif					/* PatternMatch */
-      optindex++;
+      optind++;
       }
 
 #ifndef Rttx
@@ -464,13 +458,13 @@ char **argv;
       }
 #endif					/* Rttx */
 
-   return NormalExit;
+   return EXIT_SUCCESS;
    }
 
 /*
  * trans - translate a source file.
  */
-novalue trans(src_file)
+void trans(src_file)
 char *src_file;
    {
    char *cname;
@@ -480,9 +474,8 @@ char *src_file;
    struct fileparts *fp;
    struct tdefnm *td;
    struct token *t;
-   static char *standardpp = "#define StandardPP\n";
    static char *test_largeints = "#ifdef LargeInts\nyes\n#endif\n";
-   static first_time = 1;
+   static int first_time = 1;
 
    cur_src = src_file;
 
@@ -492,7 +485,6 @@ char *src_file;
     */
    enable_out = 0;
    init_preproc(in_header, opt_lst, opt_args);
-   str_src("<rtt initialization>", standardpp, (int)strlen(standardpp));
    str_src("<rtt initialization>", compiler_def, (int)strlen(compiler_def));
    init_sym();
    for (td = tdefnm_lst; td != NULL; td = td->next)
@@ -593,7 +585,7 @@ char *src_file;
 /*
  * add_tdef - add identifier to list of typedef names.
  */
-static novalue add_tdef(name)
+static void add_tdef(name)
 char *name;
    {
    struct tdefnm *td;
@@ -608,7 +600,7 @@ char *name;
  * Add name of file to the output list, and if it contains "interesting"
  *  code, add it to the dependency list in the data base.
  */
-novalue put_c_fl(fname, keep)
+void put_c_fl(fname, keep)
 char *fname;
 int keep;
    {
@@ -630,16 +622,16 @@ int keep;
 /*
  * Print an error message if called incorrectly.
  */
-novalue show_usage()
+void show_usage()
    {
    fprintf(stderr, "usage: %s %s\n", progname, options);
-   exit(ErrorExit);
+   exit(EXIT_FAILURE);
    }
 
 /*
  * yyerror - error routine called by yacc.
  */
-novalue yyerror(s)
+void yyerror(s)
 char *s;
    {
    struct token *t;
