@@ -26,9 +26,10 @@
 /*
  * Prototypes for static functions.
  */
-hidden novalue cstos Params((unsigned int *cs, dptr dp, char *s));
-hidden novalue itos  Params((C_integer num, dptr dp, char *s));
-hidden int     ston  Params((dptr sp, union numeric *result));
+static void cstos (unsigned int *cs, dptr dp, char *s);
+static void itos (C_integer num, dptr dp, char *s);
+static int ston (dptr sp, union numeric *result);
+static int tmp_str (char *sbuf, dptr s, dptr d);
 
 /*
  * cnv_c_dbl - cnv:C_double(*s, *d), convert a value directly into a C double
@@ -318,7 +319,6 @@ dptr s, d;
    struct descrip cnvstr;			/* not tended */
    char sbuf[MaxCvtLen];
    union numeric numrc;
-   int status;
 
    type_case *s of {
       integer: {
@@ -603,7 +603,7 @@ dptr d;
 /*
  * deref - dereference a descriptor.
  */
-novalue deref(s, d)
+void deref(s, d)
 dptr s, d;
    {
    /*
@@ -669,7 +669,7 @@ dptr s, d;
 /*
  * tmp_str - Convert to temporary string.
  */
-int tmp_str(sbuf, s, d)
+static int tmp_str(sbuf, s, d)
 char *sbuf;
 dptr s;
 dptr d;
@@ -728,7 +728,6 @@ dptr s;
 C_integer arity;
    {
    C_integer i;
-   char *fnc;
    struct pstrnm *pp;
 
    if (!StrLen(*s))
@@ -771,7 +770,6 @@ dptr s;
 C_integer arity;
    {
    C_integer i;
-   char *fnc;
 
    /*
     * See if the string is the name of a global variable.
@@ -795,7 +793,7 @@ C_integer arity;
  *  making q a descriptor for the resulting string.
  */
 
-static novalue itos(num, dp, s)
+static void itos(num, dp, s)
 C_integer num;
 dptr dp;
 char *s;
@@ -1120,50 +1118,14 @@ register long len;
  * rtos - convert the real number n into a string using s as a buffer and
  *  making a descriptor for the resulting string.
  */
-novalue rtos(n, dp, s)
+void rtos(n, dp, s)
 double n;
 dptr dp;
 char *s;
    {
+   s++; 				/* leave room for leading zero */
+   sprintf(s, "%.*g", Precision, n + 0.0);  /* format string; +0.0 avoids -0 */
 
-   s++; 			/* leave room for leading zero */
-/*
- * The following code is operating-system dependent [@rconv.01]. Convert real
- *  number to string.
- *
- * If IconGcvt is defined, icon_gcvt() is actually called, due to a #define
- *  in config.h.
- */
-
-#if PORT
-   gcvt(n, Precision, s);
-Deliberate Syntax Error
-#endif					/* PORT */
-
-#if HIGHC_386
-   sprintf(s,"%.*g", Precision, n);
-#else					/* HIGHC_386 */
-#if AMIGA || ARM || ATARI_ST || MSDOS || OS2 || UNIX || VMS
-   gcvt(n, Precision, s);
-#endif                                  /* AMIGA || ARM || ATARI_ST || ... */
-#endif					/* HIGHC_386 */
-
-#if MACINTOSH
-   sprintf(s,"%.20g",n);
-#endif					/* MACINTOSH */
-
-#if VM || MVS
-   sprintf(s,"%.*g", Precision, n);
-   {
-     char *ep = strstr(s, "e+");
-     if (ep) memmove(ep+1, ep+2, strlen(ep+2)+1);
-   }
-#endif					/* VM || MVS */
-
-/*
- * End of operating-system specific code.
- */
-   
    /*
     * Now clean up possible messes.
     */
@@ -1173,9 +1135,7 @@ Deliberate Syntax Error
       s--;
       *s = '0';
       }
-   else if (strcmp(s, "-0.0") == 0)	/* negative zero */
-      s++;
-   else if (!index(s, '.') && !index(s,'e') && !index(s,'E'))
+   else if (!strchr(s, '.') && !strchr(s,'e') && !strchr(s,'E'))
          strcat(s, ".0");		/* if no decimal point or exp. */
    if (s[strlen(s) - 1] == '.')		/* if decimal point is at end ... */
       strcat(s, "0");
@@ -1188,7 +1148,7 @@ Deliberate Syntax Error
  *  s as a buffer and making a descriptor for the resulting string.
  */
 
-static novalue cstos(cs, dp, s)
+static void cstos(cs, dp, s)
 unsigned int *cs;
 dptr dp;
 char *s;
