@@ -346,8 +346,15 @@ struct errtab errtab[] = {
    144, "window system error",
    145, "bad window attribute",
    146, "incorrect number of arguments to drawing function",
+   147, "window attribute cannot be read or written as requested",
 #endif					/* Graphics */
 
+#ifdef MSWindows
+#ifdef FAttrib
+   160, "bad file attribute",
+#endif					/* FAttrib */
+#endif					/* MSWindows */
+
    201, "division by zero",
    202, "remaindering by zero",
    203, "integer overflow",
@@ -400,7 +407,7 @@ struct errtab errtab[] = {
  */
 
 #if !COMPILER
-#define OpDef(p,n,s,u) int Cat(O,p) Params((dptr cargp));
+#define OpDef(p,n,s,u) int Cat(O,p) (dptr cargp);
 #include "../h/odefs.h"
 #undef OpDef
 
@@ -433,7 +440,7 @@ int (*optab[])() = {
 /*
  *  Keyword function look-up table.
  */
-#define KDef(p,n) int Cat(K,p) Params((dptr cargp));
+#define KDef(p,n) int Cat(K,p) (dptr cargp);
 #include "../h/kdefs.h"
 #undef KDef
 
