@@ -197,7 +197,7 @@ function{1} copy(x)
             dst->table.mask = src->table.mask;
             dst->table.defvalue = src->table.defvalue;
             for (i = 0; i < HSegs && src->table.hdir[i] != NULL; i++)
-               memcopy((char *)dst->table.hdir[i], (char *)src->table.hdir[i],
+               memcpy((char *)dst->table.hdir[i], (char *)src->table.hdir[i],
                   src->table.hdir[i]->blksize);
             /*
              * Work down the chain of element blocks in each bucket
@@ -357,8 +357,9 @@ function{1} display(i,f)
       else if (i > k_level)
          i = k_level;
 
-      fprintf(std_f,"co-expression_%ld(%ld)\n\n",BlkLoc(k_current)->coexpr.id,
-         BlkLoc(k_current)->coexpr.size);
+      fprintf(std_f,"co-expression_%ld(%ld)\n\n",
+         (long)BlkLoc(k_current)->coexpr.id,
+	 (long)BlkLoc(k_current)->coexpr.size);
       fflush(std_f);
 #ifdef MultiThread
       if (ce) {
@@ -369,7 +370,7 @@ function{1} display(i,f)
        }
       else
 #endif						/* MultiThread */
-         r = xdisp(pfp, argp, (int)i, std_f);
+         r = xdisp(pfp, glbl_argp, (int)i, std_f);
       if (r == Failed)
          runerr(305);
       return nulldesc;
@@ -512,7 +513,6 @@ function{1} image(x)
       return string
       }
    inline {
-      register int i;
       if (getimage(&x,&result) == Error)
           runerr(0);
       return result;
@@ -665,7 +665,7 @@ function{1,*} seq(from, by)
       return integer
       }
    body {
-      word seq_lb, seq_ub;
+      word seq_lb = 0, seq_ub = 0;
 
       /*
        * Produce error if by is 0, i.e., an infinite sequence of from's.
@@ -681,7 +681,6 @@ function{1,*} seq(from, by)
       else if (by == 0) {
          irunerr(211, by);
          errorfail;
-
          }
 
       /*
@@ -1246,7 +1245,7 @@ end
  * nthcmp(d1,d2) - compare two descriptors on their nth fields.
  */
 word sort_field;		/* field number, set by sort function */
-static dptr nth Params((dptr d));
+static dptr nth (dptr d);
 
 int nthcmp(d1,d2)
 dptr d1, d2;
@@ -1393,21 +1392,19 @@ function{0,1} variable(s)
       }
 
    body {
-      register dptr dp;
-      register dptr np;
       register int rv;
 
 #ifdef MultiThread
       struct progstate *prog, *savedprog;
       struct pf_marker *tmp_pfp = pfp;
-      dptr tmp_argp = argp;
+      dptr tmp_argp = glbl_argp;
 
       savedprog = curpstate;
       if (!is:null(c)) {
 	 if (is:coexpr(c)) {
 	    prog = BlkLoc(c)->coexpr.program;
 	    pfp = BlkLoc(c)->coexpr.es_pfp;
-	    argp = BlkLoc(c)->coexpr.es_argp;
+	    glbl_argp = BlkLoc(c)->coexpr.es_argp;
 	    ENTERPSTATE(prog);
 	    }
 	 else {
@@ -1425,7 +1422,7 @@ function{0,1} variable(s)
 
       while (i--) {
 	 if (pfp == NULL) fail;
-	 argp = pfp->pf_argp;
+	 glbl_argp = pfp->pf_argp;
 	 pfp = pfp->pf_pfp;
          }
 #endif						/* MultiThread */
@@ -1436,7 +1433,7 @@ function{0,1} variable(s)
       if (is:coexpr(c)) {
 	 ENTERPSTATE(savedprog);
 	 pfp = tmp_pfp;
-	 argp = tmp_argp;
+	 glbl_argp = tmp_argp;
 
 	 if ((rv == LocalName) || (rv == StaticName)) {
 	    Deref(result);
@@ -1843,9 +1840,11 @@ function{1} load(s,arglist,infile,outfile,errfile,
        */
       if (!is:null(arglist)) {
          PushDesc(arglist);
+	 pstate->Glbl_argp = (dptr)(sp - 1);
          }
       else {
          PushNull;
+	 pstate->Glbl_argp = (dptr)(sp - 1);
          {
          dptr tmpargp = (dptr) (sp - 1);
          Ollist(0, tmpargp);
@@ -1975,6 +1974,9 @@ function{*} keyword(keyname,ce)
 	 ENTERPSTATE(savedp);
 	 return C_integer i;
 	 }
+      else if (strcmp(kname,"current") == 0) {
+	 return p->K_current;
+	 }
       else if (strcmp(kname,"error") == 0) {
 	 return kywdint(&(p->Kywd_err));
 	 }
@@ -2060,9 +2062,12 @@ function{*} keyword(keyname,ce)
 	 return C_integer allRegions;
 	 }
       else if (strcmp(kname,"source") == 0) {
+	 return coexpr(topact((struct b_coexpr *)BlkLoc(BlkLoc(d)->coexpr.program->K_current)));
+/*
 	 if (BlkLoc(d)->coexpr.es_actstk)
 	    return coexpr(topact((struct b_coexpr *)BlkLoc(d)));
 	 else return BlkLoc(d)->coexpr.program->parent->K_main;
+*/
 	 }
       else if (strcmp(kname,"storage") == 0) {
 	 word allRegions = 0;
