@@ -16,7 +16,7 @@
  * Prototypes.
  */
 
-novalue mmrefresh		Params((noargs));
+void mmrefresh		(void);
 
 #define evforget()
 
@@ -45,7 +45,7 @@ static char scopechars[] = "+:^-";
 /*
  * Special event function for E_Assign; allocates out of monitor's heap.
  */
-novalue EVAsgn(dx)
+void EVAsgn(dx)
 dptr dx;
 {
    int i;
@@ -56,7 +56,7 @@ dptr dx;
 #if COMPILER
    procname = &(PFDebug(*pfp)->proc->pname);
 #else					/* COMPILER */
-   procname = &((&BlkLoc(*argp)->proc)->pname);
+   procname = &((&BlkLoc(*glbl_argp)->proc)->pname);
 #endif					/* COMPILER */
    /*
     * call get_name, allocating out of the monitor if necessary.
@@ -205,8 +205,9 @@ function{0,1} EvGet(cs,flag)
 
 	 switch(*StrLoc(curpstate->eventcode)) {
 	 case E_Cofail: case E_Coret: {
-	    if (BlkLoc(curpstate->eventval)->coexpr.id == 1)
+	    if (BlkLoc(curpstate->eventsource)->coexpr.id == 1) {
 	       fail;
+	       }
 	    }
 	    }
 
@@ -221,7 +222,7 @@ end
  *  EVInit() - initialization.
  */
 
-novalue EVInit()
+void EVInit()
    {
    int i;
 
@@ -287,7 +288,7 @@ novalue EVInit()
  * mmrefresh() - redraw screen, initially or after garbage collection.
  */
 
-novalue mmrefresh()
+void mmrefresh()
    {
    char *p;
    word n;
@@ -308,5 +309,5 @@ novalue mmrefresh()
 
 
 #else					/* EventMon */
-static char x;			/* avoid empty module */
+static char xjunk;			/* avoid empty module */
 #endif					/* EventMon */
