@@ -583,7 +583,7 @@ function{1} repl(s,n)
        */
       sloc = StrLoc(s);
       if (slen == 1)
-         memfill(resloc, *sloc, size);
+         memset(resloc, *sloc, size);
       else {
          while (--n >= 0) {
             floc = sloc;
