@@ -1,8 +1,7 @@
 /*
  * File: fsys.r
  *  Contents: close, chdir, exit, getenv, open, read, reads, remove, rename,
- *  [save], seek, stop, [system], where, write, writes, [getch, getche,
- *  kbhit]
+ *  [save], seek, stop, system, where, write, writes, [getch, getche, kbhit]
  */
 
 #if MICROSOFT || SCO_XENIX
@@ -22,9 +21,23 @@
 Deliberate Syntax Error
 #endif					/* PORT */
 
-#if AMIGA || ATARI_ST || MSDOS || MVS || OS2 || UNIX || VM || VMS
+#if AMIGA
+#if __SASC
+   /* Undefine macros from fcntl.h and stdlib.h that collide with
+      function names in this module. */
+#passthru #undef close
+#passthru #undef open
+#passthru #undef read
+#passthru #undef write
+#passthru #undef getenv
+#endif __SASC                           /* SASC  */
+extern FILE *popen(const char *, const char *);
+extern int pclose(FILE *);
+#endif AMIGA                            /* AMIGA */
+
+#if ATARI_ST || MSDOS || MVS || OS2 || UNIX || VM || VMS
    /* nothing to do */
-#endif					/* AMIGA || ATARI_ST || ... */
+#endif					/* ATARI_ST || MSDOS ... */
 
 #if MACINTOSH && MPW
 extern int MPWFlush(FILE *f);
@@ -36,6 +49,101 @@ extern int MPWFlush(FILE *f);
  */
 
 
+#ifdef MSWindows
+#ifdef FAttrib
+
+"fattrib(str, att) - get the attribute of a file "
+
+function{*} fattrib (fname, att[argc])
+ 
+   if !cnv:C_string(fname) then
+        runerr(103, fname)
+
+   abstract {
+      return string ++ integer
+      }  
+
+   body {
+      tended char *s;
+      struct stat fs;
+#if UNIX
+      struct passwd *pwnam;
+      struct group  *grnam;
+#endif					/* UNIX */
+#if NT
+      HFILE hf;
+      OFSTRUCT of;
+      FILETIME ft1,ft2,ft3;
+      SYSTEMTIME st;
+#endif					/* NT */
+      int fd, i;
+      char *retval;
+      char *temp;
+      long l;
+   
+         if ( stat(fname, &fs) == -1 )
+            fail;
+	 for(i=0; i<argc; i++) {
+	    if (!cnv:C_string(att[i], s)) {
+	       runerr(103, att[i]);
+	       }
+         if ( !strcasecmp("size", s) ) {
+            suspend C_integer(fs.st_size);
+            }
+         else if ( !strcasecmp("status", s) ) {
+            temp = make_mode (fs.st_mode);
+	    l = strlen(temp);
+            Protect(retval = alcstr(temp,l), runerr(0));
+	    free(temp);
+            suspend string(l, retval); 
+            }
+#if UNIX
+         else if ( !strcasecmp("owner", s) ) {
+            pwnam = getpwuid (fs.st_uid);
+	    temp = pwnam->pw_name;
+	    l = strlen(temp);
+	    Protect(temp = alcstr(temp, l), runerr(0));
+            suspend string (l, temp);
+            }
+         else if ( !strcasecmp("group", s) ) {
+            grnam = getgrgid (fs.st_gid);
+	    temp = grnam->gr_name;
+	    l = strlen(temp);
+	    Protect(temp = alcstr(temp, l), runerr(0));
+            suspend string (l, temp);
+            }
+#endif					/* UNIX */
+         else if ( !strcasecmp("m_time", s) ) {
+	    temp = ctime(&(fs.st_mtime));
+	    l = strlen(temp);
+  	    if (temp[l-1] == '\n') l--;
+	    Protect(temp = alcstr(temp, l), runerr(0));
+            suspend string(l, temp);
+            }
+         else if ( !strcasecmp("a_time", s) ) {
+	    temp = ctime(&(fs.st_atime));
+	    l = strlen(temp);
+  	    if (temp[l-1] == '\n') l--;
+	    Protect(temp = alcstr(temp, l), runerr(0));
+            suspend string(l, temp);
+            }
+         else if ( !strcasecmp("c_time", s) ) {
+	    temp = ctime(&(fs.st_ctime));
+	    l = strlen(temp);
+  	    if (temp[l-1] == '\n') l--;
+	    Protect(temp = alcstr(temp, l), runerr(0));
+            suspend string(l, temp);
+            }
+         else {
+            runerr(160);
+            }
+      }
+      fail;
+   }
+end
+#endif					/* FAttrib */
+#endif					/* MSWindows */
+
 "close(f) - close file f."
 
 function{1} close(f)
@@ -74,7 +182,13 @@ function{1} close(f)
       else
 #endif					/* Graphics */
 
-#if ARM || OS2 || UNIX || VMS
+#ifdef ReadDirectory
+      if (BlkLoc(f)->file.status & Fs_Directory)
+         closedir((DIR*) fp);
+      else
+#endif					/* ReadDirectory */
+
+#if AMIGA || ARM || OS2 || UNIX || VMS
       /*
        * Close pipe if pipes are supported.
        */
@@ -84,7 +198,7 @@ function{1} close(f)
 	 return C_integer((pclose(fp) >> 8) & 0377);
 	 }
       else
-#endif					/* ARM || OS2 || UNIX || VMS */
+#endif					/* AMIGA || ARM || OS2 || ... */
 
       fclose(fp);
       BlkLoc(f)->file.status = 0;
@@ -102,7 +216,7 @@ end
 "exit(i) - exit process with status i, which defaults to 0."
 
 function{} exit(status)
-   if !def:C_integer(status, NormalExit) then
+   if !def:C_integer(status, EXIT_SUCCESS) then
       runerr(101, status)
    inline {
       c_exit((int)status);
@@ -112,15 +226,6 @@ end
 
 "getenv(s) - return contents of environment variable s."
 
-#ifndef EnvVars
-function{0} getenv(s)
-   abstract {
-      return empty_type
-      }
-   inline {
-      fail;
-      }
-#else					/* EnvVars */
 function{0,1} getenv(s)
 
    /*
@@ -145,12 +250,11 @@ function{0,1} getenv(s)
 	 fail;
 
       }
-#endif					/* EnvVars */
 end
 
 
 #ifdef Graphics
-"open(s1, s2, ...) - open file named s2 with options s2"
+"open(s1, s2, ...) - open file named s1 with options s2"
 " and attributes given in trailing arguments."
 function{0,1} open(fname, spec, attr[n])
 #else						/* Graphics */
@@ -218,25 +322,15 @@ function{0,1} open(fname, spec)
 Deliberate Syntax Error
 #endif					/* PORT */
 
-#if AMIGA || MACINTOSH
+#if AMIGA || ATARI_ST || MACINTOSH || MSDOS || MVS || OS2 || VM
    /* nothing is needed */
-#endif					/* AMIGA || MACINTOSH */
+#endif					/* AMIGA || ATARI_ST || ... */
 
 #if ARM
       extern FILE *popen(const char *, const char *);
       extern int pclose(FILE *);
 #endif					/* ARM */
 
-#if ATARI_ST || MSDOS || MVS || OS2 || VM
-      char untranslated;
-#endif					/* ATARI_ST || MSDOS || ... */
-
-#if MACINTOSH
-#if LSC
-      char untranslated;
-#endif					/* LSC */
-#endif					/* MACINTOSH */
-
 #if OS2 || UNIX || VMS
       extern FILE *popen();
 #endif					/* OS2 || UNIX || VMS */
@@ -253,38 +347,6 @@ Deliberate Syntax Error
 
       status = 0;
 
-/*
- * The following code is operating-system dependent [@fsys.03].  Provide
- *  declaration for untranslated line-termination mode, if supported.
- */
-
-#if PORT
-   /* nothing to do */
-Deliberate Syntax Error
-#endif					/* PORT */
-
-#if AMIGA
-   /* translated mode could be supported, but is not now */
-#endif					/* AMIGA */
-
-#if ARM || UNIX || VMS
-   /* nothing to do */
-#endif					/* ARM || UNIX || VMS */
-
-#if ATARI_ST || MSDOS || MVS || OS2 || VM
-      untranslated = 0;
-#endif					/* ATARI_ST || MSDOS || ... */
-
-#if MACINTOSH
-#if LSC
-      untranslated = 0;
-#endif					/* LSC */
-#endif					/* MACINTOSH */
-
-/*
- * End of operating-system specific code.
- */
-
       /*
        * Scan spec, setting appropriate bits in status.  Produce a
        *  run-time error if an unknown character is encountered.
@@ -314,100 +376,36 @@ Deliberate Syntax Error
 	       status |= Fs_Write;
 	       continue;
 
-/*
- * The following code is operating-system dependent [@fsys.04].  Handle
- * untranslated line-terminator mode, pipes, and/or window modes if supported.
- */
-
-#if PORT
-	    case 't':
-	    case 'T':
-	    case 'u':
-	    case 'U':
-	       continue;			/* no-op */
-Deliberate Syntax Error
-#endif					/* PORT */
-
-#if AMIGA
-	    case 't':
-	    case 'T':
-	    case 'u':
-	    case 'U':
-	       continue;			/* no-op */
-#endif					/* AMIGA */
-
-#if ARM || UNIX || VMS
-	    case 't':
-	    case 'T':
-	    case 'u':
-	    case 'U':
-	       continue;			/* no-op */
-	    case 'p':
-	    case 'P':
-	       status |= Fs_Pipe;
-	       continue;
-#endif					/* ARM || UNIX || VMS */
-
-#if ATARI_ST || MSDOS || OS2 || SASC
-	    case 't':
-	    case 'T':
-
-#ifdef RecordIO
-               status &= ~Fs_Record;
-#endif                                  /* RecordIO */
-
-	       untranslated = 0;
-	       continue;			/* no-op */
-
-#if OS2
-	    case 'p':
-	    case 'P':
-	       status |= Fs_Pipe;
-#endif					/* OS2 */
-
-	       continue;
-	    case 'u':
-	    case 'U':
-	       untranslated = 1;
-
-#ifdef RecordIO
-	       status &= ~Fs_Record;
-#endif					/* RecordIO */
-
-	       continue;
-#endif					/* ATARI_ST || MSDOS || ... */
-
 #ifdef RecordIO
 	    case 's':
 	    case 'S':
-	       untranslated = 1;
+	       status |= Fs_Untrans;
 	       status |= Fs_Record;
 	       continue;
 #endif					/* RecordIO */
 
-#if MACINTOSH
-#if LSC
 	    case 't':
 	    case 'T':
-	       untranslated = 0;
+	       status &= ~Fs_Untrans;
+#ifdef RecordIO
+               status &= ~Fs_Record;
+#endif                                  /* RecordIO */
 	       continue;
-	    case 'u':
-	    case 'U':
-	       untranslated = 1;
-	       continue;
-#endif					/* LSC */
-#if MPW
-	    case 't':
-	    case 'T':
-	    case 'u':
-	    case 'U':
-	       continue;			/* no-op */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
 
-   /*
-    * End of operating-system specific code.
-    */
+	    case 'u':
+	    case 'U':
+	       status |= Fs_Untrans;
+#ifdef RecordIO
+	       status &= ~Fs_Record;
+#endif					/* RecordIO */
+	       continue;
+
+#if AMIGA || ARM || OS2 || UNIX || VMS
+	    case 'p':
+	    case 'P':
+	       status |= Fs_Pipe;
+	       continue;
+#endif					/* AMIGA || ARM || OS2 || UNIX || VMS */
 
 	    case 'x':
 	    case 'X':
@@ -463,38 +461,36 @@ Deliberate Syntax Error
 #if ATARI_ST
       if ((status & (Fs_Read|Fs_Write)) == (Fs_Read|Fs_Write)) {
 	 mode[1] = '+';
-	 mode[2] = untranslated ? 'b' : 'a';
+	 mode[2] = ((status & Fs_Untrans) != 0) ? 'b' : 'a';
 	 }
-      else mode[1] = untranslated ? 'b' : 'a';
+      else mode[1] = ((status & Fs_Untrans) != 0) ? 'b' : 'a';
 #endif					/* ATARI_ST */
 
+#if MACINTOSH
+      /* nothing to do */
+#endif					/* MACINTOSH */
+
 #if MSDOS || OS2
       if ((status & (Fs_Read|Fs_Write)) == (Fs_Read|Fs_Write)) {
 	 mode[1] = '+';
 #if CSET2
          /* we don't have the 't' in C Set/2 */
-         if (untranslated) mode[2] = 'b';
+         if ((status & Fs_Untrans) != 0) mode[2] = 'b';
          } /* End of if - open file for reading or writing */
-      else if (untranslated) mode[1] = 'b';
+      else if ((status & Fs_Untrans) != 0) mode[1] = 'b';
 #else					/* CSET2 */
-	 mode[2] = untranslated ? 'b' : 't';
+	 mode[2] = ((status & Fs_Untrans) != 0) ? 'b' : 't';
 	 }
-      else mode[1] = untranslated ? 'b' : 't';
+      else mode[1] = ((status & Fs_Untrans) != 0) ? 'b' : 't';
 #endif					/* CSET2 */
 #endif					/* MSDOS || OS2 */
 
-#if MACINTOSH
-#if LSC
-      untranslated = 0;
-#endif					/* LSC */
-#endif					/* MACINTOSH */
-
 #if MVS || VM
       if ((status & (Fs_Read|Fs_Write)) == (Fs_Read|Fs_Write)) {
 	 mode[1] = '+';
-	 mode[2] = untranslated ? 'b' : 0;
+	 mode[2] = ((status & Fs_Untrans) != 0) ? 'b' : 0;
 	 }
-      else mode[1] = untranslated ? 'b' : 0;
+      else mode[1] = ((status & Fs_Untrans) != 0) ? 'b' : 0;
 #endif					/* MVS || VM */
 
 /*
@@ -548,23 +544,73 @@ Deliberate Syntax Error
 	 } else
 #endif					/* Graphics */
 
-#if ARM || OS2 || UNIX || VMS
+#if AMIGA || ARM || OS2 || UNIX || VMS
       if (status & Fs_Pipe) {
 	 if (status != (Fs_Read|Fs_Pipe) && status != (Fs_Write|Fs_Pipe))
 	    runerr(209, spec);
 	 f = popen(fnamestr, mode);
 	 }
       else
-#endif					/* ARM || OS2 || UNIX || VMS */
+#endif					/* AMIGA || ARM || OS2 || ... */
 
-	 f = fopen(fnamestr, mode);
+         {
+#ifdef ReadDirectory
+         struct stat sbuf;
+         if ((status & Fs_Write) == 0
+         && stat(fnamestr, &sbuf) == 0
+         && S_ISDIR(sbuf.st_mode)) {
+            status |= Fs_Directory;
+            f = (FILE*) opendir(fnamestr);
+            }
+         else
+#endif					/* ReadDirectory */
+            f = fopen(fnamestr, mode);
+         }
 #endif					/* OpenAttributes */
 
       /*
        * Fail if the file cannot be opened.
        */
-      if (f == NULL)
+      if (f == NULL) {
+#ifdef MSWindows
+         char tempbuf[512];
+	 *tempbuf = '\0';
+         if (strchr(fnamestr, '*') || strchr(fnamestr, '?')) {
+            /*
+	     * attempted to open a wildcard, do file completion
+	     */
+	    strcpy(tempbuf, fnamestr);
+            }
+	 else {
+            /*
+	     * check and see if the file was actually a directory
+	     */
+            struct stat fs;
+            if (stat(fnamestr, &fs) == -1) fail;
+	    if (fs.st_mode & _S_IFDIR) {
+	       strcpy(tempbuf, fnamestr);
+	       if (tempbuf[strlen(tempbuf)-1] != '\\')
+	          strcat(tempbuf, "\\");
+	       strcat(tempbuf, "*.*");
+	       }
+	    }
+         if (*tempbuf) {
+            FINDDATA_T fd;
+	    if (!FINDFIRST(tempbuf, &fd)) fail;
+            f = tmpfile();
+            if (f == NULL) fail;
+            do {
+               fprintf(f, "%s\n", FILENAME(&fd));
+               } while (FINDNEXT(&fd));
+            FINDCLOSE(&fd);
+            fflush(f);
+            fseek(f, 0, SEEK_SET);
+            if (f == NULL) fail;
+	    }
+#else					/* MSWindows */
 	 fail;
+#endif					/* MSWindows */
+	 }
 
 #if MACINTOSH
 #if MPW
@@ -638,13 +684,22 @@ function{0,1} read(f)
       if ((status & Fs_Read) == 0)
 	 runerr(212, f);
 
-#ifdef StandardLib
       if (status & Fs_Writing) {
 	 fseek(fp, 0L, SEEK_CUR);
 	 BlkLoc(f)->file.status &= ~Fs_Writing;
 	 }
       BlkLoc(f)->file.status |= Fs_Reading;
-#endif					/* StandardLib */
+
+#ifdef ReadDirectory
+      if ((BlkLoc(f)->file.status & Fs_Directory) != 0) {
+         struct dirent *de = readdir((DIR*) fp);
+         if (de == NULL)
+            fail;
+         slen = strlen(de->d_name);
+         Protect(sp = alcstr(de->d_name, slen), runerr(0));
+         return string(slen, sp);
+         }
+#endif					/* ReadDirectory */
 
 #ifdef ConsoleWindow
       /*
@@ -682,10 +737,10 @@ function{0,1} read(f)
 
 #ifdef RecordIO
 	 if ((slen = (status & Fs_Record ? getrec(sbuf, MaxReadStr, fp) :
-					   getstrg(sbuf, MaxReadStr, fp)))
+				   getstrg(sbuf, MaxReadStr, &BlkLoc(f)->file)))
 	     == -1) fail;
 #else					/* RecordIO */
-	 if ((slen = getstrg(sbuf,MaxReadStr,fp)) == -1)
+	 if ((slen = getstrg(sbuf, MaxReadStr, &BlkLoc(f)->file)) == -1)
 	    fail;
 #endif					/* RecordIO */
 
@@ -748,13 +803,11 @@ function{0,1} reads(f,i)
       if ((status & Fs_Read) == 0)
 	 runerr(212, f);
 
-#ifdef StandardLib
       if (status & Fs_Writing) {
 	 fseek(fp, 0L, SEEK_CUR);
 	 BlkLoc(f)->file.status &= ~Fs_Writing;
 	 }
       BlkLoc(f)->file.status |= Fs_Reading;
-#endif					/* StandardLib */
 
 #ifdef ConsoleWindow
       /*
@@ -778,6 +831,23 @@ function{0,1} reads(f,i)
 	 errorfail;
 	 }
 
+#ifdef ReadDirectory
+      /*
+       *  If reading a directory, return up to i bytes of next entry.
+       */
+      if ((BlkLoc(f)->file.status & Fs_Directory) != 0) {
+         char *sp;
+         struct dirent *de = readdir((DIR*) fp);
+         if (de == NULL)
+            fail;
+         nbytes = strlen(de->d_name);
+         if (nbytes > i)
+            nbytes = i;
+         Protect(sp = alcstr(de->d_name, nbytes), runerr(0));
+         return string(nbytes, sp);
+         }
+#endif					/* ReadDirectory */
+
       /*
        * For now, assume we can read the full number of bytes.
        */
@@ -785,6 +855,7 @@ function{0,1} reads(f,i)
       StrLen(s) = 0;
 
 #if AMIGA
+#if LATTICE
       /*
        * The following code is special for Lattice 4.0 -- it was different
        *  for Lattice 3.10.  It probably won't work correctly with other
@@ -806,6 +877,7 @@ function{0,1} reads(f,i)
 	 strfree = StrLoc(s) + i;
 	 return s;
 	 }
+#endif					/* LATTICE */
 #endif					/* AMIGA */
 
 #ifdef Graphics
@@ -856,7 +928,7 @@ function{0,1} remove(s)
       }
 
    inline {
-      if (unlink(s) != 0)
+      if (remove(s) != 0)
 	 fail;
       return nulldesc;
       }
@@ -880,36 +952,8 @@ function{0,1} rename(s1,s2)
       }
 
    body {
-/*
- * The following code is operating-system dependent [@fsys.06].  Rename the
- *  file, and fail if unsuccessful.
- */
-
-#if PORT
-   /* need something */
-Deliberate Syntax Error
-#endif					/* PORT */
-
-#if AMIGA || ARM || ATARI_ST || MACINTOSH || MSDOS || MVS || OS2 || VM || VMS
-      {
       if (rename(s1,s2) != 0)
 	 fail;
-      }
-#endif					/* AMIGA || ARM || ATARI_ST ... */
-
-#if UNIX
-      if (link(s1,s2) != 0)
-	 fail;
-      if (unlink(s1) != 0) {
-	 unlink(s2);	/* try to undo partial rename */
-	 fail;
-	 }
-#endif					/* UNIX */
-
-/*
- * End of operating-system specific code.
- */
-
       return nulldesc;
       }
 end
@@ -982,6 +1026,10 @@ function{0,1} seek(f,o)
       fd = BlkLoc(f)->file.fd;
       if (BlkLoc(f)->file.status == 0)
 	 fail;
+#ifdef ReadDirectory
+      if ((BlkLoc(f)->file.status & Fs_Directory) != 0)
+         fail;
+#endif					/* ReadDirectory */
 
 #ifdef Graphics
       pollctr >>= 1;
@@ -1022,16 +1070,12 @@ function{0,1} seek(f,o)
 	    fail;
 #endif					/* CSET2 */
 	 }
-#ifdef StandardLib
       BlkLoc(f)->file.status &= ~(Fs_Reading | Fs_Writing);
-#endif					/* StandardLib */
       return f;
       }
 end
 
 
-#ifdef SystemFnc
-
 "system(s) - execute string s as a system command."
 
 function{1} system(s)
@@ -1059,66 +1103,11 @@ function{1} system(s)
       pollctr++;
 #endif					/* Graphics */
 
-/*
- * The following code is operating-system dependent [@fsys.12].  Perform system
- *  call.  Should not get here unless system(s) is supported.
- */
-
-#if PORT
-Deliberate Syntax Error
-#endif					/* PORT */
-
-#if AMIGA || OS2 || UNIX
-      i = ((system(s) >> 8) & 0377);
-#endif					/* AMIGA || OS2 || ... */
-
-#if MSDOS
-#if HIGHC_386
       i = (C_integer)system(s);
-#else					/* HIGHC_386 */
-#ifdef MSWindows
-      i = ((mswinsystem(s) >> 8) & 0377);
-#else					/* MSWindows */
-      i = ((system(s) >> 8) & 0377);
-#endif					/* MSWindows */
-#endif					/* HIGHC_386 */
-#endif					/* MSDOS */
-
-#if ARM
-      i = (C_integer)system(s);
-#endif					/* ARM */
-
-#if ATARI_ST || VMS
-      i = system(s);
-#endif					/* ATARI_ST || VMS */
-
-#if MACINTOSH
-   /* Should not get here */
-#endif					/* MACINTOSH */
-
-#if MVS || VM
-#if SASC && MVS
-   {
-      char *wprefix;
-      wprefix = malloc(strlen(s)+5);
-		     /* hope this will do no harm... */
-      sprintf(wprefix,"tso:%s",s);
-      i = (C_integer)system(wprefix);
-      free(wprefix);
-   }
-#else					/* SASC && MVS */
-   i = (C_integer)system(s);
-#endif					/* SASC && MVS */
-#endif					/* MVS || VM */
-
-/*
- * End of operating-system specific code.
- */
       return C_integer i;
       }
 end
 
-#endif					/* SystemFnc */
 
 
 "where(f) - return current offset position in file f."
@@ -1141,6 +1130,10 @@ function{0,1} where(f)
 
       if ((BlkLoc(f)->file.status == 0))
 	 fail;
+#ifdef ReadDirectory
+      if ((BlkLoc(f)->file.status & Fs_Directory) != 0)
+         fail;
+#endif					/* ReadDirectory */
 
 #ifdef Graphics
       pollctr >>= 1;
@@ -1150,10 +1143,8 @@ function{0,1} where(f)
 #endif					/* Graphics */
 
       pos = ftell(fd) + 1;
-#ifdef StandardLib
       if (pos == 0)
 	 fail;	/* may only be effective on ANSI systems */
-#endif					/* StandardLib */
 
       return C_integer pos;
       }
@@ -1243,7 +1234,7 @@ end
 
 
 #if terminate
-	    c_exit(ErrorExit);
+	    c_exit(EXIT_FAILURE);
 #else					/* terminate */
 	    return retvalue;
 #endif					/* terminate */
@@ -1440,9 +1431,6 @@ GenWrite(write,  True,	False) /* write(s, ...) - write with new-line */
 GenWrite(writes, False, False) /* writes(s, ...) - write with no new-line */
 
 #ifdef KeyboardFncs
-#if UNIX
-int keyboard_error;
-#endif					/* UNIX */
 
 "getch() - return a character from console."
 
@@ -1452,19 +1440,7 @@ function{0,1} getch()
       }
    body {
       int i;
-#if UNIX
-      keyboard_error = 0;
-#endif					/* UNIX */
-#ifndef ConsoleWindow
       i = getch();
-#else					/* ConsoleWindow */
-      struct descrip res;
-      if (wgetchne((wbp)OpenConsole(), &res) < 0) fail;
-      i = *StrLoc(res);
-#endif					/* ConsoleWindow */
-#if UNIX
-      if (keyboard_error) runerr(keyboard_error);
-#endif					/* UNIX */
       if (i<0 || i>255)
 	 fail;
       return string(1, (char *)&allchars[FromAscii(i) & 0xFF]);
@@ -1479,19 +1455,7 @@ function{0,1} getche()
       }
    body {
       int i;
-#if UNIX
-      keyboard_error = 0;
-#endif					/* UNIX */
-#ifndef ConsoleWindow
       i = getche();
-#else					/* ConsoleWindow */
-      struct descrip res;
-      if (wgetche((wbp)OpenConsole(), &res) < 0) fail;
-      i = *StrLoc(res);
-#endif					/* ConsoleWindow */
-#if UNIX
-      if (keyboard_error) runerr(keyboard_error);
-#endif					/* UNIX */
       if (i<0 || i>255)
 	 fail;
       return string(1, (char *)&allchars[FromAscii(i) & 0xFF]);
@@ -1506,32 +1470,10 @@ function{0,1} kbhit()
       return null
       }
    inline {
-      int rv;
-#if UNIX
-      keyboard_error = 0;
-#endif					/* UNIX */
-#ifndef ConsoleWindow
-      rv = kbhit();
-#else					/* ConsoleWindow */
-     /* make sure we're up-to-date event wise */
-     if (ConsoleBinding) {
-        pollevent();
-        /*
-	 * perhaps should look in the console's icon event list for a keypress;
-	 *  either a string or event > 60k; presently, succeed for all events
-	 */
-        if (BlkLoc(((wbp)ConsoleBinding)->window->listp)->list.size > 0)
-	   return nulldesc;
-        }
-     fail;
-#endif					/* ConsoleWindow */
-#if UNIX
-      if (keyboard_error) runerr(keyboard_error);
-#endif					/* UNIX */
-      if (rv) {
+      if (kbhit())
 	 return nulldesc;
-	 }
-      else fail;
+      else
+         fail;
       }
 end
 #endif					/* KeyboardFncs */
@@ -1552,23 +1494,24 @@ function{0,1} chdir(s)
  */
 
 #if PORT
-Deliberate Syntax Error
+   Deliberate Syntax Error
 #endif					/* PORT */
 
-#if AMIGA || ARM || MACINTOSH || MVS || VM
+#if ARM || MACINTOSH || MVS || VM
       runerr(121);
-#endif					/* AMIGA || ARM || MACINTOSH ... */
+#endif					/* ARM || MACINTOSH ... */
 
-#if ATARI_ST || MSDOS || OS2 || UNIX || VMS
-#if NT
+#if AMIGA || ATARI_ST || MSDOS || OS2 || UNIX || VMS
+   #if NT
       int nt_chdir(char *);
       if (nt_chdir(s) != 0)
-#else					/* NT */
-      if (chdir(s) != 0)
-#endif					/* NT */
 	 fail;
+   #else					/* NT */
+      if (chdir(s) != 0)
+	 fail;
+   #endif					/* NT */
       return nulldesc;
-#endif					/* ATARI_ST || MSDOS || ... */
+#endif					/* AMIGA || ATARI_ST || MSDOS || ... */
 
 /*
  * End of operating-system specific code.
@@ -1629,6 +1572,11 @@ function{1} flush(f)
       FILE *fp;
       fp = BlkLoc(f)->file.fd;
 
+#ifdef ReadDirectory
+      if ((BlkLoc(f)->file.status & Fs_Directory) != 0)
+         return f;
+#endif					/* ReadDirectory */
+
 #ifdef Graphics
       pollctr >>= 1;
       pollctr++;
