@@ -216,7 +216,7 @@ function{0,1} Color(argv[argc])
             fail;
          len = strlen(colorname);
          Protect(tmp = alcstr(colorname, len), runerr(0));
-         return string(len, colorname);
+         return string(len, tmp);
          }
 
       CheckArgMultiple(2);
@@ -279,7 +279,7 @@ function{0,1} ColorValue(argv[argc])
 
       if (cnv:C_integer(argv[warg], n)) {
          if (w != NULL && (t = get_mutable_name(w, n)))
-            Protect(s = alcstr(t, (word)strlen(t)), runerr(306));
+            Protect(s = alcstr(t, (word)strlen(t)+1), runerr(306));
          else
             fail;
          }
@@ -287,7 +287,7 @@ function{0,1} ColorValue(argv[argc])
          runerr(103,argv[warg]);
 
       if (parsecolor(w, s, &r, &g, &b) == Succeeded) {
-         sprintf(tmp,"%d,%d,%d", r, g, b);
+         sprintf(tmp,"%ld,%ld,%ld", r, g, b);
 	 len = strlen(tmp);
 	 Protect(s = alcstr(tmp,len), runerr(306));
 	 return string(len, s);
@@ -911,7 +911,6 @@ function{1} DrawString(argv[argc])
       wbp w;
       int i, n, len, warg = 0;
       char *s;
-      int dx, dy;
 
       OptWindow(w);
       CheckArgMultiple(3);
@@ -1455,7 +1454,7 @@ function{0,1} PaletteColor(argv[argc])
       e += *StrLoc(d) & 0xFF;
       if (!e->valid)
          fail;
-      sprintf(tmp, "%d,%d,%d", e->clr.red, e->clr.green, e->clr.blue);
+      sprintf(tmp, "%ld,%ld,%ld", e->clr.red, e->clr.green, e->clr.blue);
       len = strlen(tmp);
       Protect(s = alcstr(tmp, len), runerr(306));
       return string(len, s);
@@ -2074,11 +2073,20 @@ function{0,1} WriteImage(argv[argc])
 
       /*
        * clip image to window, and fail if zero-sized.
+       * (the casts to long are necessary to avoid unsigned comparison.)
        */
-      if (x < 0)  x = 0;
-      if (y < 0)  y = 0;
-      if (x + width > w->window->width)    width = w->window->width - x;
-      if (y + height > w->window->height)  height = w->window->height - y;
+      if (x < 0)  {
+         width += x;
+         x = 0;
+         }
+      if (y < 0)  {
+         height += y;
+         y = 0;
+         }
+      if (x + width > (long) w->window->width)
+         width = w->window->width - x;
+      if (y + height > (long) w->window->height)
+         height = w->window->height - y;
       if (width <= 0 || height <= 0)
 	 fail;
 
@@ -2094,6 +2102,614 @@ function{0,1} WriteImage(argv[argc])
       ReturnWindow;
       }
 end
+
+#ifdef MSWindows
+
+"WinPlayMedia(w,x[]) - play a multimedia resource"
+
+function{0,1} WinPlayMedia(argv[argc])
+   abstract {
+      return null
+      }
+   body {
+      wbp w;
+      tended char *tmp;
+      int warg = 0;
+      int i;
+      wsp ws;
+      word n;
+      OptWindow(w);
+
+      ws = w->window;
+      for (n = warg; n < argc; n++) {
+         if (!cnv:C_string(argv[n], tmp))
+	    runerr(103,argv[warg]);
+         if (playmedia(w, tmp) == Failed) fail;
+         }
+      return nulldesc;
+      }
+end
+
+
+
+/*
+ * Simple Windows-native pushbutton
+ */
+"WinButton(w, s, x, y, wd, ht) - install a pushbutton with label s on window w"
+
+function{0,1} WinButton(argv[argc])
+   abstract {
+      return file
+      }
+   body {
+      wbp w;
+      wsp ws;
+      int i, ii, i2, r, total = 0;
+      C_integer x, y, width, height, warg = 0;
+      tended char *s, *s2;
+      tended struct descrip d;
+      tended struct b_list *hp;
+      OptWindow(w);
+      ws = w->window;
+      if (warg == argc) fail;
+      if (!cnv:C_string(argv[warg], s)) runerr(103, argv[warg]);
+      warg++;
+      /*
+       * look for an existing button with this id.
+       */
+      for(i = 0; i < ws->nChildren; i++) {
+         if (!strcmp(s, ws->child[i].id) && ws->child[i].type==CHILD_BUTTON)
+            break;
+         }
+      /*
+       * create a new button if none is found
+       */
+      if (i == ws->nChildren) {
+         ws->nChildren++;
+         ws->child = realloc(ws->child,
+			     ws->nChildren * sizeof(childcontrol));
+	 makebutton(ws, ws->child + i, s);
+         }
+
+      if (warg >= argc) x = 0;
+      else if (!def:C_integer(argv[warg], 0, x))
+	 runerr(101, argv[warg]);
+      warg++;
+      if (warg >= argc) y = 0;
+      else if (!def:C_integer(argv[warg], 0, y))
+	 runerr(101, argv[warg]);
+      warg++;
+      /*
+       * default width is width of text in system font + 2 chars
+       */
+      ii = sysTextWidth(w, s, strlen(s)) + 10;
+      if (warg >= argc) width = i2;
+      else if (!def:C_integer(argv[warg], i2, width))
+	 runerr(101, argv[warg]);
+      warg++;
+      /*
+       * default height is height of text in system font * 7/4
+       */
+      i2 = sysFontHeight(w) * 7 / 4;
+      if (warg >= argc) height = i2;
+      else if (!def:C_integer(argv[warg], i2, height))
+	 runerr(101, argv[warg]);
+
+      movechild(ws->child + i, x, y, width, height);
+      ReturnWindow;
+      }
+end
+
+"WinScrollBar(w, s, i1, i2, i3, x, y, wd, ht) - install a scrollbar"
+
+function{0,1} WinScrollBar(argv[argc])
+   abstract {
+      return file
+      }
+   body {
+      wbp w;
+      wsp ws;
+      C_integer x, y, width, height, warg = 0, i1, i2, i3, i, ii;
+      tended char *s, *s2;
+      tended struct descrip d;
+      tended struct b_list *hp;
+
+      OptWindow(w);
+      ws = w->window;
+      if (warg == argc) fail;
+      if (!cnv:C_string(argv[warg], s)) runerr(103, argv[warg]);
+      warg++;
+      /*
+       * look for an existing scrollbar with this id.
+       */
+      for(i = 0; i < ws->nChildren; i++) {
+         if (!strcmp(s, ws->child[i].id) && ws->child[i].type==CHILD_EDIT)
+            break;
+         }
+      /*
+       * i1, the min of the scrollbar range, defaults to 0
+       */
+      if (warg >= argc) i1 = 0;
+      else if (!def:C_integer(argv[warg], 0, i1)) runerr(101, argv[warg]);
+      warg++;
+      /*
+       * i2, the max of the scrollbar range, defaults to 100
+       */
+      if (warg >= argc) i2 = 100;
+      else if (!def:C_integer(argv[warg], 100, i2)) runerr(101, argv[warg]);
+      warg++;
+      /*
+       * create a new scrollbar at end of array if none was found
+       */
+      if (i == ws->nChildren) {
+         ws->nChildren++;
+         ws->child = realloc(ws->child, ws->nChildren * sizeof(childcontrol));
+	 makescrollbar(ws, ws->child + i, s, i1, i2);
+         }
+      /*
+       * i3, the interval, defaults to 10
+       */
+      if (warg >= argc) i3 = 10;
+      else if (!def:C_integer(argv[warg], 10, i3))
+	 runerr(101, argv[warg]);
+      warg++;
+      /*
+       * x defaults to the right edge of the window - system scrollbar width
+       */
+      ii = ws->width - sysScrollWidth();
+      if (warg >= argc) x = ii;
+      else if (!def:C_integer(argv[warg], ii, x))
+	 runerr(101, argv[warg]);
+      warg++;
+      /*
+       * y defaults to 0
+       */
+      if (warg >= argc) y = 0;
+      else if (!def:C_integer(argv[warg], 0, y))
+	 runerr(101, argv[warg]);
+      warg++;
+      /*
+       * width defaults to system scrollbar width
+       */
+      ii = sysScrollWidth();
+      if (warg >= argc) width = ii;
+      else if (!def:C_integer(argv[warg], ii, width))
+	 runerr(101, argv[warg]);
+      warg++;
+      /*
+       * height defaults to height of the client window
+       */
+      if (warg >= argc) height = ws->height;
+      else if (!def:C_integer(argv[warg], ws->height, height))
+	 runerr(101, argv[warg]);
+
+      movechild(ws->child + i, x, y, width, height);
+      ReturnWindow;
+      }
+end
+
+/*
+ * Simple Windows-native menu bar
+ */
+"WinMenuBar(w,L1,L2,...) - install a set of top-level menus"
+
+function{0,1} WinMenuBar(argv[argc])
+   abstract {
+      return file
+      }
+   body {
+      wbp w;
+      wsp ws;
+      int i, total = 0;
+      C_integer x, y, warg = 0;
+      tended char *s;
+      tended struct descrip d;
+      OptWindow(w);
+      ws = w->window;
+
+      if (warg == argc) fail;
+      for (i = warg; i < argc; i++) {
+         if (!is:list(argv[i])) runerr(108, argv[i]);
+         total += BlkLoc(argv[i])->list.size;
+	 }
+      /*
+       * free up memory for the old menu map
+       */
+      if (ws->nmMapElems) {
+         for (i=0; i<ws->nmMapElems; i++) free(ws->menuMap[i]);
+         free(ws->menuMap);
+         }
+      ws->menuMap = (char **)calloc(total, sizeof(char *));
+
+      if (nativemenubar(w, total, argc, argv, warg, &d) == Error)
+        runerr(103, d);
+      ReturnWindow;
+      }
+end
+
+/*
+ * Windows-native editor
+ */
+"WinEditRegion(w, s, s2, x, y, wd, ht) = install an edit box with label s"
+
+function{0, 1} WinEditRegion(argv[argc])
+   abstract {
+      return file ++ string
+      }
+   body {
+      wbp w;
+      wsp ws;
+      tended char *s, *s2;
+      C_integer i, x, y, width, height, warg = 0;
+      OptWindow(w);
+      ws = w->window;
+      if (warg == argc) fail;
+      if (!cnv:C_string(argv[warg], s))
+	 runerr(103, argv[warg]);
+      warg++;
+      /*
+       * look for an existing edit region with this id.
+       */
+      for(i = 0; i < ws->nChildren; i++) {
+         if (!strcmp(s, ws->child[i].id) && ws->child[i].type==CHILD_EDIT)
+            break;
+         }
+      /*
+       * create a new edit region if none is found
+       */
+      if (i == ws->nChildren) {
+         ws->nChildren++;
+         ws->child = realloc(ws->child, ws->nChildren * sizeof(childcontrol));
+	 makeeditregion(w, ws->child + i, s);
+         }
+      /*
+       * Invoked with no value, return the current value of an existing
+       * edit region (entire buffer is one gigantic string).
+       */
+      else if (warg == argc) {
+         geteditregion(ws->child + i, &result);
+	 return result;
+         }
+      /*
+       * Assign a value (s2 string contents) or perform editing command
+       */
+      if (is:null(argv[warg])) s2 = NULL;
+      else if (!cnv:C_string(argv[warg], s2)) runerr(103, argv[warg]);
+      warg++;
+
+      if (warg >= argc) x = 0;
+      else if (!def:C_integer(argv[warg], 0, x)) runerr(101, argv[warg]);
+      warg++;
+      if (warg >= argc) y = 0;
+      else if (!def:C_integer(argv[warg], 0, y)) runerr(101, argv[warg]);
+      warg++;
+      if (warg >= argc) width = ws->width - x;
+      else if (!def:C_integer(argv[warg], ws->width -x, width))
+         runerr(101, argv[warg]);
+      warg++;
+      if (warg >= argc) height = ws->height - y;
+      else if (!def:C_integer(argv[warg], ws->height - y, height))
+         runerr(101, argv[warg]);
+
+      if (s2 && !strcmp("!clear", s2)) {
+         cleareditregion(ws->child + i);
+         s2 = NULL;
+         }
+      else if (s2 && !strcmp("!copy", s2)) {
+         copyeditregion(ws->child + i);
+         s2 = NULL;
+         }
+      else if (s2 && !strcmp("!cut", s2)) {
+         cuteditregion(ws->child + i);
+         s2 = NULL;
+         }
+      else if (s2 && !strcmp("!paste", s2)) {
+         pasteeditregion(ws->child + i);
+         s2 = NULL;
+         }
+      else if (s2 && !strcmp("!undo", s2)) {
+         if (undoeditregion(ws->child + i) == Failed) fail;
+         s2 = NULL;
+         }
+      else if (s2 && !strncmp("!modified=", s2, 10)) {
+         setmodifiededitregion(ws->child + i, atoi(s2+10));
+         s2 = NULL;
+         }
+      else if (s2 && !strcmp("!modified", s2)) {
+         if (modifiededitregion(ws->child + i) == Failed) fail;
+         s2 = NULL;
+         }
+      else if (s2 && !strncmp("!font=", s2, 6)) {
+	 if (setchildfont(ws->child + i, s2 + 6) == Succeeded) {
+            ReturnWindow;
+	    }
+	 else fail;
+         }
+      else if (s2 && !strcmp("!setsel", s2)) {
+	 setchildselection(ws, ws->child + i, x, y);
+         ReturnWindow;
+         }
+
+      if (s2) {
+         seteditregion(ws->child + i, s2);
+	 }
+      movechild(ws->child + i, x, y, width, height);
+      setfocusonchild(ws, ws->child + i, width, height);
+      ReturnWindow;
+      }
+end
+
+
+/*
+ * common dialog functions
+ */
+
+"WinColorDialog(w,s) - choose a color for a window's context"
+
+function{0,1} WinColorDialog(argv[argc])
+   abstract {
+      return string
+      }
+   body {
+      wbp w;
+      C_integer x, y, width, height, warg = 0;
+      long r, g, b;
+      tended char *s;
+      char buf[64], *tmp = buf;
+      OptWindow(w);
+
+      if (warg < argc) {
+         if (is:null(argv[warg])) s = "white";
+	 else if (!cnv:C_string(argv[warg], s)) runerr(103, argv[warg]);
+         }
+      else s = "white";
+      if (parsecolor(w, s, &r, &g, &b) == Failed) fail;
+
+      if (nativecolordialog(w, r, g, b, buf) == NULL) fail;
+      StrLoc(result) = alcstr(buf, strlen(buf));
+      StrLen(result) = strlen(buf);
+      return result;
+      }
+end
+
+"WinFontDialog(w,s) - choose a font for a window's context"
+
+function{0,1} WinFontDialog(argv[argc])
+   abstract {
+      return string
+      }
+   body {
+      wbp w;
+      int r;
+      C_integer x, y, width, height, warg = 0, fheight;
+      int flags;
+      tended char *s;
+      char buf[64], *tmp = buf;
+      OptWindow(w);
+
+      if (warg < argc) {
+         if (is:null(argv[warg])) s = "fixed";
+	 else if (!cnv:C_string(argv[warg], s)) runerr(103, argv[warg]);
+         }
+      else s = "fixed";
+
+      parsefont(s, buf, &flags, &fheight);
+
+      if (nativefontdialog(w, buf, flags, fheight) == Failed) fail;
+      StrLoc(result) = alcstr(buf, strlen(buf));
+      StrLen(result) = strlen(buf);
+      return result;
+      }
+end
+
+
+"WinOpenDialog(w,s1,s2,i,s3,j) - choose a file to open"
+
+function{0,1} WinOpenDialog(argv[argc])
+   abstract {
+      return string
+      }
+   body {
+      wbp w;
+      int len, slen;
+      C_integer i, j, x, y, width, height, warg = 0;
+      char buf2[64], buf3[256], chReplace;
+      char *tmpstr;
+      tended char *s1, *s2, *s3;
+      OptWindow(w);
+
+      if (warg >= argc || is:null(argv[warg])) {
+         s1 = "Open:";
+         }
+      else if (!cnv:C_string(argv[warg], s1)) {
+         runerr(103, argv[warg]);
+         }
+      warg++;
+
+      if (warg >= argc || is:null(argv[warg])) {
+         s2 = "";
+         }
+      else if (!cnv:C_string(argv[warg], s2)) {
+         runerr(103, argv[warg]);
+         }
+      warg++;
+
+      if (warg >= argc) {
+         i = 50;
+         }
+      else if (!def:C_integer(argv[warg], 50, i)) {
+         runerr(101, argv[warg]);
+         }
+      warg++;
+
+      if (warg >= argc || is:null(argv[warg])) {
+         strcpy(buf3,"All Files(*.*)|*.*|");
+         s3 = buf3;
+         }
+      else if (!cnv:C_string(argv[warg], s3)) {
+         runerr(103, argv[warg]);
+         }
+      else {
+         strncpy(buf3, s3, 255);
+	 buf3[255] = '\0';
+	 s3 = buf3;
+         }
+      chReplace = s3[strlen(s3)-1];
+      slen = strlen(s3);
+      for(j=0; j < slen; j++)
+         if(s3[j] == chReplace) s3[j] = '\0';
+      warg++;
+
+      if (warg >= argc) {
+         j = 1;
+         }
+      else if (!def:C_integer(argv[warg], 1, j)) {
+         runerr(101, argv[warg]);
+         }
+      warg++;
+
+      if ((tmpstr = nativeopendialog(w,s1,s2,s3,i,j)) == NULL) fail;
+      len = strlen(tmpstr);
+      StrLoc(result) = alcstr(tmpstr, len);
+      StrLen(result) = len;
+      return result;
+      }
+end
+
+
+"WinSelectDialog(w, s1, buttons) - select from a set of choices"
+
+function{0,1} WinSelectDialog(argv[argc])
+   abstract {
+      return string
+      }
+   body {
+      wbp w;
+      C_integer i, j, warg = 0, len;
+      tended char *s1;
+      char *s2 = NULL, *tmpstr;
+      tended struct descrip d;
+      tended struct b_list *hp;
+      int lsize;
+      OptWindow(w);
+
+      /*
+       * look for list of text for the message.  concatenate text strings.
+       */
+      if (warg == argc)
+         fail;
+      if (!is:list(argv[warg])) runerr(108, argv[warg]);
+      hp  = (struct b_list *)BlkLoc(argv[warg]);
+      lsize = hp->size;
+      for(i=0; i < lsize; i++) {
+         c_get(hp, &d);
+         if (!cnv:C_string(d, s1)) runerr(103, d);
+         len += strlen(s1)+2;
+	 if (s2) {
+	    s2 = realloc(s2, len);
+	    if (!s2) fail;
+            strcat(s2, "\r\n");
+	    strcat(s2, s1);
+	    }
+	 else s2 = salloc(s1);
+         c_put(&(argv[warg]), &d);
+	 }
+      warg++;
+
+      if (warg >= argc) {
+         hp = NULL;
+         }
+      else {
+         if (!is:list(argv[warg])) runerr(108, argv[warg]);
+         hp  = (struct b_list *)BlkLoc(argv[warg]);
+         lsize = hp->size;
+         for(i=0; i < lsize; i++) {
+            c_get(hp, &d);
+            if (!cnv:C_string(d, s1)) runerr(103, d);
+            c_put(&(argv[warg]), &d);
+            }
+         }
+      tmpstr = nativeselectdialog(w, hp, s2);
+      if (tmpstr == NULL) fail;
+      free(s2);
+      len = strlen(tmpstr);
+      StrLoc(result) = alcstr(tmpstr, len);
+      StrLen(result) = len;
+      return result;
+      }
+end
+
+"WinSaveDialog(w,s1,s2,i,s3,j) - choose a file to save"
+
+function{0,1} WinSaveDialog(argv[argc])
+   abstract {
+      return string
+      }
+   body {
+      wbp w;
+      int len;
+      C_integer i, j, warg = 0, slen;
+      char buf3[128], chReplace;
+      tended char *tmpstr;
+      tended char *s1, *s2, *s3;
+      OptWindow(w);
+
+      if (warg >= argc || is:null(argv[warg])) {
+         s1 = "Save:";
+         }
+      else if (!cnv:C_string(argv[warg], s1)) {
+         runerr(103, argv[warg]);
+         }
+      warg++;
+
+      if (warg >= argc || is:null(argv[warg])) {
+         s2 = "";
+         }
+      else if (!cnv:C_string(argv[warg], s2)) {
+         runerr(103, argv[warg]);
+         }
+      warg++;
+
+      if (warg >= argc) {
+         i = 50;
+         }
+      else if (!def:C_integer(argv[warg], 50, i)) {
+         runerr(101, argv[warg]);
+         }
+      warg++;
+
+      if (warg >= argc || is:null(argv[warg])) {
+         strcpy(buf3,"All Files(*.*)|*.*|");
+         s3 = buf3;
+         }
+      else if (!cnv:C_string(argv[warg], s3)) {
+         runerr(103, argv[warg]);
+         }
+      else {
+         strcpy(buf3, s3);
+	 s3 = buf3;
+         }
+      chReplace = s3[strlen(s3)-1];
+      slen = strlen(s3);
+      for(j=0; j < slen; j++)
+         if(s3[j] == chReplace) s3[j] = '\0';
+      warg++;
+
+      if (warg >= argc) {
+         j = 1;
+         }
+      else if (!def:C_integer(argv[warg], 1, j)) {
+         runerr(101, argv[warg]);
+         }
+      warg++;
+      if ((tmpstr = nativesavedialog(w, s1, s2, s3, i, j)) == NULL) fail;
+      len = strlen(tmpstr);
+      StrLoc(result) = alcstr(tmpstr, len);
+      StrLen(result) = len;
+      return result;
+      }
+end
+#endif					/* MSWindows */
 
 #else					/* Graphics */
 static char x;			/* avoid empty module */
