@@ -27,8 +27,7 @@ unsigned _stack = 100000;
 /*
  * Prototypes.
  */
-hidden	novalue	env_err	Params((char *msg,char *name,char *val));
-novalue	icon_setup	Params((int argc, char **argv, int *ip));
+void	icon_setup	(int argc, char **argv, int *ip);
 
 #ifdef MacGraph
 void MacMain (int argc, char **argv);
@@ -48,22 +47,32 @@ int GetArgs (char **argv);
 Deliberate Syntax Error
 #endif					/* PORT */
 
+#if AMIGA && __SASC
+extern int _WBargc;     /* These are filled in by auto-initialization */
+extern char **_WBargv;  /* code in rlocal.r                           */
+char __stdiowin[] = "CON:10/40/640/200/IconX Console Window";
+   /* These override environment variables if set from ToolTypes. */
+extern uword WBstrsize;
+extern uword WBblksize;
+extern uword WBmstksize;
+#endif					/* AMIGA && __SASC */
+
 #if MACINTOSH
 #if MPW
 int NoOptions = 0;
 #endif					/* MPW */
 #endif					/* MACINTOSH */
 
-#if AMIGA || ARM || ATARI_ST || MSDOS || MVS || VM || OS2 || UNIX\
+#if ARM || ATARI_ST || MSDOS || MVS || VM || OS2 || UNIX\
    || VMS
    /* nothing needed */
-#endif					/* AMIGA || ARM || ATARI_ST ... */
+#endif					/* ARM || ATARI_ST ... */
 
 /*
  * End of operating-system specific code.
  */
 
-extern set_up;
+extern int set_up;
 
 /*
  * A number of important variables follow.
@@ -74,8 +83,6 @@ int n_globals = 0;			/* number of globals */
 int n_statics = 0;			/* number of statics */
 #endif					/* MultiThread */
 
-extern int_setup;
-
 
 /*
  * Initial icode sequence. This is used to invoke the main procedure with one
@@ -121,8 +128,6 @@ void detectRedirection()
      }
 }
 
-
-
 int CmdParamToArgv(char *s, char ***avp)
    {
    char tmp[MaxPath], dir[MaxPath];
@@ -132,8 +137,8 @@ int CmdParamToArgv(char *s, char ***avp)
 
    t = salloc(s);
    t2 = t;
+
    *avp = malloc(2 * sizeof(char *));
-   (*avp)[rv++] = salloc("wiconx.exe");
    (*avp)[rv] = NULL;
 
    detectRedirection();
@@ -141,7 +146,7 @@ int CmdParamToArgv(char *s, char ***avp)
    while (*t2) {
       while (*t2 && isspace(*t2)) t2++;
       switch (*t2) {
-	 case '\0': return rv;
+	 case '\0': break;
 	 case '<': case '>': {
 	    /*
 	     * perform file redirection; this is for Windows 3.1
@@ -158,8 +163,11 @@ int CmdParamToArgv(char *s, char ***avp)
 	       f = fopen(buf, "r");
 	    else
 	       f = fopen(buf, "w");
-	    if (f == NULL)
-	       syserr("unable to redirect i/o\n");
+	    if (f == NULL) {
+	       MessageBox(0, "unable to redirect i/o", "system error",
+			  MB_ICONHAND);
+	       c_exit(-1);
+	       }
 	    if (c == '<') {
 	       finredir = f;
 	       ConsoleFlags |= StdInRedirect;
@@ -172,12 +180,14 @@ int CmdParamToArgv(char *s, char ***avp)
 	    }
 	 case '"': {
 	    char *t3 = ++t2;			/* skip " */
+
             while (*t2 && (*t2 != '"')) t2++;
             if (*t2)
 	       *t2++ = '\0';
 	    *avp = realloc(*avp, (rv + 2) * sizeof (char *));
 	    (*avp)[rv++] = salloc(t3);
             (*avp)[rv] = NULL;
+
 	    break;
 	    }
          default: {
@@ -213,13 +223,35 @@ int CmdParamToArgv(char *s, char ***avp)
 	    }
          }
       }
+
    free(t);
    return rv;
    }
 
-novalue MSStartup(HINSTANCE hInstance, HINSTANCE hPrevInstance)
+char *lognam;
+char tmplognam[128];
+
+void MSStartup(HINSTANCE hInstance, HINSTANCE hPrevInstance)
    {
    WNDCLASS wc;
+#ifdef ConsoleWindow
+   extern FILE *flog;
+
+   /*
+    * Select log file name.  Might make this a command-line option.
+    * Default to "WICON.LOG".  The log file is used by Wi to report
+    * translation errors and jump to the offending source code line.
+    */
+   if ((lognam = getenv("WICONLOG")) == NULL)
+      lognam = "WICON.LOG";
+   remove(lognam);
+   lognam = strdup(lognam);
+   flog = fopen(tmpnam(tmplognam), "w");
+
+   if (flog == NULL) {
+      syserr("unable to open logfile");
+      }
+#endif					/* ConsoleWindow */
    if (!hPrevInstance) {
 #if NT
       wc.style = CS_HREDRAW | CS_VREDRAW;
@@ -251,7 +283,8 @@ int_PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
    mswinInstance = hInstance;
    ncmdShow = nCmdShow;
-   argc = CmdParamToArgv(lpszCmdLine, &argv);
+
+   argc = CmdParamToArgv(GetCommandLine(), &argv);
    MSStartup(hInstance, hPrevInstance);
    if (setjmp(mark_sj) == 0)
       iconx(argc,argv);
@@ -259,6 +292,7 @@ int_PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
       free(argv[argc]);
    free(argv);
    wfreersc();
+   xmfree();
    return 0;
 }
 #define main iconx
@@ -266,8 +300,8 @@ int_PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
 #if OS2
 int stubexe;
-novalue os2main(int stubflag, int argc, char**argv); /* Prototype OS2main */
-novalue os2main(stubflag, argc, argv)
+void os2main(int stubflag, int argc, char**argv); /* Prototype OS2main */
+void os2main(stubflag, argc, argv)
 int stubflag;
 #else					/* OS2 */
 #ifdef MacGraph
@@ -278,9 +312,9 @@ Boolean gDone;
 char *cmlArgs;
 StringHandle textHandle;
 
-novalue MacMain (argc, argv)
+void MacMain (argc, argv)
 #else					/* MacGraph */
-novalue main(argc, argv)
+int main(argc, argv)
 #endif					/* MacGraph */
 #endif					/* OS2 */
 int argc;
@@ -295,13 +329,13 @@ char **argv;
     char    **new_argv;
     struct FIND *p;
 
-    selfPtr = fopen( argv[0], "rb");
+    strcpy(settingsname, argv[0]);
+    selfPtr = fopen( settingsname, "rb");
     fseek( selfPtr, 0L, SEEK_END);
     if( ftell( selfPtr) > sizevar.value)
     {
-        strcpy( settingsname, argv[0]);
         ++argc;
-        for( i=argc-1; i>0; --i)  argv[i] = argv[i-1];
+        --argv;
         thisIsIconx = 1;
     }
     else
@@ -318,8 +352,8 @@ char **argv;
      */
     new_argc = 1;
     new_argv = malloc( new_argc * sizeof(char *));
-    new_argv[0] = malloc( (strlen(argv[0])+1) * sizeof(char));
-    memcpy( new_argv[0], argv[0], strlen(argv[0])+1);
+    new_argv[0] = malloc( (strlen(settingsname)+1) * sizeof(char));
+    memcpy( new_argv[0], settingsname, strlen(settingsname)+1);
     for( i=1; i<argc; ++i)
     {
         if( (p = findfirst( argv[i], 0)) != NULL)
@@ -342,6 +376,14 @@ char **argv;
         }
     }
 
+    if( new_argc == 1)
+    {
+        /* Add a null string to accommodate code in icon_setup() */
+        new_argv = realloc( new_argv, new_argc+1 * sizeof(char *));
+        new_argv[new_argc] = malloc( sizeof(char));
+        strcpy( new_argv[new_argc], "");
+    }
+
     argv = new_argv;
     argc = new_argc;
 
@@ -374,16 +416,6 @@ char **argv;
 #endif					/* AZTEC_C */
 #endif					/* AMIGA */
 
-#ifdef SuppressAlignmentMsg
-   {
-      /*
-       * suppress "Unaligned access" messages on Dec Alpha.
-       */
-      int buf[] = {SSIN_UACPROC, UAC_NOPRINT};
-      setsysinfo(SSI_NVPAIRS, buf, 1, 0, 0);
-   }
-#endif				/* SuppressAlignmentMsg */
-
 #if SASC
    quiet(1);                    /* suppress C library diagnostics */
 #endif					/* SASC */
@@ -423,7 +455,7 @@ char **argv;
    /*
     * Setup Icon interface.  It's done this way to avoid duplication
     *  of code, since the same thing has to be done if calling Icon
-    *  is enabled.  See istart.c.
+    *  is enabled.
     */
 
 #ifdef CRAY
@@ -458,7 +490,76 @@ char **argv;
 
       icon_init(argv[1], &argc, argv);
       }
-#else					/* PresentationManager */
+#else					/* OS2 */
+
+#if AMIGA && __SASC
+   if (argc == 0) {         /* argc == 0 flags a Workbench startup */
+      struct DiskObject *dob;
+      char *filename;
+      char *errorname;
+      char *size;
+      long newout;
+      long newerr;
+
+      if(dob = GetDiskObject(_WBargv[1])) {
+         if(dob->do_ToolTypes){
+         /* First get redirects from ToolTypes.
+            With a Workbench startup, stdin points to the console
+            window opened in MODE_NEWFILE, while stdout and stderr
+            share a pointer to the console window opened with
+            MODE_OLDFILE.  The close flag is on stderr. */
+
+            errorname = FindToolType(dob->do_ToolTypes,"STDERR");
+
+            if (errorname != NULL && strcmp(errorname, "-") == 0) {
+               if (filename = FindToolType(dob->do_ToolTypes,"STDOUT")) {
+                  if ( newout = Open(filename, MODE_NEWFILE) ) {
+                     Close(__ufbs[1].ufbfh);
+                     __ufbs[1].ufbfh = newout;
+                     __ufbs[2].ufbfh = newout;
+                     }
+                  }
+               }
+
+            else {
+               if (errorname != NULL) {
+                  if ( newerr = Open(errorname, MODE_NEWFILE) ) {
+                     __ufbs[2].ufbfh = newerr;
+                     __ufbs[1].ufbflg |= UFB_CLO;
+                     }
+                  }
+ 
+               if (filename = FindToolType(dob->do_ToolTypes,"STDOUT")) {
+                  if (newout = Open(filename, MODE_NEWFILE) ) {
+                     if (newerr) Close(__ufbs[1].ufbfh);
+                     __ufbs[1].ufbfh = newout;
+                     __ufbs[1].ufbflg |= UFB_CLO;
+                     }
+                  }
+               }
+
+            if (filename = FindToolType(dob->do_ToolTypes,"STDIN"))
+               freopen(filename, "r", stdin);
+
+         /* Set sizes from Tooltypes. */
+            if (size = FindToolType(dob->do_ToolTypes,"STRSIZE"))
+               WBstrsize = strtoul(size,NULL,10);
+
+            if (size = FindToolType(dob->do_ToolTypes,"BLKSIZE"))
+               WBblksize = strtoul(size,NULL,10);
+
+            if (size = FindToolType(dob->do_ToolTypes,"MSTKSIZE"))
+               WBmstksize = strtoul(size,NULL,10);
+
+            }
+         FreeDiskObject(dob);
+         }
+
+      argc = _WBargc;
+      argv = _WBargv;
+      }
+#endif					/* AMIGA && __SASC */
+
    icon_setup(argc, argv, &i);
 
    if (i < 0) {
@@ -474,10 +575,12 @@ char **argv;
 
    if (argc <= 1)
       error(NULL, "no icode file specified");
+
    /*
     * Call icon_init with the name of the icode file to execute.	[[I?]]
     */
    icon_init(argv[1], &argc, argv);
+
 #endif					/* OS2 */
 
    /*
@@ -491,16 +594,9 @@ char **argv;
    ipc.opnd = istart;
    *ipc.op++ = Op_Noop;  /* aligns Invoke's operand */	/*	[[I?]] */
    *ipc.op++ = Op_Invoke;				/*	[[I?]] */
-
-#if AMIGA
-   istart[0] = Op_Invoke;
-   istart[1] = 1;
-   istart[2] = Op_Quit;
-#else					/* AMIGA */
    *ipc.opnd++ = 1;
    *ipc.op = Op_Quit;
    ipc.opnd = istart;
-#endif					/* AMIGA */
 
    gfp = 0;
 
@@ -534,7 +630,7 @@ char **argv;
       fatalerr(117, NULL);
    PushDesc(globals[0]);
    PushNull;
-   argp = (dptr)(sp - 1);
+   glbl_argp = (dptr)(sp - 1);
 
    /*
     * If main() has a parameter, it is to be invoked with one argument, a list
@@ -543,7 +639,6 @@ char **argv;
     *  the list.  The null descriptor first pushed serves as Arg0 for
     *  Ollist and receives the result of the computation.
     */
-
    if (((struct b_proc *)BlkLoc(globals[0]))->nparam > 0) {
       for (i = 2; i < argc; i++) {
          char *tmp;
@@ -553,12 +648,12 @@ char **argv;
          PushAVal(tmp);
          }
 
-      Ollist(argc - 2, argp);
+      Ollist(argc - 2, glbl_argp);
       }
 
 
-   sp = (word *)argp + 1;
-   argp = 0;
+   sp = (word *)glbl_argp + 1;
+   glbl_argp = 0;
 
    set_up = 1;			/* post fact that iconx is initialized */
 
@@ -578,13 +673,14 @@ char **argv;
     dos_set_ctrl_break(ctrlbrk);   /* Restore original Ctrl-C operation */
 #endif
 
-   c_exit(NormalExit);
+   c_exit(EXIT_SUCCESS);
+   return 0;
 }
 
 /*
  * icon_setup - handle interpreter command line options.
  */
-novalue icon_setup(argc,argv,ip)
+void icon_setup(argc,argv,ip)
 int argc;
 char **argv;
 int *ip;
@@ -614,6 +710,44 @@ int *ip;
       }
 #endif					/* ExecImages */
 
+#if NT
+   /*
+    * if we didn't start with nticonx.exe or wiconx.exe, backup one
+    * so that our icode filename is argv[1].
+    */
+   {
+   char tmp[256], *t2;
+   int len = 0;
+   strcpy(tmp, argv[0]);
+   t2 = tmp;
+   while (*t2) {
+      *t2 = tolower(*t2);
+      t2++;
+      len++;
+      }
+
+   /*
+    * if argv[0] is not a reference to our interpreter, take it as the
+    * name of the icode file, and back up for it.
+    */
+#ifdef MSWindows
+   if (!((len == 6 && !strcmp(tmp+len-6, "wiconx")) ||
+         (len > 6 && !strcmp(tmp+len-7, "\\wiconx")) ||
+         (len == 10 && !strcmp(tmp+len-10, "wiconx.exe")) ||
+         (len > 10 && !strcmp(tmp+len-11, "\\wiconx.exe")))) {
+#else					/* MSWindows */
+   if (!((len == 7 && !strcmp(tmp+len-7, "nticonx")) ||
+         (len > 7 && !strcmp(tmp+len-8, "\\nticonx")) ||
+         (len == 11 && !strcmp(tmp+len-11, "nticonx.exe")) ||
+         (len > 11 && !strcmp(tmp+len-12, "\\nticonx.exe")))) {
+#endif					/* MSWindows */
+      argv--;
+      argc++;
+      (*ip)--;
+      }
+   }
+#endif					/* NT */
+
 #ifdef MaxLevel
    maxilevel = 0;
    maxplevel = 0;
@@ -714,10 +848,10 @@ int *ip;
  *  file.
  */
 #ifdef MultiThread
-novalue resolve(pstate)
-struct progstate *pstate;
+   void resolve(pstate)
+   struct progstate *pstate;
 #else					/* MultiThread */
-novalue resolve()
+   void resolve()
 #endif					/* MultiThread */
 
    {
@@ -725,18 +859,14 @@ novalue resolve()
    register struct b_proc *pp;
    register dptr dp;
    extern Omkrec();
-   extern int ftsize;
-   extern struct header hdr;
+   #ifdef MultiThread
+      register struct progstate *savedstate;
+   #endif				/* MultiThread */
 
-#ifdef MultiThread
-   register struct progstate *savedstate;
-#endif					/* MultiThread */
-
-
-#ifdef MultiThread
-   savedstate = curpstate;
-   if (pstate) curpstate = pstate;
-#endif					/* MultiThread */
+   #ifdef MultiThread
+      savedstate = curpstate;
+      if (pstate) curpstate = pstate;
+   #endif				/* MultiThread */
 
    /*
     * Relocate the names of the global variables.
@@ -830,7 +960,7 @@ novalue resolve()
  *  in rmemfix.c is defined to be other than malloc().
  */
 
-novalue xmfree()
+void xmfree()
    {
    register struct b_coexpr **ep, *xep;
    register struct astkblk *abp, *xabp;
@@ -841,11 +971,24 @@ novalue xmfree()
    code = NULL;
    free((pointer)stack);		/* interpreter stack */
    stack = NULL;
-   free((pointer)strbase);		/* allocated string region */
+   /*
+    * more is needed to free chains of heaps, also a multithread version
+    * of this function may be needed someday.
+    */
+   if (strbase)
+      free((pointer)strbase);		/* allocated string region */
    strbase = NULL;
-   free((pointer)blkbase);		/* allocated block region */
+   if (blkbase)
+      free((pointer)blkbase);		/* allocated block region */
    blkbase = NULL;
-   free((pointer)quallist);		/* qualifier list */
+   if (curstring != &rootstring)
+      free((pointer)curstring);		/* string region */
+   curstring = NULL;
+   if (curblock != &rootblock)
+      free((pointer)curblock);		/* allocated block region */
+   curblock = NULL;
+   if (quallist)
+      free((pointer)quallist);		/* qualifier list */
    quallist = NULL;
 
    /*
