@@ -18,21 +18,15 @@ LibDcl(field,2,".")
    register union block *bptr;
 #else					/* MultiThread */
    extern int *ftabp;
-#ifdef FieldTableCompression
-   extern int *fo;
-   extern unsigned char *focp;
-   extern short *fosp;
-   extern char *bm;
-#endif					/* FieldTableCompression */
+   #ifdef FieldTableCompression
+      extern int *fo;
+      extern unsigned char *focp;
+      extern short *fosp;
+      extern char *bm;
+   #endif				/* FieldTableCompression */
    extern word *records;
 #endif					/* MultiThread */
 
-#if MACINTOSH
-#if MPW
-/* #pragma unused(nargs) */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
-
    Deref(Arg1);
 
    /*
@@ -156,7 +150,7 @@ LibDcl(mkrec,-1,"mkrec")
     * Assign each argument value to a record element and dereference it.
     */
    for ( ; i > 0; i--) {
-      rp->fields[i-1] = Arg(i);
+      rp->fields[i-1] = cargp[i]; /* Arg(i), expanded to avoid CLCC bug on Sun*/
       Deref(rp->fields[i-1]);
       }
 
@@ -176,12 +170,6 @@ LibDcl(limit,2,BackSlash)
 
    C_integer tmp;
 
-#if MACINTOSH
-#if MPW
-/* #pragma unused(nargs) */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
-
    /*
     * The limit is both passed and returned in Arg0.  The limit must
     *  be an integer.  If the limit is 0, the expression being evaluated
@@ -216,16 +204,9 @@ LibDcl(limit,2,BackSlash)
 
 LibDcl(bscan,2,"?")
    {
-   char sbuf[MaxCvtLen];
    int rc;
    struct pf_marker *cur_pfp;
 
-#if MACINTOSH
-#if MPW
-/* #pragma unused(nargs) */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
-
    /*
     * Convert the new value for &subject to a string.
     */
@@ -303,12 +284,6 @@ LibDcl(escan,1,"escan")
    int rc;
    struct pf_marker *cur_pfp;
 
-#if MACINTOSH
-#if MPW
-/* #pragma unused(nargs) */
-#endif					/* MPW */
-#endif					/* MACINTOSH */
-
    /*
     * Copy the result of the scanning expression into Arg0, which will
     *  be the result of the scan.
