@@ -1,7 +1,7 @@
 /*
  * File: init.r
  * Initialization, termination, and such.
- * Contents: read_hdr, init/icon_init, envset, env_err, env_int,
+ * Contents: readhdr, init/icon_init, envset, env_err, env_int,
  *  fpe_trap, inttrag, segvtrap, error, syserr, c_exit, err,
  *  fatalerr, pstrnmcmp, datainit, [loadicode, savepstate, loadpstate]
  */
@@ -9,7 +9,7 @@
 #if !COMPILER
 #include "../h/header.h"
 
-hidden	FILE	*readhdr	Params((char *name, struct header *hdr));
+static	FILE	*readhdr	(char *name, struct header *hdr);
 #endif					/* !COMPILER */
 
 #if SCCX_MX
@@ -22,8 +22,8 @@ extern  setint_t    sizevar;
  * Prototypes.
  */
 
-hidden novalue	env_err		Params((char *msg, char *name, char *val));
-FILE		*pathOpen       Params((char *fname, char *mode));
+static void	env_err		(char *msg, char *name, char *val);
+FILE		*pathOpen       (char *fname, char *mode);
 
 /*
  * The following code is operating-system dependent [@init.01].  Declarations
@@ -37,6 +37,10 @@ Deliberate Syntax Error
 
 #if AMIGA
 int chkbreak;				/* if nonzero, check for ^C */
+  /* These override environment variables if set from ToolTypes. */
+uword WBstrsize = 0;
+uword WBblksize = 0;
+uword WBmstksize = 0;
 #endif					/* AMIGA */
 
 #if MSDOS
@@ -68,7 +72,7 @@ int stubexe;				/* TRUE if resource attached to executable */
 char *prog_name;			/* name of icode file */
 
 #if !COMPILER
-#define OpDef(p,n,s,u) int Cat(O,p) Params((dptr cargp));
+#define OpDef(p,n,s,u) int Cat(O,p) (dptr cargp);
 #include "../h/odefs.h"
 #undef OpDef
 
@@ -131,11 +135,11 @@ uword blktotal = 0;			/* cumulative total block allocation */
 int dodump;				/* if nonzero, core dump on error */
 int noerrbuf;				/* if nonzero, do not buffer stderr */
 
-struct descrip k_current;		/* current expression stack pointer */
 struct descrip maps2;			/* second cached argument of map */
 struct descrip maps3;			/* third cached argument of map */
 
 #ifndef MultiThread
+struct descrip k_current;		/* current expression stack pointer */
 int k_errornumber = 0;			/* &errornumber */
 char *k_errortext = "";			/* &errortext */
 struct descrip k_errorvalue;		/* &errorvalue */
@@ -146,13 +150,13 @@ struct descrip t_errorvalue;		/* tentative k_errorvalue value */
 #endif					/* MultiThread */
 
 struct b_coexpr *stklist;	/* base of co-expression block list */
-dptr argp = NULL;		/* argument pointer */
 
 struct tend_desc *tend = NULL;  /* chain of tended descriptors */
 
 struct region rootstring, rootblock;
 
 #ifndef MultiThread
+dptr glbl_argp = NULL;		/* argument pointer */
 dptr globals, eglobals;			/* pointer to global variables */
 dptr gnames, egnames;			/* pointer to global variable names */
 dptr estatics;				/* pointer to end of static variables */
@@ -296,8 +300,8 @@ struct header *hdr;
    else {
 #endif					/* MSDOS */
 
-   if (n <= 4 || (strcmp(name+n-4,IcodeSuffix) != 0)
-   && strcmp(name+n-4,IcodeASuffix) != 0) {
+   if (n <= 4 || (strcmp(name+n-4,IcodeSuffix) != 0
+   && strcmp(name+n-4,IcodeASuffix) != 0)) {
       char tname[100];
       if ((int)strlen(name) + 5 > 100)
 	 error(name, "icode file name too long");
@@ -306,7 +310,7 @@ struct header *hdr;
 #if MVS
    {
       char *p;
-      if (p = index(name, '(')) {
+      if (p = strchr(name, '(')) {
 	 tname[p-name] = '\0';
       }
 #endif					/* MVS */
@@ -323,6 +327,23 @@ struct header *hdr;
 #else					/* MSDOS || OS2 */
       fname = fopen(tname, ReadBinary);
 #endif					/* MSDOS || OS2 */
+
+#if NT
+    /*
+     * tried appending .exe, now try .bat or .cmd
+     */
+    if (fname == NULL) {
+       strcpy(tname,name);
+       strcat(tname,".bat");
+       fname = pathOpen(tname, ReadBinary);
+       if (fname == NULL) {
+          strcpy(tname,name);
+          strcat(tname,".cmd");
+          fname = pathOpen(tname, ReadBinary);
+          }
+      }
+#endif					/* NT */
+
       }
 
    if (fname == NULL)			/* try the name as given */
@@ -335,14 +356,13 @@ struct header *hdr;
 
 #if MSDOS
       } /* end if (n >= 4 && !stricmp(".exe", name + n - 4)) */
-#endif
+#endif					/* MSDOS */
 
    if (fname == NULL)
       return NULL;
 
    {
    static char errmsg[] = "can't read interpreter file header";
-   static char exe_errmsg[] = "can't read MS-DOS .exe header";
 
 #ifdef Header
 
@@ -367,7 +387,7 @@ struct header *hdr;
    char buf[200];
 
    for (;;) {
-      if (fgets(buf, sizeof buf, fname) == NULL)
+      if (fgets(buf, sizeof buf-1, fname) == NULL)
 	 error(name, errmsg);
 #if NT
       if (strncmp(buf, "rem [executable Icon binary follows]", 36) == 0)
@@ -376,6 +396,7 @@ struct header *hdr;
 #endif					/* NT */
 	 break;
       }
+
    while ((n = getc(fname)) != EOF && n != '\f')	/* read thru \f\n\0 */
       ;
    getc(fname);
@@ -396,6 +417,7 @@ struct header *hdr;
         else
 #endif                                  /* SCCX_MX */
         {
+            static char exe_errmsg[] = "can't read MS-DOS .exe header";
             fread (&bytesThatBeginEveryExe,
                     sizeof bytesThatBeginEveryExe, 1, fname);
             if (bytesThatBeginEveryExe[0] != 'M' ||
@@ -420,7 +442,6 @@ struct header *hdr;
       error(name, errmsg);
    }
 
-
    return fname;
    }
 #endif
@@ -433,20 +454,19 @@ struct header *hdr;
 #endif					/* !COMPILER */
 
 #if COMPILER
-novalue init(name, argcp, argv, trc_init)
+void init(name, argcp, argv, trc_init)
 char *name;
 int *argcp;
 char *argv[];
 int trc_init;
 #else					/* COMPILER */
-novalue icon_init(name, argcp, argv)
+void icon_init(name, argcp, argv)
 char *name;
 int *argcp;
 char *argv[];
 #endif					/* COMPILER */
 
    {
-   int n;
 #if !COMPILER
    FILE *fname = NULL;
    word cbread, longread();
@@ -512,20 +532,21 @@ char *argv[];
     * initialize root pstate
     */
    curpstate = &rootpstate;
-   rootpstate.opcodemask = nulldesc;
    rootpstate.parentdesc = nulldesc;
    rootpstate.eventmask= nulldesc;
+   rootpstate.opcodemask = nulldesc;
    rootpstate.eventcode= nulldesc;
    rootpstate.eventval = nulldesc;
    rootpstate.eventsource = nulldesc;
+   rootpstate.Glbl_argp = NULL;
    MakeInt(0, &(rootpstate.Kywd_err));
    MakeInt(1, &(rootpstate.Kywd_pos));
-   MakeInt(0, &(rootpstate.Kywd_ran));
-   StrLen(rootpstate.Kywd_prog) = strlen(prog_name);
-   StrLoc(rootpstate.Kywd_prog) = prog_name;
    StrLen(rootpstate.ksub) = 0;
    StrLoc(rootpstate.ksub) = "";
    MakeInt(hdr.trace, &(rootpstate.Kywd_trc));
+   StrLen(rootpstate.Kywd_prog) = strlen(prog_name);
+   StrLoc(rootpstate.Kywd_prog) = prog_name;
+   MakeInt(0, &(rootpstate.Kywd_ran));
    rootpstate.K_errornumber = 0;
    rootpstate.T_errornumber = 0;
    rootpstate.Have_errval = 0;
@@ -600,7 +621,7 @@ Deliberate Syntax Error
    {
       void MacInit(void);
       void SetFloatTrap(void (*fpetrap)());
-      novalue fpetrap();
+      void fpetrap();
 
       MacInit();
       SetFloatTrap(fpetrap);
@@ -687,20 +708,17 @@ Deliberate Syntax Error
    fname = readhdr(name,&hdr);
    if (fname == NULL) {
       error(name, "cannot open interpreter file");
-#endif					/* PresentationManager */
+#endif					/* OS2 */
       }
 
    k_trace = hdr.trace;
 
 #endif					/* COMPILER */
 
-#ifdef EnvVars
    /*
     * Examine the environment and make appropriate settings.    [[I?]]
     */
    envset();
-#endif					/* EnvVars */
-
 
    /*
     * Convert stack sizes from words to bytes.
@@ -956,11 +974,15 @@ Deliberate Syntax Error
    if (noerrbuf)
       setbuf(stderr, NULL);
    else {
+#ifdef MSWindows
+      char buf[BUFSIZ];
+#else					/* MSWindows */
       char *buf;
 
       buf = (char *)malloc((msize)BUFSIZ);
       if (buf == NULL)
 	 fatalerr(305, NULL);
+#endif					/* MSWindows */
       setbuf(stderr, buf);
       }
 #endif					/* !HIGHC_386 */
@@ -982,12 +1004,11 @@ Deliberate Syntax Error
  */
 
 
-#ifdef EnvVars
 /*
  * Check for environment variables that Icon uses and set system
  *  values as is appropriate.
  */
-novalue envset()
+void envset()
    {
    register char *p;
 
@@ -997,7 +1018,9 @@ novalue envset()
    env_int(COEXPSIZE, &stksize, 1, (uword)MaxUnsigned);
    env_int(STRSIZE, &ssize, 1, (uword)MaxBlock);
    env_int(HEAPSIZE, &abrsize, 1, (uword)MaxBlock);
+#ifndef BSD_4_4_LITE
    env_int(BLOCKSIZE, &abrsize, 1, (uword)MaxBlock);    /* synonym */
+#endif					/* BSD_4_4_LITE */
    env_int(BLKSIZE, &abrsize, 1, (uword)MaxBlock);      /* synonym */
    env_int(MSTKSIZE, &mstksize, 1, (uword)MaxUnsigned);
    env_int(QLSIZE, &qualsize, 1, (uword)MaxBlock);
@@ -1017,6 +1040,9 @@ Deliberate Syntax Error
 #if AMIGA
    if ((p = getenv("CHECKBREAK")) != NULL)
       chkbreak++;
+   if (WBstrsize != 0 && WBstrsize <= MaxBlock) ssize = WBstrsize;
+   if (WBblksize != 0 && WBblksize <= MaxBlock) abrsize = WBblksize;
+   if (WBmstksize != 0 && WBmstksize <= (uword) MaxUnsigned) mstksize = WBmstksize; 
 #endif					/* AMIGA */
 
 #if ARM || ATARI_ST || MACINTOSH || MSDOS || MVS || OS2 || UNIX || VM || VMS
@@ -1073,7 +1099,7 @@ Deliberate Syntax Error
  * env_err - print an error mesage about the value of an environment
  *  variable.
  */
-static novalue env_err(msg, name, val)
+static void env_err(msg, name, val)
 char *msg;
 char *name;
 char *val;
@@ -1091,7 +1117,7 @@ char *val;
 /*
  * env_int - get the value of an integer-valued environment variable.
  */
-novalue env_int(name, variable, non_neg, limit)
+void env_int(name, variable, non_neg, limit)
 char *name;
 word *variable;
 int non_neg;
@@ -1128,7 +1154,6 @@ uword limit;
       env_err("environment variable not numeric", name, value);
    *variable = sign * n;
 }
-#endif					/* EnvVars */
 
 /*
  * Termination routines.
@@ -1138,7 +1163,7 @@ uword limit;
  * Produce run-time error 204 on floating-point traps.
  */
 
-novalue fpetrap()
+void fpetrap()
    {
    fatalerr(204, NULL);
    }
@@ -1147,7 +1172,7 @@ novalue fpetrap()
  * Produce run-time error 320 on ^C interrupts. Not used at present,
  *  since malfunction may occur during traceback.
  */
-novalue inttrap()
+void inttrap()
    {
    fatalerr(320, NULL);
    }
@@ -1155,7 +1180,7 @@ novalue inttrap()
 /*
  * Produce run-time error 302 on segmentation faults.
  */
-novalue segvtrap()
+void segvtrap()
    {
 
 #if MVS || VM
@@ -1170,7 +1195,7 @@ novalue segvtrap()
 /*
  * error - print error message from s1 and s2; used only in startup code.
  */
-novalue error(s1, s2)
+void error(s1, s2)
 char *s1, *s2;
    {
 
@@ -1198,13 +1223,13 @@ char *s1, *s2;
 
    if (dodump)
       abort();
-   c_exit(ErrorExit);
+   c_exit(EXIT_FAILURE);
    }
 
 /*
  * syserr - print s as a system error.
  */
-novalue syserr(s)
+void syserr(s)
 char *s;
    {
 
@@ -1232,13 +1257,41 @@ char *s;
    fflush(stderr);
    if (dodump)
       abort();
-   c_exit(ErrorExit);
+   c_exit(EXIT_FAILURE);
    }
 
+#ifdef ConsoleWindow
+void closelogfile()
+{
+   if (flog) {
+      extern char *lognam;
+      extern char tmplognam[];
+      FILE *flog2;
+      int i;
+      fclose(flog);
+
+      /*
+       * copy to the permanent file name
+       */
+      if ((flog = fopen(tmplognam, "r")) &&
+	  (flog2 = fopen(lognam, "w"))) {
+	 while ((i = getc(flog)) != EOF)
+	    putc(i, flog2);
+	 fclose(flog);
+	 fclose(flog2);
+	 remove(tmplognam);
+	 }
+
+      free(lognam);
+      flog = NULL;
+      }
+}
+#endif					/* ConsoleWindow */
+
 /*
  * c_exit(i) - flush all buffers and exit with status i.
  */
-novalue c_exit(i)
+void c_exit(i)
 int i;
 {
 #ifdef ConsoleWindow
@@ -1277,9 +1330,10 @@ int i;
       fprintf(stderr,"\nTermination dump:\n\n");
       fflush(stderr);
       fprintf(stderr,"co-expression #%ld(%ld)\n",
-	 BlkLoc(k_current)->coexpr.id, BlkLoc(k_current)->coexpr.size);
+	 (long)BlkLoc(k_current)->coexpr.id,
+	 (long)BlkLoc(k_current)->coexpr.size);
       fflush(stderr);
-      xdisp(pfp,argp,k_level,stderr);
+      xdisp(pfp,glbl_argp,k_level,stderr);
       }
 
 #ifdef MultipleRuns
@@ -1316,10 +1370,14 @@ int i;
 /* undo the #define exit c_exit */
 #undef exit
 #passthru #undef exit
+
+   closelogfile();
+
 #endif					/* ConsoleWindow */
 
 #ifdef MSWindows
    PostQuitMessage(0);
+   while (wstates != NULL) pollevent();
 #endif					/* MSWindows */
 
 #if TURBO || BORLAND_286 || BORLAND_386
@@ -1352,7 +1410,7 @@ int err()
 /*
  * fatalerr - disable error conversion and call run-time error routine.
  */
-novalue fatalerr(n, v)
+void fatalerr(n, v)
 int n;
 dptr v;
    {
@@ -1372,9 +1430,12 @@ struct pstrnm *a, *b;
 /*
  * datainit - initialize some global variables.
  */
-novalue datainit()
+void datainit()
    {
+#ifdef MSWindows
    extern FILE *finredir, *fouredir, *ferredir;
+#endif					/* MSWindows */
+
    /*
     * Initializations that cannot be performed statically (at least for
     * some compilers).					[[I?]]
@@ -1534,7 +1595,7 @@ C_integer bs, ss, stk;
     * Allocate memory for icode and the struct that describes it
     */
      Protect(coexp = alccoexp(hdr.hsize, stk),
-      { fprintf(stderr,"can't malloc new icode region\n");c_exit(ErrorExit);});
+      { fprintf(stderr,"can't malloc new icode region\n");c_exit(EXIT_FAILURE);});
 
    pstate = coexp->program;
    /*
@@ -1548,6 +1609,7 @@ C_integer bs, ss, stk;
    pstate->eventcode= nulldesc;
    pstate->eventval = nulldesc;
    pstate->eventsource = nulldesc;
+   pstate->K_current.dword = D_Coexpr;
 
    MakeInt(0, &(pstate->Kywd_err));
    MakeInt(1, &(pstate->Kywd_pos));
@@ -1626,7 +1688,6 @@ C_integer bs, ss, stk;
 #endif					/* Graphics */
 
    pstate->Coexp_ser = 2;
-   pstate->Coexp_ser = 1;
    pstate->List_ser = 1;
    pstate->Set_ser = 1;
    pstate->Table_ser = 1;
