@@ -13,10 +13,10 @@ extern fptr fncentry[];
  * Prototypes for static functions.
  */
 #ifdef EventMon
-hidden struct ef_marker *vanq_bound Params((struct ef_marker *efp_v,
-                                      struct gf_marker *gfp_v));
-hidden novalue           vanq_proc Params((struct ef_marker *efp_v,
-                                     struct gf_marker *gfp_v));
+static struct ef_marker *vanq_bound (struct ef_marker *efp_v,
+                                      struct gf_marker *gfp_v);
+static void           vanq_proc (struct ef_marker *efp_v,
+                                     struct gf_marker *gfp_v);
 #endif					/* EventMon */
 
 /*
@@ -524,7 +524,7 @@ Deliberate Syntax Error
 
 	 case Op_Arg:		/* argument */
 	    PushVal(D_Var);
-	    PushAVal(&argp[GetWord + 1]);
+	    PushAVal(&glbl_argp[GetWord + 1]);
 	    break;
 
 	 case Op_Global:	/* global */
@@ -690,16 +690,14 @@ Deliberate Syntax Error
 
          case Op_Colm:		/* source column number */
             {
-             word loc;
-
 #ifdef EventMon
+            word loc;
             column = GetWord;
             loc = column;
-            loc <<= (WordBits >> 1);		/* column in high-order part */
+            loc <<= (WordBits >> 1);	/* column in high-order part */
             loc += linenum;
             InterpEVVal(loc, E_Loc);
 #endif					/* EventMon */
-
             break;
             }
 
@@ -887,7 +885,6 @@ invokej:
 	       goto C_rtn_term;
 	       }
 	    }
-	    break;
 	    }
 
 	 case Op_Keywd: 	/* keyword */
@@ -1146,7 +1143,7 @@ Lsusp_uw:
 	    svalp = (dptr)(rsp - 1);
 	    if (Var(*svalp)) {
                ExInterp;
-               retderef(svalp, (word *)argp, sp);
+               retderef(svalp, (word *)glbl_argp, sp);
                EntInterp;
                }
 
@@ -1159,7 +1156,7 @@ Lsusp_uw:
 	    newgfp->gf_gfp = gfp;
 	    newgfp->gf_efp = efp;
 	    newgfp->gf_ipc = ipc;
-	    newgfp->gf_argp = argp;
+	    newgfp->gf_argp = glbl_argp;
 	    newgfp->gf_pfp = pfp;
 	    gfp = newgfp;
 	    rsp += Wsizeof(*gfp);
@@ -1179,7 +1176,7 @@ Lsusp_uw:
 	       }
 	    else
 	       firstwd = (word *)pfp->pf_efp + Wsizeof(*efp);
-	    lastwd = (word *)argp - 1;
+	    lastwd = (word *)glbl_argp - 1;
 	       efp = efp->ef_efp;
 
 	    /*
@@ -1193,7 +1190,7 @@ Lsusp_uw:
 	    --k_level;
 	    if (k_trace) {
                k_trace--;
-	       sproc = (struct b_proc *)BlkLoc(*argp);
+	       sproc = (struct b_proc *)BlkLoc(*glbl_argp);
 	       strace(&(sproc->pname), svalp);
 	       }
 
@@ -1226,7 +1223,7 @@ Lsusp_uw:
 
 	    efp = pfp->pf_efp;
 	    ipc = pfp->pf_ipc;
-	    argp = pfp->pf_argp;
+	    glbl_argp = pfp->pf_argp;
 	    pfp = pfp->pf_pfp;
 	    break;
 	    }
@@ -1281,26 +1278,27 @@ Eret_uw:
 	     *	the activation of the procedure are deactivated.  Appropriate
 	     *	values are restored from the procedure frame.
 	     */
-	    struct b_proc *rproc = (struct b_proc *)BlkLoc(*argp);
+	    struct b_proc *rproc;
+	    rproc = (struct b_proc *)BlkLoc(*glbl_argp);
 #ifdef EventMon
-            oldargp = *argp;
+            oldargp = *glbl_argp;
 	    ExInterp;
             vanq_proc(efp, gfp);
 	    EntInterp;
 	    /* used to InterpEVValD(argp,E_Pret); here */
 #endif					/* EventMon */
 
-	    *argp = *(dptr)(rsp - 1);
-	    if (Var(*argp)) {
+	    *glbl_argp = *(dptr)(rsp - 1);
+	    if (Var(*glbl_argp)) {
                ExInterp;
-               retderef(argp, (word *)argp, sp);
+               retderef(glbl_argp, (word *)glbl_argp, sp);
                EntInterp;
                }
 
 	    --k_level;
 	    if (k_trace) {
                k_trace--;
-	       rtrace(&(rproc->pname), argp);
+	       rtrace(&(rproc->pname), glbl_argp);
                }
 Pret_uw:
 	    if (pfp->pf_ilevel < ilevel) {
@@ -1320,11 +1318,11 @@ Pret_uw:
 	   if (!is:proc(oldargp) && is:proc(unwinder))
 	      oldargp = unwinder;
 #endif					/* EventMon */
-	    rsp = (word *)argp + 1;
+	    rsp = (word *)glbl_argp + 1;
 	    efp = pfp->pf_efp;
 	    gfp = pfp->pf_gfp;
 	    ipc = pfp->pf_ipc;
-	    argp = pfp->pf_argp;
+	    glbl_argp = pfp->pf_argp;
 	    pfp = pfp->pf_pfp;
 
 #ifdef MultiThread
@@ -1390,11 +1388,11 @@ efail_noev:
 	       type = (int)resgfp->gf_gentype;
 
 	       if (type == G_Psusp) {
-		  argp = resgfp->gf_argp;
+		  glbl_argp = resgfp->gf_argp;
 		  if (k_trace) {	/* procedure tracing */
                      k_trace--;
 		     ExInterp;
-		     atrace(&(((struct b_proc *)BlkLoc(*argp))->pname));
+		     atrace(&(((struct b_proc *)BlkLoc(*glbl_argp))->pname));
 		     EntInterp;
 		     }
 		  }
@@ -1468,7 +1466,7 @@ efail_noev:
 		     goto efail_noev;
 
 		  case G_Psusp:		/* resuming a procedure */
-                     InterpEVValD(argp, E_Presum);
+                     InterpEVValD(glbl_argp, E_Presum);
 		     break;
 		  }
 
@@ -1480,7 +1478,7 @@ efail_noev:
 #ifdef EventMon
 	    ExInterp;
             vanq_proc(efp, gfp);
-            EVValD(argp, E_Pfail);
+            EVValD(glbl_argp, E_Pfail);
 	    EntInterp;
 #endif					/* EventMon */
 
@@ -1494,7 +1492,7 @@ efail_noev:
 	    --k_level;
 	    if (k_trace) {
                k_trace--;
-	       failtrace(&(((struct b_proc *)BlkLoc(*argp))->pname));
+	       failtrace(&(((struct b_proc *)BlkLoc(*glbl_argp))->pname));
                }
 Pfail_uw:
 
@@ -1510,7 +1508,7 @@ Pfail_uw:
 	    efp = pfp->pf_efp;
 	    gfp = pfp->pf_gfp;
 	    ipc = pfp->pf_ipc;
-	    argp = pfp->pf_argp;
+	    glbl_argp = pfp->pf_argp;
 	    pfp = pfp->pf_pfp;
 
 #ifdef MultiThread
@@ -1812,7 +1810,8 @@ interp_quit:
    --ilevel;
    if (ilevel != 0)
       syserr("interp: termination with inactive generators.");
-
+   /*NOTREACHED*/
+   return 0;	/* avoid gcc warning */
    }
 
 #ifdef StackPic
@@ -1830,7 +1829,7 @@ Deliberate Syntax Error
 #endif					/* AMIGA || ATARI_ST || ... */
 
 #if ARM
-novalue stkdump(op)
+void stkdump(op)
    int op;
    {
    word *stk;
@@ -1843,7 +1842,7 @@ novalue stkdump(op)
    fprintf(stderr,">  efp:    %.8x\n", (word)efp);
    fprintf(stderr,">  gfp:    %.8x\n", (word)gfp);
    fprintf(stderr,">  ipc:    %.8x\n", (word)ipc.op);
-   fprintf(stderr,">  argp:   %.8x\n", (word)argp);
+   fprintf(stderr,">  argp:   %.8x\n", (word)glbl_argp);
    fprintf(stderr,">  ilevel: %.8x\n", (word)ilevel);
    fprintf(stderr,">  op:     %d\n",    (int)op);
    for (i = stk; i <= (word *)sp; i++)
@@ -1855,7 +1854,7 @@ novalue stkdump(op)
 
 #if MSDOS || OS2
 #if MICROSOFT || TURBO || BORLAND_286 || BORLAND_386
-novalue stkdump(op)
+void stkdump(op)
    int op;
    {
    word far *stk;
@@ -1868,7 +1867,7 @@ novalue stkdump(op)
    fprintf(stderr,">  efp:    %08lx\n", (word)efp);
    fprintf(stderr,">  gfp:    %08lx\n", (word)gfp);
    fprintf(stderr,">  ipc:    %08lx\n", (word)ipc.op);
-   fprintf(stderr,">  argp:   %08lx\n", (word)argp);
+   fprintf(stderr,">  argp:   %08lx\n", (word)glbl_argp);
    fprintf(stderr,">  ilevel: %08lx\n", (word)ilevel);
    fprintf(stderr,">  op:     %d\n",    (int)op);
    for (i = stk; i <= (word far *)sp; i++)
@@ -1880,7 +1879,7 @@ novalue stkdump(op)
 #endif					/* MSDOS || OS2 */
 
 #if UNIX || VMS
-novalue stkdump(op)
+void stkdump(op)
    int op;
    {
    word *i;
@@ -1889,7 +1888,7 @@ novalue stkdump(op)
    fprintf(stderr,"\001efp: %lx\n",(long)efp);
    fprintf(stderr,"\001gfp: %lx\n",(long)gfp);
    fprintf(stderr,"\001ipc: %lx\n",(long)ipc.op);
-   fprintf(stderr,"\001argp: %lx\n",(long)argp);
+   fprintf(stderr,"\001argp: %lx\n",(long)glbl_argp);
    fprintf(stderr,"\001ilevel: %lx\n",(long)ilevel);
    fprintf(stderr,"\001op: \%d\n",(int)op);
    for (i = stack + Wsizeof(struct b_coexpr); i <= sp; i++)
@@ -1909,7 +1908,7 @@ novalue stkdump(op)
  * vanq_proc - monitor the removal of suspended operations from within
  *   a procedure.
  */
-static novalue vanq_proc(efp_v, gfp_v)
+static void vanq_proc(efp_v, gfp_v)
 struct ef_marker *efp_v;
 struct gf_marker *gfp_v;
    {
@@ -2038,7 +2037,7 @@ register struct b_coexpr *ncp;
 /*
  * activate the "&parent" co-expression from anywhere, if there is one
  */
-novalue actparent(event)
+void actparent(event)
 int event;
    {
    struct progstate *parent = curpstate->parent;
