@@ -146,7 +146,7 @@ continuation succ_cont;
 /*
  * invoke -- Perform setup for invocation.  
  */
-invoke(nargs,cargp,n)
+int invoke(nargs,cargp,n)
 dptr *cargp;
 int nargs, *n;
 {
@@ -162,7 +162,6 @@ int nargs, *n;
    register word i;
    struct b_proc *proc;
    int nparam;
-   char strbuf[MaxCvtLen];
 
    /*
     * Point newargp at Arg0 and dereference it.
@@ -330,7 +329,7 @@ int nargs, *n;
     */
    newpfp = (struct pf_marker *)(newsp + 1);
    newpfp->pf_nargs = nargs;
-   newpfp->pf_argp = argp;
+   newpfp->pf_argp = glbl_argp;
    newpfp->pf_pfp = pfp;
    newpfp->pf_ilevel = ilevel;
    newpfp->pf_scan = NULL;
@@ -343,7 +342,7 @@ int nargs, *n;
    newpfp->pf_prog = curpstate;
 #endif					/* MultiThread */
 
-   argp = newargp;
+   glbl_argp = newargp;
    pfp = newpfp;
    newsp += Vwsizeof(*pfp);
 
