@@ -6,8 +6,8 @@
 /*
  * Prototypes.
  */
-hidden struct region *findgap	Params((struct region *curr, word nbytes));
-hidden struct region *newregion	Params((word nbytes, word stdsize));
+static struct region *findgap	(struct region *curr, word nbytes);
+static struct region *newregion	(word nbytes, word stdsize);
 
 extern word alcnum;
 
@@ -24,8 +24,6 @@ word table_ser = 1;	/* serial numbers for tables */
  */
 #begdef AlcBlk(var, struct_nm, t_code, nbytes)
 {
-   struct region *rp = NULL;
-
 #ifdef MultiThread
    EVVal((word)nbytes, typech[t_code]);
 #endif					/* MultiThread */
@@ -66,7 +64,11 @@ word table_ser = 1;	/* serial numbers for tables */
  */
 #begdef AlcVarBlk(var, struct_nm, t_code, n_desc)
    {
+#ifdef EventMon
+   uword size;
+#else					/* EventMon */
    register uword size;
+#endif					/* EventMon */
 
    /*
     * Variable size blocks are declared with one descriptor, thus
@@ -193,7 +195,7 @@ struct b_coexpr *alccoexp()
    struct b_coexpr *ep;
 
 #ifdef MultiThread
-   if (icodesize>0) {
+   if (icodesize > 0) {
 #ifdef ATTM32
 Deliberate Syntax Error
 #else					/* ATTM32 */
@@ -254,7 +256,7 @@ Deliberate Syntax Error
    ep->es_argp = NULL;
    ep->tvalloc = NULL;
 
-   if (icodesize>0)
+   if (icodesize > 0)
       ep->id = 1;
    else
 #endif					/* MultiThread */
@@ -388,7 +390,7 @@ struct b_lelem *alclstb(nslots, first, nused)
 uword nslots, first, nused;
    {
    register struct b_lelem *blk;
-   register word i, size;
+   register word i;
 
    AlcVarBlk(blk, b_lelem, T_Lelem, nslots)
    blk->nslots = nslots;
@@ -440,7 +442,6 @@ union block *recptr;
    {
    tended union block *trecptr = recptr;
    register struct b_record *blk;
-   register int i;
 
    AlcVarBlk(blk, b_record, T_Record, nflds)
    blk->recdesc = trecptr;
@@ -473,7 +474,6 @@ struct b_refresh *alcrefresh(entryx, na, nl)
 word *entryx;
 int na, nl;
    {
-   int size;
    struct b_refresh *blk;
 
    AlcVarBlk(blk, b_refresh, T_Refresh, na + nl);
@@ -612,7 +612,7 @@ uword hashnum;
  *
  *  The block must be the one that is at the very end of a block region.
  */
-novalue deallocate (bp)
+void deallocate (bp)
 union block *bp;
 {
    word nbytes;
@@ -668,11 +668,7 @@ word nbytes;
    if (DiffPtrs(curr->end, curr->free) >= nbytes)
       return curr->free;		/* quick return: current region is OK */
 
-#if SCCX_MX
-   if( (rp = findgap(curr, nbytes)) != 0) {    /* check all regions on chain */
-#else					/* SCCX_MX */
-   if (rp = findgap(curr, nbytes)) {	/* check all regions on chain */
-#endif					/* SCCX_MX */
+   if ((rp = findgap(curr, nbytes)) != 0) {    /* check all regions on chain */
       *pcurr = rp;			/* switch regions */
       return rp->free;
       }
@@ -710,11 +706,7 @@ word nbytes;
    if (newsize < MinAbrSize)
       newsize = MinAbrSize;
 
-#if SCCX_MX
-   if( (rp = newregion(nbytes, newsize)) != 0) {
-#else					/* SCCX_MX */
-   if (rp = newregion(nbytes, newsize)) {
-#endif					/* SCCX_MX */
+   if ((rp = newregion(nbytes, newsize)) != 0) {
       rp->prev = curr;
       rp->next = NULL;
       curr->next = rp;
@@ -748,11 +740,7 @@ word nbytes;
          if (DiffPtrs(rp->end,rp->free) >= want)
             return rp->free;
          }
-#if SCCX_MX
-   if( (rp = findgap(curr, nbytes)) != 0) {
-#else					/* SCCX_MX */
-   if (rp = findgap(curr, nbytes)) {
-#endif					/* SCCX_MX */
+   if ((rp = findgap(curr, nbytes)) != 0) {
       *pcurr = rp;
       return rp->free;
       }
