@@ -6,22 +6,22 @@
 /*
  * Prototypes.
  */
-hidden int     glbcmp    Params((char *pi, char *pj));
-hidden int     keyref    Params((union block *bp, dptr dp));
-hidden novalue showline  Params((char *f, int l));
-hidden novalue showlevel Params((register int n));
-hidden novalue ttrace	Params((noargs));
-hidden novalue xtrace
-   Params((struct b_proc *bp, word nargs, dptr arg, int pline, char *pfile));
+static int     glbcmp    (char *pi, char *pj);
+static int     keyref    (union block *bp, dptr dp);
+static void showline  (char *f, int l);
+static void showlevel (register int n);
+static void ttrace	(void);
+static void xtrace
+   (struct b_proc *bp, word nargs, dptr arg, int pline, char *pfile);
 
 /*
  * tracebk - print a trace of procedure calls.
  */
 #ifdef PresentationManager
 /* have to add COMPILER support too */
-novalue tracebk(void *foo, dptr argp, HWND hwndMLE)
+void tracebk(void *foo, dptr argp, HWND hwndMLE)
 #else					/* PresentationManager */
-novalue tracebk(lcl_pfp, argp)
+void tracebk(lcl_pfp, argp)
 
 #if COMPILER
 struct p_frame *lcl_pfp;
@@ -112,7 +112,7 @@ dptr argp;
  * xtrace - procedure *bp is being called with nargs arguments, the first
  *  of which is at arg; produce a trace message.
  */
-static novalue xtrace(bp, nargs, arg, pline, pfile)
+static void xtrace(bp, nargs, arg, pline, pfile)
 struct b_proc *bp;
 word nargs;
 dptr arg;
@@ -169,21 +169,21 @@ int get_name(dp1,dp0)
    int t;
 
 #if COMPILER
-   arg1 = argp;
+   arg1 = glbl_argp;
    loc1 = pfp->tend.d;
    proc = PFDebug(*pfp)->proc;
 #else					/* COMPILER */
-   arg1 = &argp[1];
+   arg1 = &glbl_argp[1];
    loc1 = pfp->pf_locals;
-   proc = &BlkLoc(*argp)->proc;
+   proc = &BlkLoc(*glbl_argp)->proc;
 #endif					/* COMPILER */
 
    type_case *dp1 of {
       tvsubs: {
          blkptr = BlkLoc(*dp1);
          get_name(&(blkptr->tvsubs.ssvar),dp0);
-         sprintf(sbuf,"[%ld:%ld]",blkptr->tvsubs.sspos,
-            blkptr->tvsubs.sspos+blkptr->tvsubs.sslen);
+         sprintf(sbuf,"[%ld:%ld]",(long)blkptr->tvsubs.sspos,
+            (long)blkptr->tvsubs.sspos+blkptr->tvsubs.sslen);
          k = StrLen(*dp0);
          j = strlen(sbuf);
 
@@ -324,9 +324,9 @@ int get_name(dp1,dp0)
                      }
 #ifdef ListFix
                   sprintf(sbuf,"list_%d[%ld]",
-			  blkptr->lelem.listprev->list.id, i);
+			  (long)blkptr->lelem.listprev->list.id, (long)i);
 #else					/* ListFix */
-                  sprintf(sbuf,"L[%ld]", i);
+                  sprintf(sbuf,"L[%ld]", (long)i);
 #endif					/* ListFix */
                   i = strlen(sbuf);
                   Protect(StrLoc(*dp0) = alcstr(sbuf,i), return Error);
@@ -381,7 +381,7 @@ int get_name(dp1,dp0)
 /*
  * ctrace - a procedure is being called; produce a trace message.
  */
-novalue ctrace()
+void ctrace()
    {
    dptr arg;
    int n;
@@ -389,7 +389,7 @@ novalue ctrace()
    PTraceSetup();
 
    putc('(', stderr);
-   arg = argp;
+   arg = glbl_argp;
    n = abs((int)proc->nparam);
    while (n--) {
       outimage(stderr, arg++, 0);
@@ -405,7 +405,7 @@ novalue ctrace()
  * rtrace - a procedure is returning; produce a trace message.
  */
 
-novalue rtrace()
+void rtrace()
    {
    PTraceSetup();
 
@@ -419,7 +419,7 @@ novalue rtrace()
  * failtrace - procedure named s is failing; produce a trace message.
  */
 
-novalue failtrace()
+void failtrace()
    {
    PTraceSetup();
 
@@ -431,7 +431,7 @@ novalue failtrace()
  * strace - a procedure is suspending; produce a trace message.
  */
 
-novalue strace()
+void strace()
    {
    PTraceSetup();
 
@@ -444,7 +444,7 @@ novalue strace()
 /*
  * atrace - a procedure is being resumed; produce a trace message.
  */
-novalue atrace()
+void atrace()
    {
    PTraceSetup();
 
@@ -462,7 +462,7 @@ static int keyref(bp, dp)
    {
    char *s, *s2;
    char sbuf[100];			/* buffer; might be too small */
-   int i, len;
+   int len;
 
    if (getimage(&(bp->telem.tref),dp) == Error)
       return Error;	
@@ -498,7 +498,7 @@ static int keyref(bp, dp)
 /*
  * cotrace -- a co-expression context switch; produce a trace message.
  */
-novalue cotrace(ccp, ncp, swtch_typ, valloc)
+void cotrace(ccp, ncp, swtch_typ, valloc)
 struct b_coexpr *ccp;
 struct b_coexpr *ncp;
 int swtch_typ;
@@ -522,7 +522,7 @@ dptr valloc;
     */
    t_ipc.op = ipc.op - 1;
    showline(findfile(t_ipc.opnd), findline(t_ipc.opnd));
-   proc = (struct b_proc *)BlkLoc(*argp);
+   proc = (struct b_proc *)BlkLoc(*glbl_argp);
 #endif					/* COMPILER */
 
    showlevel(k_level);
@@ -551,7 +551,7 @@ dptr valloc;
 /*
  * showline - print file and line number information.
  */
-static novalue showline(f, l)
+static void showline(f, l)
 char *f;
 int l;
    {
@@ -584,7 +584,7 @@ int l;
 /*
  * showlevel - print "| " n times.
  */
-static novalue showlevel(n)
+static void showlevel(n)
 register int n;
    {
    while (n-- > 0) {
@@ -598,14 +598,14 @@ register int n;
 #include "../h/opdefs.h"
 
 
-extern struct b_list value_tmp;		/* argument of Op_Apply */
+extern struct descrip value_tmp;		/* argument of Op_Apply */
 extern struct b_proc *opblks[];
 
 
 /*
  * ttrace - show offending expression.
  */
-hidden novalue ttrace()
+static void ttrace()
    {
    struct b_proc *bp;
    word nargs;
@@ -711,7 +711,7 @@ hidden novalue ttrace()
       case Op_Apply:
          outimage(stderr, xargp++, 0);
          fprintf(stderr," ! ");
-         outimage(stderr, (dptr)&value_tmp, 0);
+         outimage(stderr, &value_tmp, 0);
          break;
 
       case Op_Create:
@@ -779,7 +779,7 @@ oneop:
  * ctrace - procedure named s is being called with nargs arguments, the first
  *  of which is at arg; produce a trace message.
  */
-novalue ctrace(dp, nargs, arg)
+void ctrace(dp, nargs, arg)
 dptr dp;
 int nargs;
 dptr arg;
@@ -803,7 +803,7 @@ dptr arg;
  * rtrace - procedure named s is returning *rval; produce a trace message.
  */
 
-novalue rtrace(dp, rval)
+void rtrace(dp, rval)
 dptr dp;
 dptr rval;
    {
@@ -826,7 +826,7 @@ dptr rval;
  * failtrace - procedure named s is failing; produce a trace message.
  */
 
-novalue failtrace(dp)
+void failtrace(dp)
 dptr dp;
    {
    inst t_ipc;
@@ -847,7 +847,7 @@ dptr dp;
  * strace - procedure named s is suspending *rval; produce a trace message.
  */
 
-novalue strace(dp, rval)
+void strace(dp, rval)
 dptr dp;
 dptr rval;
    {
@@ -870,7 +870,7 @@ dptr rval;
  * atrace - procedure named s is being resumed; produce a trace message.
  */
 
-novalue atrace(dp)
+void atrace(dp)
 dptr dp;
    {
    inst t_ipc;
@@ -891,14 +891,14 @@ dptr dp;
 /*
  * coacttrace -- co-expression is being activated; produce a trace message.
  */
-novalue coacttrace(ccp, ncp)
+void coacttrace(ccp, ncp)
 struct b_coexpr *ccp;
 struct b_coexpr *ncp;
    {
    struct b_proc *bp;
    inst t_ipc;
 
-   bp = (struct b_proc *)BlkLoc(*argp);
+   bp = (struct b_proc *)BlkLoc(*glbl_argp);
    /*
     * Compute the ipc of the activation instruction.
     */
@@ -915,14 +915,14 @@ struct b_coexpr *ncp;
 /*
  * corettrace -- return from co-expression; produce a trace message.
  */
-novalue corettrace(ccp, ncp)
+void corettrace(ccp, ncp)
 struct b_coexpr *ccp;
 struct b_coexpr *ncp;
    {
    struct b_proc *bp;
    inst t_ipc;
 
-   bp = (struct b_proc *)BlkLoc(*argp);
+   bp = (struct b_proc *)BlkLoc(*glbl_argp);
    /*
     * Compute the ipc of the coret instruction.
     */
@@ -939,14 +939,14 @@ struct b_coexpr *ncp;
 /*
  * cofailtrace -- failure return from co-expression; produce a trace message.
  */
-novalue cofailtrace(ccp, ncp)
+void cofailtrace(ccp, ncp)
 struct b_coexpr *ccp;
 struct b_coexpr *ncp;
    {
    struct b_proc *bp;
    inst t_ipc;
 
-   bp = (struct b_proc *)BlkLoc(*argp);
+   bp = (struct b_proc *)BlkLoc(*glbl_argp);
    /*
     * Compute the ipc of the cofail instruction.
     */
