@@ -20,12 +20,18 @@
    /* place for anything system-specific */
 Deliberate Syntax Error
 #endif					/* PORT */
+
 
+/*********************************** AMIGA ***********************************/
+
 #if AMIGA
 #if LATTICE
 long _STACK = 20000;
 long _MNEED = 200000;	/* reserve space for allocation (may be too large) */
 #endif					/* LATTICE */
+#if __SASC
+long __stack = 20000;
+#endif					/* __SASC */
 #if AZTEC_C
 /*
  * abs
@@ -54,52 +60,39 @@ double value;
 /*
  *  abort()
  */
-novalue abort()
+void abort()
 {
   fprintf(stderr,"icon error with ICONCORE set\n");
   fflush(stderr);
   exit(1);
 }
 
-#ifdef SystemFnc
-
-/*
- * Aztec C version 3.6 does not support system(), but here is a substitute.
- */
-
-#define KLUDGE1 256
-#define KLUDGE2 64
-int system(s)
-char *s;
-{
-   char text[KLUDGE1], *cp=text;
-   char *av[KLUDGE2];
-   int ac = 0;
-   int l  = strlen(s);
-
-   if (l >= KLUDGE1)
-      return -1;
-   strcpy(text,s);
-   av[ac++] = text;
-   while(*cp && ac<KLUDGE2-1) {
-      if (isspace(*cp)) {
-         *cp++ = '\0';
-	 while(isspace(*cp))
-	    cp++;
-         if (*cp)
-	    av[ac++] = cp;
-         }
-      else {
-         cp++;
-         }
-      }
-    av[ac] = NULL;
-    return fexecv(av[0], av);
-}
-#endif					/* SystemFnc */
 #endif					/* AZTEC_C */
+
+/* Implement popen and pclose using Per Bojsen's APipe-Handler */
+
+FILE *popen(const char *pname, const char *mode) {
+
+   char *exname;
+   char device[] = "APIPE:";
+   FILE *fp;
+
+   exname = malloc(strlen(pname) + strlen(device) + 1);
+   strcpy(exname, device);
+   strcat(exname, pname);
+   fp = fopen(exname, mode);
+   free(exname);
+   return fp;
+   }
+
+int pclose(FILE *pipe) {
+   return fclose(pipe);
+   }
+
 #endif					/* AMIGA */
 
+/*********************************** ARM ***********************************/
+
 #if ARM
 #include "kernel.h"
 
@@ -689,13 +682,6 @@ int main (int argc, char *argv[])
 }
 #endif
 
-int unlink (const char *name)
-{
-	_kernel_osfile_block blk;
-
-	return (_kernel_osfile(6,name,&blk) <= 0);
-}
-
 int getch (void)
 {
 	return _kernel_osrdch();
@@ -717,6 +703,8 @@ int kbhit (void)
 
 #endif					/* ARM */
 
+/*********************************** ATARI ***********************************/
+
 #if ATARI_ST
 #if LATTICE
 
@@ -805,6 +793,8 @@ char *p;
 #endif					/* LATTICE */
 #endif					/* ATARI_ST */
 
+/*********************************** MAC ***********************************/
+
 #if MACINTOSH
 #if MPW
 /*
@@ -812,8 +802,6 @@ char *p;
  *  of the Icon Programming Language
  */
 
-#include <stdlib.h>
-#include <stdio.h>
 #include <Types.h>
 #include <Events.h>
 #include <Files.h>
@@ -963,98 +951,7 @@ MacDelay(int n) {
 #endif					/* MPW */
 #endif					/* MACINTOSH */
 
-#if MSDOS
-#if NT
-/*
- * header comment says not to do this, but the "right" way doesn't work
- */
-#passthru #include <direct.h>
-#endif					/* NT */
-
-int pathFind(char target[], char buf[], int n)
-   {
-   char *path;
-   register int i;
-   int res;
-   struct stat sbuf;
-
-   if ((path = getenv("PATH")) == 0)
-      path = "";
-
-   if (!getcwd(buf, n)) {		/* get current working directory */
-      *buf = 0;		/* may be better to do something nicer if we can't */
-      return 0;		/* find out where we are -- struggling to achieve */
-      }			/* something can be better than not trying */
-
-   /* attempt to find the icode file in the current directory first */
-   /* this mimicks the behavior of COMMAND.COM */
-   if ((i = strlen(buf)) > 0) {
-      i = buf[i - 1];
-      if (i != '\\' && i != '/' && i != ':')
-         strcat(buf, "/");
-      }
-   strcat(buf, target);
-   res = stat(buf, &sbuf);
-
-   while(res && *path) {
-      for (i = 0; *path && *path != ';'; ++i)
-         buf[i] = *path++;
-      if (*path)			/* skip the ; or : separator */
-         ++path;
-      if (i == 0)			/* skip empty fragments in PATH */
-         continue;
-      if (i > 0 && buf[i - 1] != '/' && buf[i - 1] != '\\' &&
-         buf[i - 1] != ':')
-            buf[i++] = '/';
-      strcpy(buf + i, target);
-      res = stat(buf, &sbuf);
-      /* exclude directories (and any other nasties) from selection */
-      if (res == 0 && sbuf.st_mode & S_IFDIR)
-         res = -1;
-      }
-   if (res != 0)
-      *buf = 0;
-   return res == 0;
-   }
-#endif					/* MSDOS */
-
-#if MSDOS || OS2
-FILE *pathOpen(fname, mode)
-   char *fname;
-   char *mode;
-   {
-#if OS2
-   char buf[260 + 1];
-#else					/* OS2 */
-   char buf[150 + 1];
-#endif					/* OS2 */
-   int i, use = 1;
-
-#if SCCX_MX
-   /* Avoid compiler warning */
-   for( i = 0; (buf[i] = fname[i]) != 0; ++i)
-#else
-   for( i = 0; buf[i] = fname[i]; ++i)
-#endif					/* SCCX_MX */
-
-      /* find out if a path has been given in the file name */
-      if (buf[i] == '/' || buf[i] == ':' || buf[i] == '\\')
-         use = 0;
-
-   /* If a path has been given with the file name, don't bother to
-      use the PATH */
-
-#if OS2
-   if (use && DosSearchPath(SEARCH_CUR_DIRECTORY | SEARCH_ENVIRONMENT, 
-                            "PATH", fname, buf, 260))
-#else					/* OS2 */
-   if (use && !pathFind(fname, buf, 150))
-#endif 					/* OS2 */
-       return 0;
-
-   return fopen(buf, mode);
-   }
-#endif					/* MSDOS || OS2 */
+/*********************************** MSDOS ***********************************/
 
 #if MSDOS
 #if INTEL_386
@@ -1110,6 +1007,8 @@ extern unsigned _stklen = 16 * 1024;
 
 #endif					/* MSDOS */
 
+/********************************* MVS || VM ********************************/
+
 #if MVS || VM
 #if SASC 
 #passthru #include <options.h>
@@ -1136,8 +1035,10 @@ char *_style = "tso:";          /* use dsnames as file names */
 #endif                                  /* SASC */
 #endif                                  /* MVS || VM */
 
+/*********************************** OS/2 ***********************************/
+
 #if OS2
-novalue abort()
+void abort()
 {
 #ifdef DeBugIconx
     blkdump();
@@ -1249,12 +1150,91 @@ FILE *ptr;
 /* End of pipe support for OS/2 */
 #endif					/* OS2 */
 
+/*********************************** UNIX ***********************************/
+
 #if UNIX
-#ifdef ATTM32
+
+#ifdef KeyboardFncs
 
 /*
- * This file contains the routine necessary to allocate legal AT&T
- * 3B2/15/4000 stack space for co-expression stacks.
+ * Documentation notwithstanding, the Unix versions of the keyboard functions
+ * read from standard input and not necessarily from the keyboard (/dev/tty).
+ */
+#define STDIN 0
+
+/*
+ * int getch() -- read character without echoing
+ * int getche() -- read character with echoing
+ *
+ * Read and return a character from standard input in non-canonical
+ * ("cbreak") mode.  Return -1 for EOF.
+ *
+ * Reading is done even if stdin is not a tty;
+ * the tty get/set functions are just rejected by the system.
+ */
+
+int rchar(int with_echo);
+
+int getch(void)		{ return rchar(0); }
+int getche(void)	{ return rchar(1); }
+
+int rchar(int with_echo)
+{
+   struct termios otty, tty;
+   char c;
+   int n;
+
+   tcgetattr(STDIN, &otty);		/* get current tty attributes */
+
+   tty = otty;
+   tty.c_lflag &= ~ICANON;
+   if (with_echo)
+      tty.c_lflag |= ECHO;
+   else
+      tty.c_lflag &= ~ECHO;
+   tcsetattr(STDIN, TCSANOW, &tty);	/* set temporary attributes */
+
+   n = read(STDIN, &c, 1);		/* read one char from stdin */
+
+   tcsetattr(STDIN, TCSANOW, &otty);	/* reset tty to original state */
+
+   if (n == 1)				/* if read succeeded */
+      return c & 0xFF;
+   else
+      return -1;
+}
+
+/*
+ * kbhit() -- return nonzero if characters are available for getch/getche.
+ */
+int kbhit(void)
+{
+   struct termios otty, tty;
+   fd_set fds;
+   struct timeval tv;
+   int rv;
+
+   tcgetattr(STDIN, &otty);		/* get current tty attributes */
+
+   tty = otty;
+   tty.c_lflag &= ~ICANON;		/* disable input batching */
+   tcsetattr(STDIN, TCSANOW, &tty);	/* set attribute temporarily */
+
+   FD_ZERO(&fds);			/* initialize fd struct */
+   FD_SET(STDIN, &fds);			/* set STDIN bit */
+   tv.tv_sec = tv.tv_usec = 0;		/* set immediate return */
+   rv = select(STDIN + 1, &fds, NULL, NULL, &tv);
+
+   tcsetattr(STDIN, TCSANOW, &otty);	/* reset tty to original state */
+
+   return rv;				/* return result */
+}
+#endif					/* KeyboardFncs */
+
+#ifdef ATTM32
+/*
+ * coexp_salloc() - return pointer in legal stack space for start
+ *                  of a coexpression stack on AT&T 3B2/15/400 systems.
  *
  * Legal stack region begins at 0xC0020000, and UNIX will grow stack space
  * up to 50 Megabytes. 0xC0030000 should provide plenty of room for
@@ -1262,12 +1242,6 @@ FILE *ptr;
  * adds mstksize (max main stack size) and returns a new address,
  * meaning each coexpression stack is potentially as large as the main stack.
  */
-
-/*
- * coexp_salloc() - return pointer in legal stack space for start
- *                  of a coexpression stack.
- */
-
 pointer coexp_salloc()
    {
    static pointer sp = 0xC0030000 ;     /* pointer to stack region */
@@ -1277,278 +1251,9 @@ pointer coexp_salloc()
 }
 #endif					/* ATTM32 */
 
-#ifdef KeyboardFncs
-
-#begdef CopyTty(t1,t2)
-   if (!reset_flag) {
-      t2 = t1;
-      reset_flag = 1;
-      }
-#enddef
-
-#begdef ResetTty(t1, theIOCTL)
-   if (reset_flag) {
-      if (ioctl(0, theIOCTL, t1) == -1) {
-         keyboard_error = 214;
-         return 0;
-         }
-      }
-#enddef
-
-#define ECHO_ON 1
-#define ECHO_OFF 0
-
-
-#ifdef HaveTermio
-
-#define RawIOCTL   TCGETA
-#define ResetIOCTL TCSETA
-#define TtyFlags(t)   (t).c_lflag
-
-#else				/* not HaveTermio */
-#ifdef HaveTioc
-
-#define ResetIOCTL TIOCSETN
-#define RawIOCTL   TIOCGETP
-#define TtyFlags(t)   (t).sg_flags
-#endif				/* HaveTioc */
-#endif				/* HaveTermio */
-
-/*
- * read_a_char(turn_echo_on): int
- *
- * Routine to actually do the reading (either with echo or without,
- * depending on whether turn_echo_on is 1 or 0).
- */
-int read_a_char(turn_echo_on)
-int turn_echo_on;
-{
-   char c;
-#ifdef HaveTermio
-   struct termio tty, new_tty;
-   novalue abort_on_signal();
-#else					/* HaveTermio */
-   struct tchars tty_characters;
-   struct sgttyb tty, new_tty;
-#endif					/* HaveTermio */
-   register word status, isa_tty = 0, reset_flag = 0;
-   extern int errno;
-  
-   if (isatty(0)) {
-
-      isa_tty = 1;
-      if (ioctl(0, RawIOCTL, &tty) == -1) {
-	 keyboard_error = 214;
-	 return 0;
-         }
-#ifdef HaveTioc
-      if (ioctl(0, TIOCGETC, &tty_characters) == -1) {
-	 keyboard_error = 214;
-	 return 0;
-         }
-#endif					/* HaveTioc */
-#ifdef HaveTermio
-      /* disable keyboard signals quit & interrupt */
-      if (tty.c_lflag & ISIG) {
-	 CopyTty(tty, new_tty);	/* a macro, defined above */
-	 new_tty.c_lflag &= ~ISIG;
-         }
-      /* disable canonical input processing (like BSD cbreak) */
-      if (tty.c_lflag & ICANON) {
-	 CopyTty(tty, new_tty);
-	 new_tty.c_lflag &= ~ICANON;
-         }
-      if (tty.c_cc[VMIN] != '\1') {
-	 CopyTty(tty, new_tty);
-	 new_tty.c_cc[VMIN] = (unsigned char )'\1';
-         }
-      if (tty.c_cc[VTIME]) {
-	 CopyTty(tty, new_tty);
-	 new_tty.c_cc[VTIME] = (unsigned char )'\0';
-         }
-#endif					/* HaveTermio */
-      if (turn_echo_on) {
-	 /* set echo bit, i.e. enable echo */
-	 if (! (TtyFlags(tty) & ECHO)) {
-	    CopyTty(tty, new_tty);
-	    TtyFlags(new_tty) |= ECHO;
-	    }
-         }
-      else { /* i.e. if _not_ turn_echo_on */
-	 /* mask out echo bit, i.e. disable echo */
-	if (TtyFlags(tty) & ECHO) {
-	   CopyTty(tty, new_tty);
-	   TtyFlags(new_tty) &= ~ECHO;
-	   }
-        }
-#ifdef HaveTioc
-      /* raw mode; we'll process quit and interrupt by hand */
-      if (! (tty.sg_flags & RAW)) {
-	 CopyTty(tty, new_tty);
-	 new_tty.sg_flags |= RAW;
-         }
-#endif					/* HaveTioc */
-      ResetTty(&new_tty, ResetIOCTL);		/* a macro, defined above */
-      }
-
-   /* finally, read 1 char from the standard input */
-   status = read(0, &c, 1);
-   if (isa_tty)
-      ResetTty(&tty, ResetIOCTL);
-   if (status == -1) {
-      switch (errno) {
-      case EBADF:
-	 keyboard_error = 212;
-	 return 0;
-      default:
-	 keyboard_error = 214;
-	 return 0;
-         }
-      }
-
-   /* Check for quit and interrupt characters. */
-   if (isa_tty) {
-#ifdef HaveTermio
-      if ((unsigned char)c == tty.c_cc[VINTR]) {
-#else					/* HaveTermio */
-      if ((char )c == tty_characters.t_intrc) {
-#endif					/* HaveTermio */
-	 if (kill(getpid(), SIGINT) == -1) {
-	    perror("kill");
-	    keyboard_error = 500;
-	    return 0;
-	    }
-         }
-#ifdef HaveTermio
-      else if ((unsigned char)c == tty.c_cc[VQUIT]) {
-#else					/* HaveTermio */
-      else if ((char )c == tty_characters.t_quitc) {
-#endif					/* HaveTermio */
-	 if (kill(getpid(), SIGQUIT) == -1) {
-	    perror("kill");
-	    keyboard_error = 500;
-	    return 0;
-	    }
-         }
-      }
- 
-   if (! status) return -1;
-   else return (int)c;
-}
-
-/*
- * getch(): int
- *
- * Routine to read one char from the standard input.  Enables cbreak mode
- * so as to read a character right from the buffer, without having to wait
- * for a carriage return.  Disables cbreak mode after reading a character.
- * Note that getch() does not echo any characters to the screen, although
- * characters might appear on the screen anyway, if they were typed before
- * getch() was invoked.
- */
-int getch()
-{
-   int read_a_char();
-   return read_a_char(ECHO_OFF);
-}
-
-/*
- * getche(): int
- *
- * Routine to read one char from the standard input.  Enables cbreak mode,
- * to read the character right from the buffer, without having to wait for
- * a carriage return.  Disables cbreak mode after reading a character.  NOTE:
- * Getche() does not disable echoing, so anything typed after it is invoked
- * will be echoed to the screen (unlike getch(), which disables echoing; see
- * above).
- */
-int getche()
-{
-   int read_a_char();
-   return read_a_char(ECHO_ON);
-}
-
-/*
- * kbhit(): int
- *
- * Routine to check for the availability of characters on the stdin
- * stream.  Does not actually read any characters.  Returns nonzero
- * value if characters are waiting; otherwise, zero.  The idea here,
- * as in getch() and getche(), is not to touch the tty settings
- * unless we have to.
- */
-int kbhit()
-{
-   word arg;
-#ifdef HaveTermio
-   struct termio tty, new_tty;
-#else					/* HaveTermio */
-   struct sgttyb tty, new_tty;
-#endif					/* HaveTermio */
-   register word status, isa_tty = 0, reset_flag = 0;
-   extern int errno;
-
-   if (isatty(0)) {
-
-      isa_tty = 1;
-      if (ioctl(0, RawIOCTL, &tty) == -1) {
-	 keyboard_error = 214;
-	 return 0;
-         }
-#ifdef HaveTermio
-      if (tty.c_lflag & ICANON) {
-	 CopyTty(tty, new_tty);
-	 new_tty.c_lflag &= ~ICANON;
-         }
-#else					/* HaveTermio */
-      /* Some Sun4s need this */
-      if (tty.sg_flags & ECHO) {
-	 CopyTty(tty, new_tty);
-	 new_tty.sg_flags &= ~ECHO;
-         }
-      /* enable cbreak mode */
-      if (! (tty.sg_flags & CBREAK)) {
-	 CopyTty(tty, new_tty);
-	 new_tty.sg_flags |= CBREAK;
-         }
-#endif					/* HaveTermio */
-      ResetTty(&new_tty, ResetIOCTL);
-      }
-
-   /* see if anything is waiting to be read from the file */
-#ifdef HaveTioc
-   status = ioctl(0, FIONREAD, &arg);
-#endif					/* HaveTioc */
-#ifdef HaveTermio
-#ifndef Linux
-#ifdef Xenix386
-   status = rdchk(0);
-#else					/* Xenix386 */
-   status = ioctl(0, FIORDCHK, &arg);
-#endif					/* Xenix386 */
-#else
-   status = ioctl(0, FIONREAD, &arg);
-#endif					/* Linux */
-#endif					/* HaveTermio */
-   if (isa_tty) ResetTty(&tty, ResetIOCTL);
-   if (status == -1) {
-      switch (errno) {
-      case EBADF:
-	 keyboard_error = 212;
-	 return 0;
-      default:
-	 keyboard_error = 214;
-	 return 0;
-         }
-      }
-   return arg;
-}
-
-#endif				/* KeyboardFncs */
-
-
 #endif					/* UNIX */
 
+/*********************************** VMS ***********************************/
 
 #if VMS
 #passthru #define LIB_GET_EF     LIB$GET_EF
@@ -1811,7 +1516,7 @@ FILE *fp;
       fp = freopen(argv[n]+i,mode,fp);
    if (fp == NULL)  {			/* abort on error */
       fprintf(stderr,"%%can't open %s",argv[n]+i);
-      exit(ErrorExit);
+      exit(EXIT_FAILURE);
    }
    for ( ;  n < *argc;  n++)		/* move down following arguments */
       argv[n] = argv[n+1];
@@ -1862,12 +1567,12 @@ return_char:
    return (word)char_typed;
 }
 
-word getch()
+int getch()
 {
    return read_a_char(0);
 }
 
-word getche()
+int getche()
 {
    return read_a_char(1);
 }
@@ -1895,4 +1600,4 @@ int kbhit()
  * End of operating-system specific code.
  */
 
-static char x;			/* avoid empty module */
+static char xjunk;			/* avoid empty module */
