@@ -11,21 +11,21 @@
 /*
  * Prototypes
  */
-hidden novalue postqual		Params((dptr dp));
-hidden novalue markblock	Params((dptr dp));
-hidden novalue markptr		Params((union block **ptr));
-hidden novalue sweep		Params((struct b_coexpr *ce));
-hidden novalue sweep_stk	Params((struct b_coexpr *ce));
-hidden novalue reclaim		Params((noargs));
-hidden novalue cofree		Params((noargs));
-hidden novalue scollect		Params((word extra));
-hidden int     qlcmp		Params((dptr  *q1,dptr  *q2));
-hidden novalue adjust		Params((char *source, char *dest));
-hidden novalue compact		Params((char *source));
-hidden novalue mvc		Params((uword n, char *src, char *dest));
+static void postqual		(dptr dp);
+static void markblock	(dptr dp);
+static void markptr		(union block **ptr);
+static void sweep		(struct b_coexpr *ce);
+static void sweep_stk	(struct b_coexpr *ce);
+static void reclaim		(void);
+static void cofree		(void);
+static void scollect		(word extra);
+static int     qlcmp		(dptr  *q1,dptr  *q2);
+static void adjust		(char *source, char *dest);
+static void compact		(char *source);
+static void mvc		(uword n, char *src, char *dest);
 
 #ifdef MultiThread
-hidden novalue markprogram	Params((struct progstate *pstate));
+static void markprogram	(struct progstate *pstate);
 #endif					/*MultiThread*/
 
 /*
@@ -255,15 +255,15 @@ uword segsize[] = {
  */
 
 #if COMPILER
-novalue initalloc()
+void initalloc()
    {
 
 #else					/* COMPILER */
 #ifdef MultiThread
-novalue initalloc(codesize,p)
+void initalloc(codesize,p)
 struct progstate *p;
 #else					/* MultiThread */
-novalue initalloc(codesize)
+void initalloc(codesize)
 #endif					/* MultiThread */
 word codesize;
    {
@@ -515,7 +515,7 @@ int region;
    else if (Pointer(d))\
       markblock(&(d));
 
-static novalue markprogram(pstate)
+static void markprogram(pstate)
 struct progstate *pstate;
    {
    struct descrip *dp;
@@ -567,7 +567,7 @@ struct progstate *pstate;
  *  are ignored.
  */
 
-static novalue postqual(dp)
+static void postqual(dp)
 dptr dp;
    {
    char *newqual;
@@ -608,7 +608,7 @@ dptr dp;
  *  back-list of descriptors pointing to that block. (Phase I of garbage
  *  collection.)
  */
-static novalue markblock(dp)
+static void markblock(dp)
 dptr dp;
    {
    register dptr dp1;
@@ -814,7 +814,7 @@ dptr dp;
  *  is just a block pointer, not a descriptor.
  */
 
-static novalue markptr(ptr)
+static void markptr(ptr)
 union block **ptr;
    {
    register dptr dp;
@@ -950,7 +950,7 @@ union block **ptr;
  *  marking the descriptors.
  */
 
-static novalue sweep(ce)
+static void sweep(ce)
 struct b_coexpr *ce;
    {
    register struct tend_desc *tp;
@@ -986,14 +986,14 @@ struct b_coexpr *ce;
  *  the "if it can't be identified it's a descriptor" methodology.
  */
 
-static novalue sweep_stk(ce)
+static void sweep_stk(ce)
 struct b_coexpr *ce;
    {
    register word *s_sp;
    register struct pf_marker *fp;
    register struct gf_marker *s_gfp;
    register struct ef_marker *s_efp;
-   word nargs, type, gsize;
+   word nargs, type = 0, gsize = 0;
 
    fp = ce->es_pfp;
    s_gfp = ce->es_gfp;
@@ -1094,7 +1094,7 @@ struct b_coexpr *ce;
  *   phase has already been completed.
  */
 
-static novalue reclaim()
+static void reclaim()
    {
    /*
     * Collect available co-expression blocks.
@@ -1125,7 +1125,7 @@ static novalue reclaim()
  *  in their type field.
  */
 
-static novalue cofree()
+static void cofree()
    {
    register struct b_coexpr **ep, *xep;
    register struct astkblk *abp, *xabp;
@@ -1183,7 +1183,7 @@ static novalue cofree()
  *  descriptors rather than pointers to them.
  */
 
-static novalue scollect(extra)
+static void scollect(extra)
 word extra;
    {
    register char *source, *dest;
@@ -1274,7 +1274,7 @@ dptr *q1, *q2;
  *  collection.)
  */
 
-static novalue adjust(source,dest)
+static void adjust(source,dest)
 char *source, *dest;
    {
    register union block **nxtptr, **tptr;
@@ -1309,7 +1309,7 @@ char *source, *dest;
  *  collection.)
  */
 
-static novalue compact(source)
+static void compact(source)
 char *source;
    {
    register char *dest;
@@ -1349,13 +1349,13 @@ char *source;
 /*
  * mvc - move n bytes from src to dest
  *
- *      The algorithm is to copy the data (using memcopy) in the largest
+ *      The algorithm is to copy the data (using memcpy) in the largest
  * chunks possible, which is the size of area of the source data not in
  * the destination area (ie non-overlapped area).  (Chunks are expected to
  * be fairly large.)
  */
 
-static novalue mvc(n, src, dest)
+static void mvc(n, src, dest)
 uword n;
 register char *src, *dest;
    {
@@ -1370,7 +1370,7 @@ register char *src, *dest;
    destend = dest + n;   /* point at byte after dest area */
 
    if ((destend <= src) || (srcend <= dest))  /* not overlapping */
-      memcopy(dest,src,n);
+      memcpy(dest,src,n);
 
    else {                     /* overlapping data areas */
       if (dest < src) {
@@ -1382,7 +1382,7 @@ register char *src, *dest;
          /* now loop round copying copy_size chunks of data */
 
          do {
-            memcopy(dest,src,copy_size);
+            memcpy(dest,src,copy_size);
             dest = src;
             src = src + copy_size;
             }
@@ -1393,7 +1393,7 @@ register char *src, *dest;
          /* copy final fragment of data - if there is one */
 
          if (left_over > 0)
-            memcopy(dest,src,left_over);
+            memcpy(dest,src,left_over);
          }
 
       else if (dest > src) {
@@ -1407,7 +1407,7 @@ register char *src, *dest;
          do {
             destend = srcend;
             srcend  = srcend - copy_size;
-            memcopy(destend,srcend,copy_size);
+            memcpy(destend,srcend,copy_size);
             }
          while (DiffPtrs(srcend,src) > copy_size);
 
@@ -1415,7 +1415,7 @@ register char *src, *dest;
 
          /* copy intial fragment of data - if there is one */
 
-         if (left_over > 0) memcopy(dest,src,left_over);
+         if (left_over > 0) memcpy(dest,src,left_over);
          }
 
       } /* end of overlapping data area code */
@@ -1430,7 +1430,7 @@ register char *src, *dest;
  * descr - dump a descriptor.  Used only for debugging.
  */
 
-novalue descr(dp)
+void descr(dp)
 dptr dp;
    {
    int i;
@@ -1462,7 +1462,7 @@ dptr dp;
  *   NOTE:  Not adapted for multiple regions.
  */
 
-novalue blkdump()
+void blkdump()
    {
    register char *blk;
    register word type, size, fdesc;
