@@ -1,6 +1,6 @@
 /*
  * File: rmisc.r
- *  Contents: deref, eq, [gcvt], getvar, hash, outimage,
+ *  Contents: deref, eq, getvar, hash, outimage,
  *  qtos, pushact, popact, topact, [dumpact], 
  *  findline, findipc, findfile, doimage, getimage
  *  printable, sig_rsm, cmd_line, varargs.
@@ -12,10 +12,10 @@
  * Prototypes.
  */
 
-hidden novalue	listimage
-   Params((FILE *f,struct b_list *lp, int restrict));
-hidden novalue	printimage	Params((FILE *f,int c,int q));
-hidden char *	csname		Params((dptr dp));
+static void	listimage
+   (FILE *f,struct b_list *lp, int noimage);
+static void	printimage	(FILE *f,int c,int q);
+static char *	csname		(dptr dp);
 
 
 /* 
@@ -37,155 +37,6 @@ dptr d1, d2;
 	return 1;
 }
 
-#ifdef IconEcvt
-/*
- *  Convert floating-point number to a string.
- */
-
-char *ecvt(double number, int ndigit, int *decpt, int *sign)
-{
-	int n = 0;
-	static char buf[30];
-
-	/* Sort out the sign */
-	if (number >= 0)
-		*sign = 0;
-	else
-	{
-		*sign = 1;
-		number = -number;
-	}
-
-	/* Normalize the number to 0.1 <= number < 1, setting decpt */
-	if (number >= 1)
-	{
-		while (number >= 1)
-		{
-			++n;
-			number /= 10.0;
-		}
-	}
-	else if (number != 0.0 && number < 0.1)
-	{
-		while (number < 0.1)
-		{
-			--n;
-			number *= 10.0;
-		}
-	}
-	*decpt = n;
-
-	sprintf(buf, "%#.*f", ndigit, number);
-
-	/* Skip the leading "0." */
-	return (buf+2);
-}
-#endif					/* IconEcvt */
-
-#ifdef IconGcvt
-/*
- * gcvt - Convert number to a string in buf.  If possible, ndigit
- *  significant digits are produced, otherwise a form with an exponent is used.
- *
- */
-char *gcvt(number, ndigit, buf)
-double number;
-#ifndef Linux
-int ndigit;
-#else					/* Linux */
-unsigned int ndigit;
-#endif					/* Linux */
-char *buf;
-   {
-   int sign, decpt;
-   register char *p1, *p2;
-   register i;
-
-#if AMIGA
-#if AZTEC_C
-   /* shameless kludge around C library deficiency */
-#define ecvt(w,x,y,z) 0
-
-   ftoa(number,buf,ndigit,2);			/* 2 is MANX for "G" format */
-   for(i=strlen(buf); i>1 && buf[i-1]=='0' && buf[i-2]!='.'; i--)
-      buf[i-1] = '\0';
-   return buf;
-#endif					/* AZTEC_C */
-#endif					/* AMIGA */
-
-#ifdef OS2EMX
-   /* shameless kludge around C library deficiency */
-#define ecvt(w,x,y,z) 0
-
-   sprintf(buf, "%f", number);
-   for(i=strlen(buf); i>1 && buf[i-1]=='0' && buf[i-2]!='.'; i--)
-      buf[i-1] = '\0';
-   return buf;
-#endif					/* OS2EMX */
-
-#if FreeBSD
-#define ecvt(w,x,y,z) 0
-
-   sprintf(buf, "%f", number);
-   for(i=strlen(buf); i>1 && buf[i-1]=='0' && buf[i-2]!='.'; i--)
-      buf[i-1] = '\0';
-   return buf;
-#endif					/* FreeBSD */
-
-   p1 = ecvt(number, ndigit, &decpt, &sign);
-   p2 = buf;
-   if (sign)
-      *p2++ = '-';
-   for (i=ndigit-1; i>0 && p1[i]=='0'; i--)
-      ndigit--;
-   if (decpt >= 0 && decpt-(int)ndigit > 4
-      || decpt < 0 && decpt < -3) { /* use E-style */
-         decpt--;
-         *p2++ = *p1++;
-         *p2++ = '.';
-         for (i=1; i<ndigit; i++)
-            *p2++ = *p1++;
-         *p2++ = 'e';
-         if (decpt<0) {
-            decpt = -decpt;
-            *p2++ = '-';
-            }
-         else
-            *p2++ = '+';
-         if (decpt/100 > 0)
-            *p2++ = decpt/100 + '0';
-         if (decpt/10 > 0)
-            *p2++ = (decpt%100)/10 + '0';
-         *p2++ = decpt%10 + '0';
-      } else {
-         if (decpt<=0) {
-         /* if (*p1!='0') */
-         *p2++ = '0';
-         *p2++ = '.';
-         while (decpt<0) {
-            decpt++;
-            *p2++ = '0';
-            }
-         }
-         for (i=1; i<=ndigit; i++) {
-            *p2++ = *p1++;
-            if (i==decpt)
-               *p2++ = '.';
-            }
-      if (ndigit<decpt) {
-         while (ndigit++<decpt)
-            *p2++ = '0';
-         *p2++ = '.';
-         }
-   }
-   if (p2[-1]=='.')
-      *p2++ = '0';
-   *p2 = '\0';
-
-   return(buf);
-   }
-#endif					/* IconGcvt */
-
 /*
  * Get variable descriptor from name.  Returns the (integer-encoded) scope
  *  of the variable (Succeeded for keywords), or Failed if the variable
@@ -307,7 +158,7 @@ int getvar(s,vp)
       goto glbvars;
 #endif					/* !COMPILER */
 
-   dp = argp;
+   dp = glbl_argp;
 #if COMPILER
    bp = PFDebug(*pfp)->proc;  /* get address of procedure block */
 #else					/* COMPILER */
@@ -502,7 +353,6 @@ dptr dp;
 	 case T_Proc:
 	    dp = &(BlkLoc(*dp)->proc.pname);
 	    goto hashstring;
-	    break;
 
          default:
             /*
@@ -522,14 +372,14 @@ dptr dp;
 #define ListLimit	 6		/* limit on list items in image */
 
 /*
- * outimage - print image of *dp on file f.  If restrict is nonzero,
+ * outimage - print image of *dp on file f.  If noimage is nonzero,
  *  fields of records will not be imaged.
  */
 
-novalue outimage(f, dp, restrict)
+void outimage(f, dp, noimage)
 FILE *f;
 dptr dp;
-int restrict;
+int noimage;
    {
    register word i, j;
    register char *s;
@@ -629,7 +479,7 @@ int restrict;
 	    if (BlkLoc(*dp)->file.status & Fs_Window) {
 	       s = ((wbp)(BlkLoc(*dp)->file.fd))->window->windowlabel;
 	       i = strlen(s);
-	       fprintf(f, "window_%d,%d(",
+	       fprintf(f, "window_%d:%d(",
 		       ((wbp)BlkLoc(*dp)->file.fd)->window->serial,
 		       ((wbp)BlkLoc(*dp)->file.fd)->context->serial
 		       );
@@ -669,7 +519,7 @@ int restrict;
          /*
           * listimage does the work for lists.
           */
-         listimage(f, (struct b_list *)BlkLoc(*dp), restrict);
+         listimage(f, (struct b_list *)BlkLoc(*dp), noimage);
          }
 
       table: {
@@ -690,8 +540,8 @@ int restrict;
 
       record: {
          /*
-          * If restrict is nonzero, print "record(n)" where n is the
-          *  number of fields in the record.  If restrict is zero, print
+          * If noimage is nonzero, print "record(n)" where n is the
+          *  number of fields in the record.  If noimage is zero, print
           *  the image of each field instead of the number of fields.
           */
          bp = BlkLoc(*dp);
@@ -700,17 +550,17 @@ int restrict;
          fprintf(f, "record ");
          while (i-- > 0)
             printimage(f, *s++, '\0');
-        fprintf(f, "_%ld", bp->record.id);
+         fprintf(f, "_%ld", (long)bp->record.id);
          j = bp->record.recdesc->proc.nfields;
          if (j <= 0)
             fprintf(f, "()");
-         else if (restrict > 0)
+         else if (noimage > 0)
             fprintf(f, "(%ld)", (long)j);
          else {
             putc('(', f);
             i = 0;
             for (;;) {
-               outimage(f, &bp->record.fields[i], restrict+1);
+               outimage(f, &bp->record.fields[i], noimage+1);
                if (++i >= j)
                   break;
                putc(',', f);
@@ -740,7 +590,7 @@ int restrict;
             }
          else {
             dp = (dptr)((word *)dp + Offset(bp->tvsubs.ssvar));
-            outimage(f, dp, restrict);
+            outimage(f, dp, noimage);
             }
 
          if (bp->tvsubs.sslen == 1)
@@ -769,7 +619,7 @@ int restrict;
             StrLen(q) = bp->tvsubs.sslen;
             StrLoc(q) = StrLoc(*dp) + bp->tvsubs.sspos - 1;
             fprintf(f, " = ");
-            outimage(f, &q, restrict);
+            outimage(f, &q, noimage);
             }
         }
 
@@ -781,7 +631,7 @@ int restrict;
          bp = BlkLoc(*dp);
 	 tdp.dword = D_Table;
 	 BlkLoc(tdp) = bp->tvtbl.clink;
-         outimage(f, &tdp, restrict);
+         outimage(f, &tdp, noimage);
 
 #if EBCDIC != 1
          putc('[', f);
@@ -790,7 +640,7 @@ int restrict;
          putc('<', f);
 #endif					/* EBCDIC != 1 */
 
-         outimage(f, &bp->tvtbl.tref, restrict);
+         outimage(f, &bp->tvtbl.tref, noimage);
 
 #if EBCDIC != 1
          putc(']', f);
@@ -815,7 +665,7 @@ int restrict;
             fprintf(f, "&dump = ");
          else if (VarLoc(*dp) == &kywd_err)
             fprintf(f, "&error = ");
-         outimage(f, VarLoc(*dp), restrict);
+         outimage(f, VarLoc(*dp), noimage);
          }
 
       kywdevent: {
@@ -827,25 +677,25 @@ int restrict;
          else if (VarLoc(*dp) == &curpstate->eventval)
             fprintf(f, "&eventval = ");
 #endif					/* MultiThread */
-         outimage(f, VarLoc(*dp), restrict);
+         outimage(f, VarLoc(*dp), noimage);
          }
 
       kywdstr: {
-         outimage(f, VarLoc(*dp), restrict);
+         outimage(f, VarLoc(*dp), noimage);
          }
 
       kywdpos: {
          fprintf(f, "&pos = ");
-         outimage(f, VarLoc(*dp), restrict);
+         outimage(f, VarLoc(*dp), noimage);
          }
 
       kywdsubj: {
          fprintf(f, "&subject = ");
-         outimage(f, VarLoc(*dp), restrict);
+         outimage(f, VarLoc(*dp), noimage);
          }
       kywdwin: {
          fprintf(f, "&window = ");
-         outimage(f, VarLoc(*dp), restrict);
+         outimage(f, VarLoc(*dp), noimage);
          }
 
       default: { 
@@ -856,7 +706,7 @@ int restrict;
              */
             fprintf(f, "(variable = ");
             dp = (dptr)((word *)VarLoc(*dp) + Offset(*dp));
-            outimage(f, dp, restrict);
+            outimage(f, dp, noimage);
             putc(')', f);
             }
          else if (Type(*dp) == T_External)
@@ -874,7 +724,7 @@ int restrict;
  *  if c is unprintable, '\', or equal to q.
  */
 
-static novalue printimage(f, c, q)
+static void printimage(f, c, q)
 FILE *f;
 int c, q;
    {
@@ -958,10 +808,10 @@ int c, q;
  * listimage - print an image of a list.
  */
 
-static novalue listimage(f, lp, restrict)
+static void listimage(f, lp, noimage)
 FILE *f;
 struct b_list *lp;
-int restrict;
+int noimage;
    {
    register word i, j;
    register struct b_lelem *bp;
@@ -970,7 +820,7 @@ int restrict;
    bp = (struct b_lelem *) lp->listhead;
    size = lp->size;
 
-   if (restrict > 0 && size > 0) {
+   if (noimage > 0 && size > 0) {
       /*
        * Just give indication of size if the list isn't empty.
        */
@@ -1002,7 +852,7 @@ int restrict;
             j = bp->first + i - 1;
             if (j >= bp->nslots)
                j -= bp->nslots;
-            outimage(f, &bp->lslots[j], restrict+1);
+            outimage(f, &bp->lslots[j], noimage+1);
             if (count >= size)
                break;
             putc(',', f);
@@ -1208,7 +1058,7 @@ struct b_coexpr *ce;
 /*
  * dumpact - dump an activator stack
  */
-novalue dumpact(ce)
+void dumpact(ce)
 struct b_coexpr *ce;
 {
    struct astkblk *abp = ce->es_actstk;
@@ -1263,7 +1113,7 @@ word *ipc;
    extern word *records;
 #endif					/* MultiThread */
 
-   static two = 2;	/* some compilers generate bad code for division
+   static int two = 2;	/* some compilers generate bad code for division
 			   by a constant that is a power of two ... */
 
    if (!InRange(code,ipc,ecode))
@@ -1297,7 +1147,7 @@ int line;
    extern struct ipc_line *ilines, *elines;
 #endif					/* MultiThread */
 
-   static two = 2;	/* some compilers generate bad code for division
+   static int two = 2;	/* some compilers generate bad code for division
 			   by a constant that is a power of two ... */
 
    base = ilines;
@@ -1335,7 +1185,9 @@ word *ipc;
          return strcons + p->fname;
    fprintf(stderr,"bad ipc/file name table\n");
    fflush(stderr);
-   c_exit(ErrorExit);
+   c_exit(EXIT_FAILURE);
+   /*NOTREACHED*/
+   return 0;  /* avoid gcc warning */
 }
 #endif					/* !COMPILER */
 
@@ -1345,7 +1197,7 @@ word *ipc;
  *  Returns number of characters allocated.
  */
 
-doimage(c, q)
+int doimage(c, q)
 int c, q;
    {
    static char cbuf[5];
@@ -1475,7 +1327,6 @@ dptr dp1, dp2;
          }
 
       integer: {
-
 #ifdef LargeInts
          if (Type(source) == T_Lrgint) {
             word slen;
@@ -1487,7 +1338,7 @@ dptr dp1, dp2;
                + log((double)blk->digits[blk->msd]) * 0.4342944819032518 + 0.5;
 							/* 1 / ln(10) */
             if (dlen >= MaxDigits) {
-               sprintf(sbuf,"integer(~10^%ld)",dlen);
+               sprintf(sbuf, "integer(~10^%ld)", (long)dlen);
 	       len = strlen(sbuf);
                Protect(StrLoc(*dp2) = alcstr(sbuf,len), return Error);
 
@@ -1501,7 +1352,6 @@ dptr dp1, dp2;
 #else					/* LargeInts */
          cnv:string(source, *dp2);
 #endif					/* LargeInts */
-
 	 }
 
       real: {
@@ -1562,13 +1412,12 @@ dptr dp1, dp2;
              *	file(nm) where nm is the argument originally given to
              *	open.
              */
-
 #ifdef Graphics
 	    if (BlkLoc(source)->file.status & Fs_Window) {
 	       s = ((wbp)(BlkLoc(source)->file.fd))->window->windowlabel;
 	       len = strlen(s);
                Protect (reserve(Strings, (len << 2) + 16), return Error);
-	       sprintf(sbuf, "window_%d,%d(", 
+	       sprintf(sbuf, "window_%d:%d(", 
 		       ((wbp)BlkLoc(source)->file.fd)->window->serial,
 		       ((wbp)BlkLoc(source)->file.fd)->context->serial
 		       );
@@ -1578,18 +1427,15 @@ dptr dp1, dp2;
 	       }
 	    else {
 #endif					/* Graphics */
-
                s = StrLoc(BlkLoc(source)->file.fname);
                len = StrLen(BlkLoc(source)->file.fname);
                Protect (reserve(Strings, (len << 2) + 12), return Error);
 	       Protect(t = alcstr("file(", (word)(5)), return Error);
 	       StrLoc(*dp2) = t;
 	       StrLen(*dp2) = 5;
-
 #ifdef Graphics
 	     }
 #endif					/* Graphics */
-
             while (len-- > 0)
                StrLen(*dp2) += doimage(*s++, '\0');
             Protect(alcstr(")", (word)(1)), return Error);
@@ -1727,8 +1573,7 @@ dptr dp1, dp2;
 static char *csname(dp)
 dptr dp;
    {
-   register int j, n;
-   register unsigned int w;
+   register int n;
 
    n = BlkLoc(*dp)->cset.size;
    if (n < 0) 
@@ -1927,6 +1772,51 @@ word a, b;
    return a * b;
 }
 
+/* MinLong / -1 overflows; need div3 too */
+word mod3(a, b)
+word a, b;
+{
+   word retval;
+
+   switch ( b )
+   {
+      case 0:
+	 over_flow = 1; /* Not really an overflow, but definitely an error */
+	 return 0;
+
+      case MinLong:
+	 /* Handle this separately, since -MinLong can overflow */
+	 retval = ( a > MinLong ) ? a : 0;
+	 break;
+
+      default:
+	 /* First, we make b positive */
+      	 if ( b < 0 ) b = -b;	
+
+	 /* Make sure retval should have the same sign as 'a' */
+	 retval = a % b;
+	 if ( ( a < 0 ) && ( retval > 0 ) )
+	    retval -= b;
+	 break;
+      }
+
+   over_flow = 0;
+   return retval;
+}
+
+word div3(a, b)
+word a, b;
+{
+   if ( ( b == 0 ) ||	/* Not really an overflow, but definitely an error */
+        ( b == -1 && a == MinLong ) ) {
+      over_flow = 1;
+      return 0;
+      }
+
+   over_flow = 0;
+   return ( a - mod3 ( a, b ) ) / b;
+}
+
 /* MinLong / -1 overflows; need div3 too */
 
 word neg(a)
@@ -1954,7 +1844,7 @@ int sig_rsm()
 /*
  * cmd_line - convert command line arguments into a list of strings.
  */
-novalue cmd_line(argc, argv, rslt)
+void cmd_line(argc, argv, rslt)
 int argc;
 char **argv;
 dptr rslt;
@@ -2000,7 +1890,7 @@ dptr rslt;
  * varargs - construct list for use in procedures with variable length
  *  argument list.
  */
-novalue varargs(argp, nargs, rslt)
+void varargs(argp, nargs, rslt)
 dptr argp;
 int nargs;
 dptr rslt;
@@ -2040,7 +1930,7 @@ dptr rslt;
  *  string-valued variables. This is used for return, suspend, and
  *  transmitting values across co-expression context switches.
  */
-novalue retderef(valp, low, high)
+void retderef(valp, low, high)
 dptr valp;
 word *low;
 word *high;
@@ -2057,3 +1947,26 @@ word *high;
    if (InRange(low, loc, high))
       deref(valp, valp);
    }
+
+#if MSDOS
+int strcasecmp(char *s1, char *s2)
+{
+   while (*s1 && *s2) {
+      if (tolower(*s1) != tolower(*s2))
+         return tolower(*s1) - tolower(*s2);
+      s1++; s2++;
+      }
+   return tolower(*s1) - tolower(*s2);
+}
+
+int strncasecmp(char *s1, char *s2, int n)
+{
+   int i, j;
+   for(i=0;i<n;i++) {
+      j = tolower(s1[i]) - tolower(s2[i]);
+      if (j) return j;
+      if (s1[i] == '\0') return 0; /* terminate if both at end-of-string */
+      }
+   return 0;
+}
+#endif					/* MSDOS */
