@@ -9,7 +9,7 @@
  *  the bucket chain.
  */
 
-novalue addmem(ps,pe,pl)
+void addmem(ps,pe,pl)
 union block **pl;
 struct b_set *ps;
 struct b_selem *pe;
@@ -25,7 +25,7 @@ struct b_selem *pe;
  *  into an array of descriptors.
  */
 
-novalue cpslots(dp1, slotptr, i, j)
+void cpslots(dp1, slotptr, i, j)
 dptr dp1, slotptr;
 word i, j;
    {
@@ -155,7 +155,7 @@ int tcode;
    dst->set.size = src->set.size;	/* actual set size */
    dst->set.mask = src->set.mask;	/* hash mask */
    for (i = 0; i < HSegs && src->set.hdir[i] != NULL; i++)
-      memcopy((char *)dst->set.hdir[i], (char *)src->set.hdir[i],
+      memcpy((char *)dst->set.hdir[i], (char *)src->set.hdir[i],
          src->set.hdir[i]->blksize);
    /*
     * Work down the chain of element blocks in each bucket
@@ -219,7 +219,7 @@ word n;
    dst->set.size = src->set.size;	/* actual set size */
    dst->set.mask = src->set.mask;	/* hash mask */
    for (i = 0; i < HSegs && src->set.hdir[i] != NULL; i++)
-      memcopy((char *)dst->set.hdir[i], (char *)src->set.hdir[i],
+      memcpy((char *)dst->set.hdir[i], (char *)src->set.hdir[i],
          src->set.hdir[i]->blksize);
    /*
     * Work down the chain of element blocks in each bucket
@@ -479,7 +479,7 @@ union block *ep;
  * hgrow - split a hashed structure (doubling the buckets) for faster access.
  */
 
-novalue hgrow(bp)
+void hgrow(bp)
 union block *bp;
    {
    register union block **tp0, **tp1, *ep;
@@ -540,7 +540,7 @@ union block *bp;
  *  Call this only for newly created structures.  Shrinking an active structure
  *  can wreak havoc on suspended generators.
  */
-novalue hshrink(bp)
+void hshrink(bp)
 union block *bp;
    {
    register union block **tp, *ep0, *ep1;
