@@ -9,7 +9,7 @@
  *  Applies only to files with mode "s".
  */
 
-novalue flushrec(fd)
+void flushrec(fd)
 FILE *fd;
 {
 #if SASC
@@ -44,19 +44,21 @@ FILE *fd;
 #endif					/* RecordIO */
 
 /*
- * getstrg - read a line into buf from file fd.  At most maxi characters
- *  are read.  getstrg returns the length of the line, not counting
- *  the newline.  Returns -1 if EOF and -2 if length was limited by
- *  maxi. [[ Needs ferror() check. ]]
+ * getstrg - read a line into buf from file fbp.  At most maxi characters
+ *  are read.  getstrg returns the length of the line, not counting the
+ *  newline.  Returns -1 if EOF and -2 if length was limited by maxi.
+ *  Discards \r before \n in translated mode.  [[ Needs ferror() check. ]]
  */
 
-int getstrg(buf, maxi, fd)
+int getstrg(buf, maxi, fbp)
 register char *buf;
 int maxi;
-FILE *fd;
+struct b_file *fbp;
    {
    register int c, l;
+   FILE *fd;
 
+   fd = fbp->fd;
 
 #if AMIGA
 #if LATTICE
@@ -76,10 +78,19 @@ FILE *fd;
 
    l = 0;
    while (1) {
+
 #ifdef Graphics
       /* insert non-blocking read/code to service windows here */
 #endif					/* Graphics */
-      if ((c = fgetc(fd)) == '\n') break;
+
+      if ((c = fgetc(fd)) == '\n')	/* \n terminates line */
+	 break;
+      if (c == '\r' && (fbp->status & Fs_Untrans) == 0) {
+	 /* \r terminates line in translated mode */
+	 if ((c = fgetc(fd)) != '\n')	/* consume following \n */
+	     ungetc(c, fd);		/* (put back if not \n) */
+	 break;
+	 }
       if (c == EOF)
 	 if (l > 0) return l;
 	 else return -1;
@@ -97,36 +108,16 @@ FILE *fd;
  *  by hostname.  This code accommodates several different host name
  *  fetching schemes.
  */
-novalue iconhost(hostname)
+void iconhost(hostname)
 char *hostname;
    {
 
-#ifdef WhoHost
+#ifdef HostStr
    /*
-    * The host name is in /usr/include/whoami.h. (V7, 4.[01]bsd)
+    * The string constant HostStr contains the host name.
     */
-   whohost(hostname);
-#endif					/* WhoHost */
-
-#ifdef UtsName
-   {
-   /*
-    * Use the uname system call.  (System III & V)
-    */
-   struct utsname utsn;
-   uname(&utsn);
-   strcpy(hostname,utsn.nodename);
-   }
-#endif					/* UtsName */
-
-#ifdef GetHost
-   /*
-    * Use the gethostname system call.  (4.2bsd)
-    */
-   gethostname(hostname,MaxCvtLen);
-#endif					/* GetHost */
-
-#if VMS
+   strcpy(hostname,HostStr);
+#elif VMS				/* HostStr */
    /*
     * VMS has its own special logic.
     */
@@ -134,51 +125,19 @@ char *hostname;
    if (!(h = getenv("ICON_HOST")) && !(h = getenv("SYS$NODE")))
       h = "VAX/VMS";
    strcpy(hostname,h);
-#endif					/* VMS */
-
-#ifdef HostStr
+#else					/* HostStr */
+   {
    /*
-    * The string constant HostStr contains the host name.
+    * Use the uname system call.  (POSIX)
     */
-   strcpy(hostname,HostStr);
+   struct utsname utsn;
+   uname(&utsn);
+   strcpy(hostname,utsn.nodename);
+   }
 #endif					/* HostStr */
 
    }
 
-#ifdef WhoHost
-#define HdrFile "/usr/include/whoami.h"
-
-/*
- * whohost - look for a line of the form
- *  #define sysname "name"
- * in HdrFile and return the name.
- */
-novalue whohost(hostname)
-char *hostname;
-   {
-   char buf[BUFSIZ];
-   FILE *fd;
-
-   fd = fopen(HdrFile, ReadText);
-   if (fd == NULL) {
-      sprintf(buf, "Cannot open %s, no value for &host\n", HdrFile);
-      syserr(buf);
-   }
-
-   for (;;) {   /* each line in the file */
-      if (fgets(buf, sizeof buf, fd) == NULL) {
-         sprintf(buf, "No #define for sysname in %s, no value for &host\n",
-            HdrFile);
-         syserr(buf);
-      }
-      if (sscanf(buf,"#define sysname \"%[^\"]\"", hostname) == 1) {
-         fclose(fd);
-         return;
-      }
-   }
-   }
-#endif					/* WhoHost */
-
 /*
  * Read a long string in shorter parts. (Standard read may not handle long
  *  strings.)
@@ -193,18 +152,33 @@ long len;
    long tally = 0;
    long n = 0;
 
+#if NT
+   /*
+    * Under NT/MSVC++, ftell() used in Icon where() returns bad answers
+    * after a wlongread().  We work around it here by fseeking after fread.
+    */
+   long pos = ftell(fd);
+#endif					/* NT */
+
 #ifdef XWindows
    if (isatty(fileno(fd))) wflushall();
 #endif					/* XWindows */
 
    while (len > 0) {
       n = fread(ts, width, (int)((len < MaxIn) ? len : MaxIn), fd);
-      if (n <= 0)
+      if (n <= 0) {
+#if NT
+         fseek(fd, pos + tally, SEEK_SET);
+#endif					/* NT */
          return tally;
+	 }
       tally += n;
       ts += n;
       len -= n;
       }
+#if NT
+   fseek(fd, pos + tally, SEEK_SET);
+#endif					/* NT */
    return tally;
    }
 
@@ -252,7 +226,6 @@ dptr d;
    {
    register char *s;
    register word l;
-   register int  i;
 
    l = StrLen(*d);
    if (l == 0)
@@ -268,7 +241,7 @@ dptr d;
 #if BORLAND_286
       goto Lab1;
 #else
-      wputstr((wbp)ConsoleBinding, s, l);
+      { int i; for(i=0;i<l;i++) Consoleputc(s[i], f); }
 #endif
       return Succeeded;
       }
@@ -294,13 +267,16 @@ Lab1:
    /*
     * This is to get around a bug in VMS C's fwrite routine.
     */
-   for (i = 0; i < l; i++)
-      if (putc(s[i], f) == EOF)
-         break;
-   if (i == l)
-      return Succeeded;
-   else
-      return Failed;
+   {
+      int i;
+      for (i = 0; i < l; i++)
+         if (putc(s[i], f) == EOF)
+            break;
+      if (i == l)
+         return Succeeded;
+      else
+         return Failed;
+   }
 #else					/* VMS */
    if (longwrite(s,l,f) < 0)
       return Failed;
@@ -312,7 +288,7 @@ Lab1:
 /*
  * idelay(n) - delay for n milliseconds
  */
-idelay(n)
+int idelay(n)
 int n;
    {
 
@@ -339,15 +315,11 @@ int n;
 #endif                                   /* SASC */
 
 #if UNIX
-#ifndef NoSelect
    struct timeval t;
    t.tv_sec = n / 1000;
    t.tv_usec = (n % 1000) * 1000;
-   select(1, FD_NULL, FD_NULL, FD_NULL, &t);
+   select(1, NULL, NULL, NULL, &t);
    return Succeeded;
-#else					/* NoSelect */
-   return Failed;
-#endif					/* NoSelect */
 #endif					/* UNIX */
 
 #if MSDOS
@@ -358,11 +330,11 @@ int n;
 #if NT
 #ifdef MSWindows
    Sleep(n);
-   return Succeeded;
 #else					/* MSWindows */
    /* ? should be a way for NT console apps to sleep... */
    return Failed;
 #endif					/* MSWindows */
+   return Succeeded;
 #else					/* NT */
 #if BORLAND_286
    /* evil busy wait */
@@ -383,11 +355,102 @@ int n;
 #endif					/* MACINTOSH */
 
 
-#if PORT || AMIGA || ARM || ATARI_ST || MVS || VM
+#if AMIGA
+#if __SASC
+   Delay(n/20);
+   return Succeeded;
+#else					/* __SASC */
+   return Failed
+#endif                                  /* __SASC */
+#endif					/* AMIGA */
+
+#if PORT || ARM || ATARI_ST || MVS || VM
    return Failed;
-#endif
+#endif					/* PORT || ARM || ATARI_ST ... */
 
    /*
     * End of operating-system dependent code.
     */
    }
+
+#ifdef MSWindows
+#ifdef FAttrib
+/*
+ * make_mode takes mode_t type (an integer) input and returns the file permission
+ * in the format of a string.
+*/
+#if UNIX
+char *make_mode (mode_t st_mode)
+#endif					/* UNIX */
+#if MSDOS
+char *make_mode (unsigned short st_mode)
+#endif					/* MSDOS */
+{
+   char *buf;
+
+   if ( (buf = (char *) malloc(sizeof(char)*11)) == NULL ) {
+      fprintf(stderr,"fatal malloc error\n");
+      return NULL;
+   }
+
+#if UNIX
+   if ( st_mode & S_IFIFO )      buf[0] = 'f';
+   else if ( st_mode & S_IFCHR ) buf[0] = 'c';
+   else if ( st_mode & S_IFDIR ) buf[0] = 'd';
+   else if ( st_mode & S_IFBLK ) buf[0] = 'b';
+   else if ( st_mode & S_IFREG ) buf[0] = '-';
+   else			         buf[0] = '\?';
+
+   if (st_mode & S_IRUSR) buf[1] = 'r';
+   else    buf[1] = '-';
+   if (st_mode & S_IWUSR) buf[2] = 'w';
+   else    buf[2] = '-';
+   if (st_mode & S_IXUSR) buf[3] = 'x';
+   else    buf[3] = '-';
+   if (st_mode & S_IRGRP) buf[4] = 'r';
+   else    buf[4] = '-';
+   if (st_mode & S_IWGRP) buf[5] = 'w';
+   else    buf[5] = '-';
+   if (st_mode & S_IXGRP) buf[6] = 'x';
+   else    buf[6] = '-';
+   if (st_mode & S_IROTH) buf[7] = 'r';
+   else    buf[7] = '-';
+   if (st_mode & S_IWOTH) buf[8] = 'w';
+   else    buf[8] = '-';
+   if (st_mode & S_IXOTH) buf[9] = 'x';
+   else    buf[9] = '-';
+#endif					/* UNIX */
+#if MSDOS
+   if ( st_mode & _S_IFIFO )      buf[0] = 'f';
+   else if ( st_mode & _S_IFCHR ) buf[0] = 'c';
+   else if ( st_mode & _S_IFDIR ) buf[0] = 'd';
+   else if ( st_mode & _S_IFREG ) buf[0] = '-';
+   else			         buf[0] = '\?';
+
+   if (st_mode & S_IREAD) buf[1] = 'r';
+   else    buf[1] = '-';
+   if (st_mode & S_IWRITE) buf[2] = 'w';
+   else    buf[2] = '-';
+   if (st_mode & S_IEXEC) buf[3] = 'x';
+   else    buf[3] = '-';
+   if (st_mode & S_IREAD) buf[4] = 'r';
+   else    buf[4] = '-';
+   if (st_mode & S_IWRITE) buf[5] = 'w';
+   else    buf[5] = '-';
+   if (st_mode & S_IEXEC) buf[6] = 'x';
+   else    buf[6] = '-';
+   if (st_mode & S_IREAD) buf[7] = 'r';
+   else    buf[7] = '-';
+   if (st_mode & S_IWRITE) buf[8] = 'w';
+   else    buf[8] = '-';
+   if (st_mode & S_IEXEC) buf[9] = 'x';
+   else    buf[9] = '-';
+#endif					/* MSDOS */
+ 
+   buf[10] = '\0';
+
+   return buf;     
+}
+
+#endif					/* FAttrib */
+#endif					/* MSWindows */
