@@ -5,11 +5,11 @@
 
 #ifdef Graphics
 
-hidden	int	colorphrase	Params((char *buf, long *r, long *g, long *b));
-hidden	double	rgbval		Params((double n1, double n2, double hue));
+static	int	colorphrase	(char *buf, long *r, long *g, long *b);
+static	double	rgbval		(double n1, double n2, double hue);
 
-hidden	int	setpos          Params((wbp w, char *s));
-hidden	int	sicmp		Params((siptr sip1, siptr sip2));
+static	int	setpos          (wbp w, char *s);
+static	int	sicmp		(siptr sip1, siptr sip2);
 
 int canvas_serial, context_serial;
 
@@ -82,11 +82,15 @@ dptr res;
 #if UNIX || VMS || OS2_32
          idelay(XICONSLEEP);
 #endif					/* UNIX || VMS || OS2_32 */
+#ifdef MSWindows
+	 Sleep(20);
+#endif					/* MSWindows */
 	 }
       }
 
    if (wgetq(w,res) == -1)
       return -1;					/* window died */
+
    if (BlkLoc(w->window->listp)->list.size < 2)
       return -2;					/* malformed queue */
 
@@ -165,6 +169,53 @@ dptr res;
       }
    return 1;
    }
+
+#ifdef ConsoleWindow
+/*
+ *  getch() -- return char from window, without echoing
+ */
+int getch(void)
+{
+   struct descrip res;
+   if (wgetchne((wbp)OpenConsole(), &res) >= 0)
+      return *StrLoc(res) & 0xFF;
+   else
+      return -1;	/* fail */
+}
+
+/*
+ *  getch() -- return char from window, with echoing
+ */
+int getche(void)
+{
+   struct descrip res;
+   if (wgetche((wbp)OpenConsole(), &res) >= 0)
+      return *StrLoc(res) & 0xFF;
+   else
+      return -1;	/* fail */
+}
+
+/*
+ *  kbhit() -- check for availability of char from window
+ */
+int kbhit(void)
+{
+   if (ConsoleBinding) {
+      /* make sure we're up-to-date event wise */
+      pollevent();
+      /*
+       * perhaps should look in the console's icon event list for a keypress;
+       *  either a string or event > 60k; presently, succeed for all events
+       */
+      if (BlkLoc(((wbp)ConsoleBinding)->window->listp)->list.size > 0)
+         return 1;
+      else
+        return 0;  /* fail */
+      }
+   else
+      return 0;  /* fail */
+}
+#endif					/* ConsoleWindow */
 
 /*
  * Get a window that has an event pending (queued)
@@ -354,7 +405,7 @@ dptr dx;
 /*
  * Enqueue an event, encoding time interval and key state with x and y values.
  */
-novalue qevent(ws,e,x,y,t,f)
+void qevent(ws,e,x,y,t,f)
 wsp ws;		/* canvas */
 dptr e;		/* event code (descriptor pointer) */
 int x, y;	/* x and y values */
@@ -475,25 +526,46 @@ char *s;
 /*
  * put a string out to a window using the current attributes
  */
-novalue wputstr(w,s,len)
+void wputstr(w,s,len)
 wbp w;
 char *s;
 int len;
    {
-  char *s2 = s;
-  wstate *ws = w->window;
-  /* turn off the cursor */
-  hidecrsr(ws);
+   char *s2 = s;
+   wstate *ws = w->window;
+   /* turn off the cursor */
+   hidecrsr(ws);
 
-  while (len > 0) {
-    /* find a chunk of printable text */
-    while (isprint(*s2) && len > 0) { s2++; len--; }
-    /* if a chunk was parsed, write it out */
-    if (s2 != s)
-       xdis(w, s, s2 - s);
-    /* put the 'unprintable' character, if didn't just hit the end */
-    if (len-- > 0)
-       wputc(*s2++, w);
+   while (len > 0) {
+      /*
+       * find a chunk of printable text
+       */
+#ifdef MSWindows
+      while (len > 0) {
+	 if (IsDBCSLeadByte(*s2)) {
+	    s2++; s2++; len--; len--;
+	    }
+	 else if (isprint(*s2)) {
+	    s2++; len--;
+	    }
+	 else break;
+	 }
+#else					/* MSWindows */
+      while (isprint(*s2) && len > 0) {
+	 s2++; len--;
+	 }
+#endif					/* MSWindows */
+      /*
+       * if a chunk was parsed, write it out
+       */
+      if (s2 != s)
+         xdis(w, s, s2 - s);
+      /*
+       * put the 'unprintable' character, if didn't just hit the end
+       */
+      if (len-- > 0) {
+         wputc(*s2++, w);
+         }
     s = s2;
     }
 
@@ -674,7 +746,7 @@ long *r, *g, *b;
    /*
     * copy spec, lowering case and replacing spaces and hyphens with NULs
     */
-   for(p = cbuffer; c = *buf; p++, buf++) {
+   for(p = cbuffer; (c = *buf) != 0; p++, buf++) {
       if (isupper(c)) *p = tolower(c);
       else if (c == ' ' || c == '-') *p = '\0';
       else *p = c;
@@ -683,7 +755,6 @@ long *r, *g, *b;
 
    buf = cbuffer;
    ebuf = buf + len;
-
    /* check for "very" */
    if (strcmp(buf, "very") == 0) {
       very = 1;
@@ -725,15 +796,15 @@ long *r, *g, *b;
       }
 
    if (buf + strlen(buf) >= ebuf)
-      blend = 0.0;			/* only one word left */
+      blend = h1 = l1 = s1 = 0.0;	/* only one word left */
    else {
       /* we have two (or more) name words; get the first */
-      if (p = qsearch(buf, colortable[0].name,
-            ElemCount(colortable), ElemSize(colortable), strcmp)) {
+      if ((p = qsearch(buf, colortable[0].name,
+            ElemCount(colortable), ElemSize(colortable), strcmp)) != NULL) {
          blend = 0.5;
          }
-      else if (p = qsearch(buf, colortable[0].ish,
-            ElemCount(colortable), ElemSize(colortable), strcmp)) {
+      else if ((p = qsearch(buf, colortable[0].ish,
+            ElemCount(colortable), ElemSize(colortable), strcmp)) != NULL) {
          p -= sizeof(colortable[0].name);
          blend = 0.25;
          }
@@ -851,27 +922,27 @@ typedef struct lzwnode {	/* structure of LZW encoding tree node */
    unsigned short sibling;	/* next sibling */
    } lzwnode;
 
-hidden	int	gfread		Params((char *fn, int p));
-hidden	int	gfheader	Params((FILE *f));
-hidden	int	gfskip		Params((FILE *f));
-hidden	novalue	gfcontrol	Params((FILE *f));
-hidden	int	gfimhdr		Params((FILE *f));
-hidden	int	gfmap		Params((FILE *f, int p));
-hidden	int	gfsetup		Params((noargs));
-hidden	int	gfrdata		Params((FILE *f));
-hidden	int	gfrcode		Params((FILE *f));
-hidden	novalue	gfinsert	Params((int prev, int c));
-hidden	int	gffirst		Params((int c));
-hidden	novalue	gfgen		Params((int c));
-hidden	novalue	gfput		Params((int b));
+static	int	gfread		(char *fn, int p);
+static	int	gfheader	(FILE *f);
+static	int	gfskip		(FILE *f);
+static	void	gfcontrol	(FILE *f);
+static	int	gfimhdr		(FILE *f);
+static	int	gfmap		(FILE *f, int p);
+static	int	gfsetup		(void);
+static	int	gfrdata		(FILE *f);
+static	int	gfrcode		(FILE *f);
+static	void	gfinsert	(int prev, int c);
+static	int	gffirst		(int c);
+static	void	gfgen		(int c);
+static	void	gfput		(int b);
 
-hidden	int	gfwrite		Params((wbp w, char *filename,
-				   int x, int y, int width, int height));
-hidden	novalue	gfpack		Params((unsigned char *data, long len, 
-				   struct palentry *paltbl));
-hidden	novalue	gfmktree	Params((lzwnode *tree));
-hidden	novalue	gfout		Params((int tcode));
-hidden	novalue	gfdump		Params((noargs));
+static	int	gfwrite		(wbp w, char *filename,
+				   int x, int y, int width, int height);
+static	void	gfpack		(unsigned char *data, long len, 
+				   struct palentry *paltbl);
+static	void	gfmktree	(lzwnode *tree);
+static	void	gfout		(int tcode);
+static	void	gfdump		(void);
 
 static FILE *gf_f;			/* input file */
 
@@ -1030,13 +1101,13 @@ FILE *f;
 /*
  * gfcontrol(f) - process control extension subblock
  */
-static novalue gfcontrol(f)
+static void gfcontrol(f)
 FILE *f;
    {
    int i, n, c, t;
 
    n = getc(f) & 0xFF;				/* subblock length (s/b 4) */
-   for (i = 0; i < n; i++) {
+   for (i = t = 0; i < n; i++) {
       c = getc(f) & 0xFF;
       if (i == 0)
 	 t = c & 1;				/* transparency flag */
@@ -1220,7 +1291,7 @@ FILE *f;
 /*
  * gfinsert(prev, c) - insert into table
  */
-static novalue gfinsert(prev, c)
+static void gfinsert(prev, c)
 int prev, c;
    {
 
@@ -1256,7 +1327,7 @@ int c;
 /*
  * gfgen(c) - generate and output prefix
  */
-static novalue gfgen(c)
+static void gfgen(c)
 int c;
    {
    int d;
@@ -1269,7 +1340,7 @@ int c;
 /*
  * gfput(b) - add a byte to the output string
  */
-static novalue gfput(b)
+static void gfput(b)
 int b;
    {
    if (gf_nxt >= gf_lim) {		/* if current row is full */
@@ -1454,7 +1525,7 @@ int x, y, width, height;
  *
  * Sets gf_nbits and gf_cdsize based on the number of colors.
  */
-static novalue gfpack(data, len, paltbl)
+static void gfpack(data, len, paltbl)
 unsigned char *data;
 long len;
 struct palentry *paltbl;
@@ -1498,7 +1569,7 @@ struct palentry *paltbl;
  * gfmktree() - initialize or reinitialize encoding tree
  */
 
-hidden novalue gfmktree(tree)
+static void gfmktree(tree)
 lzwnode *tree;
    {
    int i;
@@ -1515,7 +1586,7 @@ lzwnode *tree;
 /*
  * gfout(code) - output one LZW token
  */
-hidden novalue gfout(tcode)
+static void gfout(tcode)
 int tcode;
    {
    gf_curr |= tcode << gf_valid;		/* add to current word */
@@ -1532,7 +1603,7 @@ int tcode;
 /*
  * gfdump() - dump output buffer
  */
-hidden novalue gfdump()
+static void gfdump()
    {
    int n;
 
@@ -1774,6 +1845,9 @@ dptr d;
    return 0;
    }
 
+
+struct palentry *palsetup_palette;	/* current palette */
+
 /*
  * palsetup(p) - set up palette for specified palette.
  */
@@ -1782,26 +1856,25 @@ int p;
    {
    int r, g, b, i, n, c;
    unsigned int rr, gg, bb;
-   unsigned char *s, *t;
+   unsigned char *s = NULL, *t;
    double m;
    struct palentry *e;
-
-   static struct palentry *palette;	/* current palette */
    static int palnumber;		/* current palette number */
 
    if (palnumber == p)
-      return palette;
-   if (palette == NULL) {
-      palette = (struct palentry *)malloc(256 * sizeof(struct palentry));
-      if (palette == NULL)
+      return palsetup_palette;
+   if (palsetup_palette == NULL) {
+      palsetup_palette =
+         (struct palentry *)malloc(256 * sizeof(struct palentry));
+      if (palsetup_palette == NULL)
          return NULL;
       }
    palnumber = p;
 
    for (i = 0; i < 256; i++)
-      palette[i].valid = palette[i].transpt = 0;
-   palette[TCH1].transpt = 1;
-   palette[TCH2].transpt = 1;
+      palsetup_palette[i].valid = palsetup_palette[i].transpt = 0;
+   palsetup_palette[TCH1].transpt = 1;
+   palsetup_palette[TCH2].transpt = 1;
    
    if (p < 0) {				/* grayscale palette */
       n = -p;
@@ -1812,27 +1885,27 @@ int p;
       m = 1.0 / (n - 1);
 
       for (i = 0; i < n; i++) {
-         e = &palette[*s++];
+         e = &palsetup_palette[*s++];
          gg = 65535 * m * i;
          e->clr.red = e->clr.green = e->clr.blue = gg;
          e->valid = 1;
 	 e->transpt = 0;
          }
-      return palette;
+      return palsetup_palette;
       }
 
    if (p == 1) {			/* special c1 palette */
       s = (unsigned char *)c1list;
       t = c1rgb;
-      while (c = *s++) {
-         e = &palette[c];
+      while ((c = *s++) != 0) {
+         e = &palsetup_palette[c];
          e->clr.red   = 65535 * (((int)*t++) / 48.0);
          e->clr.green = 65535 * (((int)*t++) / 48.0);
          e->clr.blue  = 65535 * (((int)*t++) / 48.0);
          e->valid = 1;
 	 e->transpt = 0;
          }
-      return palette;
+      return palsetup_palette;
       }
 
    switch (p) {				/* color cube plus extra grays */
@@ -1849,7 +1922,7 @@ int p;
          gg = 65535 * m * g;
          for (b = 0; b < p; b++) {
             bb = 65535 * m * b;
-            e = &palette[*s++];
+            e = &palsetup_palette[*s++];
             e->clr.red = rr;
             e->clr.green = gg;
             e->clr.blue = bb;
@@ -1862,12 +1935,12 @@ int p;
    for (g = 0; g < p * (p - 1); g++)
       if (g % p != 0) {
          gg = 65535 * m * g;
-         e = &palette[*s++];
+         e = &palsetup_palette[*s++];
          e->clr.red = e->clr.green = e->clr.blue = gg;
          e->valid = 1;
 	 e->transpt = 0;
          }
-   return palette;
+   return palsetup_palette;
    }
 
 /*
@@ -1918,6 +1991,8 @@ double r, g, b;
       return s + (int)(0.5 + (0.299 * r + 0.587 * g + 0.114 * b) * (-p - 1));
       }
 
+   /*NOTREACHED*/
+   return 0;  /* avoid gcc warning */
    }
 
 /*
@@ -2062,7 +2137,7 @@ C_integer *bits;
       hexdigits_per_row = *width / 4;
       if (*width % 4) hexdigits_per_row++;
       *nbits = len / hexdigits_per_row;
-      if (len % hexdigits_per_row) *nbits++;
+      if (len % hexdigits_per_row) (*nbits)++;
       if (*nbits > maxbits) return Failed;
       for (i = 0; i < *nbits; i++) {
          v = 0;
@@ -2120,7 +2195,7 @@ SHORT *x, *y, *width, *height;
    int retval = 0;
    if (isdigit(*buf)) {
       retval++;
-      *width = atoi(buf);
+      if ((*width = atoi(buf)) <= 0) return 0;
       while (isdigit(*++buf));
       if (*buf++ != 'x') return 0;
       if ((*height = atoi(buf)) <= 0) return 0;
@@ -2242,7 +2317,6 @@ char * abuf;
 	 break;
          }
       case A_RESIZE: {
-	 int on_off;
 	 if (strcmp(val, "on") & strcmp(val, "off"))
 	    return Failed;
          allowresize(w, ATOBOOL(val));
@@ -2451,7 +2525,7 @@ char * abuf;
 	 break;
 	 }
       case A_REVERSE: {
-	 if (strcmp(val, "on") & strcmp(val, "off"))
+	 if (strcmp(val, "on") && strcmp(val, "off"))
 	    return Failed;
 	 if ((!ATOBOOL(val) && ISREVERSE(w)) ||
 	     (ATOBOOL(val) && !ISREVERSE(w))) {
@@ -2488,6 +2562,20 @@ char * abuf;
 	 warpPointer(w, ws->pointerx, ws->pointery);
 	 break;
 	 }
+      /*
+       * remaining valid attributes are error #147
+       */
+      case A_DEPTH:
+      case A_DISPLAYHEIGHT:
+      case A_DISPLAYWIDTH:
+      case A_FHEIGHT:
+      case A_FWIDTH:
+      case A_ASCENT:
+      case A_DESCENT:
+         ReturnErrNum(147, Error);
+      /*
+       * invalid attribute
+       */
       default:
 	 ReturnErrNum(145, Error);
 	 }
@@ -2502,6 +2590,8 @@ char * abuf;
       strncpy(abuf, s, len);
       abuf[len] = '\0';
       switch (a=si_s2i(attribs, abuf)) {
+      case A_IMAGE:
+         ReturnErrNum(147, Error);
       case A_VISUAL:
 	 if (getvisual(w, abuf) == Failed) return Failed;
 	 MakeStr(abuf, strlen(abuf), answer);
@@ -2651,11 +2741,7 @@ char * abuf;
 	 break;
       case A_GEOMETRY:
 	 if (getpos(w) == Failed) return Failed;
-#if MACINTOSH
-         if (ws->theWindow)
-#else					/* MACINTOSH */
          if (ws->win)
-#endif					/* MACINTOSH */
            sprintf(abuf, "%dx%d+%d+%d",
 		   ws->width, ws->height, ws->posx, ws->posy);
          else
@@ -2892,11 +2978,11 @@ int fill;
  *  A Recursive Evaluation Algorithm for a class of Catmull-Rom Splines.
  *  Computer Graphics 22(4), 199-204.
  */
-novalue genCurve(w, p, n, helper)
+void genCurve(w, p, n, helper)
 wbp w;
 XPoint *p;
 int n;
-void (*helper)	Params((wbp, XPoint [], int));
+void (*helper)	(wbp, XPoint [], int);
    {
    int    i, j, steps;
    float  ax, ay, bx, by, stepsize, stepsize2, stepsize3;
@@ -2979,10 +3065,7 @@ void (*helper)	Params((wbp, XPoint [], int));
       }
    }
 
-static void curveHelper(w, thepoints, n)
-wbp w;
-XPoint thepoints[];
-int n;
+static void curveHelper(wbp w, XPoint *thepoints, int n)
    {
    /*
     * Could use drawpoints(w, thepoints, n)
@@ -2996,7 +3079,7 @@ int n;
 /*
  * draw a smooth curve through the array of points
  */
-novalue drawCurve(w, p, n)
+void drawCurve(w, p, n)
 wbp w;
 XPoint *p;
 int n;
